# -*- coding: utf-8 -*-
#
import numpy
from .helpers import _s3, _s21, _s111


class Dunavant(object):
    '''
    Triangle integration schemes from

    D. A. Dunavant,
    High Degree Efficient Symmetrical Gaussian Quadrature Rules for the
    Triangle,
    Article in International Journal for Numerical Methods in Engineering,
    21(6):1129-1148, June 1985,
    10.1002/nme.1620210612,
    <https://dx.doi.org/10.1002/nme.1620210612>.
    '''
    def __init__(self, index):
        self.name = 'Dunavant(%d)' % index
        if index == 1:
            self.weights = numpy.array([1.0])
            bary = _s3()
            self.degree = 1
        elif index == 2:
            self.weights = 1.0/3.0 * numpy.ones(3)
            bary = _s21(1.0/6.0)
            self.degree = 2
        elif index == 3:
            self.weights = numpy.concatenate([
                -0.5625 * numpy.ones(1),
                25.0 / 48.0 * numpy.ones(3),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.2),
                ])
            self.degree = 3
        elif index == 4:
            self.weights = numpy.concatenate([
                0.223381589678011 * numpy.ones(3),
                0.109951743655322 * numpy.ones(3),
                ])
            bary = numpy.concatenate([
                _s21(0.445948490915965),
                _s21(0.091576213509771),
                ])
            self.degree = 4
        elif index == 5:
            self.weights = numpy.concatenate([
                0.225 * numpy.ones(1),
                0.132394152788506 * numpy.ones(3),
                0.125939180544827 * numpy.ones(3),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.4701420641051),
                _s21(0.101286507323456),
                ])
            self.degree = 5
        elif index == 6:
            self.weights = numpy.concatenate([
                0.116786275726379 * numpy.ones(3),
                0.050844906370207 * numpy.ones(3),
                0.082851075618374 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s21(0.249286745170910),
                _s21(0.063089014491502),
                _s111(0.053145049844817, 0.310352451033784),
                ])
            self.degree = 6
        elif index == 7:
            self.weights = numpy.concatenate([
                -0.149570044467682 * numpy.ones(1),
                0.175615257433208 * numpy.ones(3),
                0.053347235608838 * numpy.ones(3),
                0.077113760890257 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.260345966079040),
                _s21(0.065130102902216),
                _s111(0.048690315425316, 0.312865496004874),
                ])
            self.degree = 7
        elif index == 8:
            self.weights = numpy.concatenate([
                0.144315607677787 * numpy.ones(1),
                0.095091634267285 * numpy.ones(3),
                0.103217370534718 * numpy.ones(3),
                0.032458497623198 * numpy.ones(3),
                0.027230314174435 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.459292588292723),
                _s21(0.170569307751760),
                _s21(0.050547228317031),
                _s111(0.008394777409958, 0.263112829634638),
                ])
            self.degree = 8
        elif index == 9:
            self.weights = numpy.concatenate([
                0.097135796282799 * numpy.ones(1),
                0.031334700227139 * numpy.ones(3),
                0.077827541004774 * numpy.ones(3),
                0.079647738927210 * numpy.ones(3),
                0.025577675658698 * numpy.ones(3),
                0.043283539377289 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.489682519198738),
                _s21(0.437089591492937),
                _s21(0.188203535619033),
                _s21(0.044729513394453),
                _s111(0.036838412054736, 0.221962989160766),
                ])
            self.degree = 9
        elif index == 10:
            self.weights = numpy.concatenate([
                0.090817990382754 * numpy.ones(1),
                0.036725957756467 * numpy.ones(3),
                0.045321059435528 * numpy.ones(3),
                0.072757916845420 * numpy.ones(6),
                0.028327242531057 * numpy.ones(6),
                0.009421666963733 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.485577633383657),
                _s21(0.109481575485037),
                _s111(0.141707219414880, 0.307939838764121),
                _s111(0.025003534762686, 0.246672560639903),
                _s111(0.009540815400299, 0.066803251012200),
                ])
            self.degree = 10
        elif index == 11:
            self.weights = numpy.concatenate([
                0.000927006328961 * numpy.ones(3),
                0.077149534914813 * numpy.ones(3),
                0.059322977380774 * numpy.ones(3),
                0.036184540503418 * numpy.ones(3),
                0.013659731002678 * numpy.ones(3),
                0.052337111962204 * numpy.ones(6),
                0.020707659639141 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s21(0.534611048270758),
                _s21(0.398969302965855),
                _s21(0.203309900431282),
                _s21(0.119350912282581),
                _s21(0.032364948111276),
                _s111(0.050178138310495, 0.356620648261293),
                _s111(0.021022016536166, 0.171488980304042),
                ])
            self.degree = 11
        elif index == 12:
            self.weights = numpy.concatenate([
                0.025731066440455 * numpy.ones(3),
                0.043692544538038 * numpy.ones(3),
                0.062858224217885 * numpy.ones(3),
                0.034796112930709 * numpy.ones(3),
                0.006166261051559 * numpy.ones(3),
                0.040371557766381 * numpy.ones(6),
                0.022356773202303 * numpy.ones(6),
                0.017316231108659 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s21(0.488217389773805),
                _s21(0.439724392294460),
                _s21(0.271210385012116),
                _s21(0.127576145541586),
                _s21(0.021317350453210),
                _s111(0.115343494534698, 0.275713269685514),
                _s111(0.022838332222257, 0.281325580989940),
                _s111(0.025734050548330, 0.116251915907597),
                ])
            self.degree = 12
        elif index == 13:
            self.weights = numpy.concatenate([
                0.052520923400802 * numpy.ones(1),
                0.011280145209330 * numpy.ones(3),
                0.031423518362454 * numpy.ones(3),
                0.047072502504194 * numpy.ones(3),
                0.047363586536355 * numpy.ones(3),
                0.031167529045794 * numpy.ones(3),
                0.007975771465074 * numpy.ones(3),
                0.036848402728732 * numpy.ones(6),
                0.017401463303822 * numpy.ones(6),
                0.015521786839045 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.495048184939705),
                _s21(0.468716635109574),
                _s21(0.414521336801277),
                _s21(0.229399572042831),
                _s21(0.114424495196330),
                _s21(0.024811391363459),
                _s111(0.094853828379579, 0.268794997058761),
                _s111(0.018100773278807, 0.291730066734288),
                _s111(0.022233076674090, 0.126357385491669),
                ])
            self.degree = 13
        elif index == 14:
            self.weights = numpy.concatenate([
                0.021883581369429 * numpy.ones(3),
                0.032788353544125 * numpy.ones(3),
                0.051774104507292 * numpy.ones(3),
                0.042162588736993 * numpy.ones(3),
                0.014433699669777 * numpy.ones(3),
                0.004923403602400 * numpy.ones(3),
                0.024665753212564 * numpy.ones(6),
                0.038571510787061 * numpy.ones(6),
                0.014436308113534 * numpy.ones(6),
                0.005010228838501 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s21(0.488963910362179),
                _s21(0.417644719340454),
                _s21(0.273477528308839),
                _s21(0.177205532412543),
                _s21(0.061799883090873),
                _s21(0.019390961248701),
                _s111(0.057124757403648, 0.172266687821356),
                _s111(0.092916249356972, 0.336861459796345),
                _s111(0.014646950055654, 0.298372882136258),
                _s111(0.001268330932872, 0.118974497696957),
                ])
            self.degree = 14
        elif index == 15:
            self.weights = numpy.concatenate([
                0.001916875642849 * numpy.ones(3),
                0.044249027271145 * numpy.ones(3),
                0.051186548718852 * numpy.ones(3),
                0.023687735870688 * numpy.ones(3),
                0.013289775690021 * numpy.ones(3),
                0.004748916608192 * numpy.ones(3),
                0.038550072599593 * numpy.ones(6),
                0.027215814320624 * numpy.ones(6),
                0.002182077366797 * numpy.ones(6),
                0.021505319847731 * numpy.ones(6),
                0.007673942631049 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s21(0.506972916858243),
                _s21(0.431406354283023),
                _s21(0.277693644847144),
                _s21(0.126464891041254),
                _s21(0.070808385974686),
                _s21(0.018965170241073),
                _s111(0.133734161966621, 0.261311371140087),
                _s111(0.036366677396917, 0.575586555512814),
                _s111(-0.010174883126571, 0.285712220049916),
                _s111(0.036843869875878, 0.215599664072284),
                _s111(0.012459809331199, 0.103575616576386),
                ])
            self.degree = 15
        elif index == 16:
            self.weights = numpy.concatenate([
                0.046875697427642 * numpy.ones(1),
                0.006405878578585 * numpy.ones(3),
                0.041710296739387 * numpy.ones(3),
                0.026891484250064 * numpy.ones(3),
                0.042132522761650 * numpy.ones(3),
                0.030000266842773 * numpy.ones(3),
                0.014200098925024 * numpy.ones(3),
                0.003582462351273 * numpy.ones(3),
                0.032773147460627 * numpy.ones(6),
                0.015298306248441 * numpy.ones(6),
                0.002386244192839 * numpy.ones(6),
                0.019084792755899 * numpy.ones(6),
                0.006850054546542 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.497380541948438),
                _s21(0.413469438549352),
                _s21(0.470458599066991),
                _s21(0.240553749969521),
                _s21(0.147965794222573),
                _s21(0.075465187657474),
                _s21(0.016596402623025),
                _s111(0.103575692245252, 0.296555596579887),
                _s111(0.020083411655416, 0.337723063403079),
                _s111(-0.004341002614139, 0.204748281642812),
                _s111(0.041941786468010, 0.189358492130623),
                _s111(0.014317320230681, 0.085283615682657),
                ])
            self.degree = 16
        elif index == 17:
            self.weights = numpy.concatenate([
                0.033437199290803 * numpy.ones(1),
                0.005093415440507 * numpy.ones(3),
                0.014670864527638 * numpy.ones(3),
                0.024350878353672 * numpy.ones(3),
                0.031107550868969 * numpy.ones(3),
                0.031257111218620 * numpy.ones(3),
                0.024815654339665 * numpy.ones(3),
                0.014056073070557 * numpy.ones(3),
                0.003194676173779 * numpy.ones(3),
                0.008119655318993 * numpy.ones(6),
                0.026805742283163 * numpy.ones(6),
                0.018459993210822 * numpy.ones(6),
                0.008476868534328 * numpy.ones(6),
                0.018292796770025 * numpy.ones(6),
                0.006665632004165 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.497170540556774),
                _s21(0.482176322624625),
                _s21(0.450239969020782),
                _s21(0.400266239377397),
                _s21(0.252141267970953),
                _s21(0.162047004658461),
                _s21(0.075875882260746),
                _s21(0.015654726967822),
                _s111(0.334319867363658, 0.655493203809423),
                _s111(0.292221537796944, 0.572337590532020),
                _s111(0.319574885423190, 0.626001190286228),
                _s111(0.190704224192292, 0.796427214974071),
                _s111(0.180483211648746, 0.752351005937729),
                _s111(0.080711313679564, 0.904625504095608),
                ])
            self.degree = 17
        elif index == 18:
            self.weights = numpy.concatenate([
                0.030809939937647 * numpy.ones(1),
                0.009072436679404 * numpy.ones(3),
                0.018761316939594 * numpy.ones(3),
                0.019441097985477 * numpy.ones(3),
                0.027753948610810 * numpy.ones(3),
                0.032256225351457 * numpy.ones(3),
                0.025074032616922 * numpy.ones(3),
                0.015271927971832 * numpy.ones(3),
                0.006793922022963 * numpy.ones(3),
                -0.002223098729920 * numpy.ones(3),
                0.006331914076406 * numpy.ones(6),
                0.027257538049138 * numpy.ones(6),
                0.017676785649465 * numpy.ones(6),
                0.018379484638070 * numpy.ones(6),
                0.008104732808192 * numpy.ones(6),
                0.007634129070725 * numpy.ones(6),
                0.000046187660794 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.493344808630921),
                _s21(0.469210594241957),
                _s21(0.436281395887006),
                _s21(0.394846170673416),
                _s21(0.249794568803157),
                _s21(0.161432193743843),
                _s21(0.076598227485371),
                _s21(0.024252439353450),
                _s21(0.043146367216965),
                _s111(0.358911494940944, 0.632657968856636),
                _s111(0.294402476751957, 0.574410971510855),
                _s111(0.325017801641814, 0.624779046792512),
                _s111(0.184737559666046, 0.748933176523037),
                _s111(0.218796800013321, 0.769207005420443),
                _s111(0.101179597136408, 0.883962302273467),
                _s111(0.020874755282586, 1.014347260005363),
                ])

            self.degree = 18
        elif index == 19:
            self.weights = numpy.concatenate([
                0.032906331388919 * numpy.ones(1),
                #
                0.010330731891272 * numpy.ones(3),
                0.022387247263016 * numpy.ones(3),
                0.030266125869468 * numpy.ones(3),
                0.030490967802198 * numpy.ones(3),
                0.024159212741641 * numpy.ones(3),
                0.016050803586801 * numpy.ones(3),
                0.008084580261784 * numpy.ones(3),
                0.002079362027485 * numpy.ones(3),
                #
                0.003884876904981 * numpy.ones(6),
                0.025574160612022 * numpy.ones(6),
                0.008880903573338 * numpy.ones(6),
                0.016124546761731 * numpy.ones(6),
                0.002491941817491 * numpy.ones(6),
                0.018242840118951 * numpy.ones(6),
                0.010258563736199 * numpy.ones(6),
                0.003799928855302 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.489609987073006),
                _s21(0.454536892697893),
                _s21(0.401416680649431),
                _s21(0.255551654403098),
                _s21(0.177077942152130),
                _s21(0.110061053227952),
                _s21(0.055528624251840),
                _s21(0.012621863777229),
                _s111(0.395754787356943, 0.600633794794645),
                _s111(0.307929983880436, 0.557603261588784),
                _s111(0.264566948406520, 0.720987025817365),
                _s111(0.358539352205951, 0.594527068955871),
                _s111(0.157807405968595, 0.839331473680839),
                _s111(0.075050596975911, 0.701087978926173),
                _s111(0.142421601113383, 0.822931324069857),
                _s111(0.065494628082938, 0.924344252620784),
                ])
            self.degree = 19
        else:
            assert index == 20
            self.weights = numpy.concatenate([
                0.033057055541624 * numpy.ones(1),
                #
                0.000867019185663 * numpy.ones(3),
                0.011660052716448 * numpy.ones(3),
                0.022876936356421 * numpy.ones(3),
                0.030448982673938 * numpy.ones(3),
                0.030624891725355 * numpy.ones(3),
                0.024368057676800 * numpy.ones(3),
                0.015997432032024 * numpy.ones(3),
                0.007698301815602 * numpy.ones(3),
                -0.000632060497488 * numpy.ones(3),
                0.001751134301193 * numpy.ones(3),
                #
                0.016465839189576 * numpy.ones(6),
                0.004839033540485 * numpy.ones(6),
                0.025804906534650 * numpy.ones(6),
                0.008471091054441 * numpy.ones(6),
                0.018354914106280 * numpy.ones(6),
                0.000704404677908 * numpy.ones(6),
                0.010112684927462 * numpy.ones(6),
                0.003573909385950 * numpy.ones(6),
                ])
            bary = numpy.concatenate([
                _s3(),
                _s21(0.500950464352200),
                _s21(0.488212957934729),
                _s21(0.455136681950283),
                _s21(0.401996259318289),
                _s21(0.255892909759421),
                _s21(0.176488255995106),
                _s21(0.104170855336758),
                _s21(0.053068963840930),
                _s21(0.041618715196029),
                _s21(0.011581921406822),
                _s111(0.344855770229001, 0.606402646106160),
                _s111(0.377843269594854, 0.615842614456541),
                _s111(0.306635479062357, 0.559048000390295),
                _s111(0.249419362774742, 0.736606743262866),
                _s111(0.212775724802802, 0.711675142287434),
                _s111(0.146965436053239, 0.861402717154987),
                _s111(0.137726978828923, 0.835586957912363),
                _s111(0.059696109149007, 0.929756171556853),
                ])
            self.degree = 20

        # convert barycentric coordinates to reference triangle
        self.points = bary[:, [1, 2]]
        return
