# -*- coding: utf-8 -*-
#
import numpy

from . import helpers


def area(radius):
    return 4*numpy.pi * numpy.array(radius)**2


def show(scheme):
    from matplotlib import pyplot as plt
    # pylint: disable=relative-import, unused-variable
    from mpl_toolkits.mplot3d import Axes3D

    fig = plt.figure()
    ax = fig.gca(projection='3d')
    ax.set_aspect('equal')

    # http://matplotlib.org/examples/mplot3d/surface3d_demo2.html
    u = numpy.linspace(0, 2 * numpy.pi, 100)
    v = numpy.linspace(0, numpy.pi, 100)
    x = numpy.outer(numpy.cos(u), numpy.sin(v))
    y = numpy.outer(numpy.sin(u), numpy.sin(v))
    z = numpy.outer(numpy.ones(numpy.size(u)), numpy.cos(v))

    ax.plot_surface(
            x, y, z,
            rstride=3, cstride=3,
            color='0.9',
            alpha=1.0,
            linewidth=0
            )

    ax.scatter(
        1.05 * scheme.points[:, 0],
        1.05 * scheme.points[:, 1],
        1.05 * scheme.points[:, 2],
        color='#1f77b4',
        s=60
        )

    ax.set_axis_off()
    plt.show()
    return


def integrate(f, center, radius, rule, sumfun=helpers.kahan_sum):
    center = numpy.array(center)
    rr = numpy.multiply.outer(radius, rule.points)
    rr = numpy.swapaxes(rr, 0, -2)
    ff = numpy.array(f((rr + center).T))
    out = sumfun(rule.weights * ff, axis=-1)
    return area(radius) * out


class Lebedev(object):
    '''
    Sphere integration schemes from

    Lebedev, V. I. (1976),
    Quadratures on a sphere,
    Zh. Vȳchisl. Mat. Mat. Fiz. 16 (2): 293–306.
    doi:10.1016/0041-5553(76)90100-2.

    <https://en.wikipedia.org/wiki/Lebedev_quadrature>
    <https://people.sc.fsu.edu/~jburkardt/datasets/sphere_lebedev_rule/sphere_lebedev_rule.html>
    '''
    def __init__(self, index):
        if index == 1:
            self.weights = numpy.array(
                6 * [1.0/6.0]
                )
            self.points = numpy.concatenate([
                _a1(),
                ])
            self.degree = 3
        elif index == 2:
            self.weights = numpy.array(
                6 * [1.0/15.0] +
                8 * [0.075]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                ])
            self.degree = 5
        elif index == 3:
            self.weights = numpy.array(
                6 * [1.0/21.0] +
                12 * [4.0/105.0] +
                8 * [9.0/280.0]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                ])
            self.degree = 7
        elif index == 4:
            self.weights = numpy.array(
                6 * [1.0/105.0] +
                8 * [9.0/280.0] +
                24 * [1.0/35.0]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(8.8807383397711525674e-01, 4.5970084338098304855e-01)
                ])
            self.degree = 9
        elif index == 5:
            self.weights = numpy.array(
                6 * [1.2698412698413000e-02] +
                12 * [2.2574955908289000e-02] +
                8 * [2.1093750000000001e-02] +
                24 * [2.0173335537919002e-02]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _llm(3.0151134457776352e-01, 9.0453403373329089e-01),
                ])
            self.degree = 11
        elif index == 6:
            self.weights = numpy.array(
                6 * [5.1306717973400001e-04] +
                12 * [1.6604069565742001e-02] +
                8 * [-2.9586038961039000e-02] +
                24 * [1.6522170993716001e-02] +
                24 * [2.6576207082158999e-02]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(3.2077264898077640e-01, 9.4715622136258792e-01),
                _llm(4.8038446141526142e-01, 7.3379938570534264e-01),
                ])
            self.degree = 13
        elif index == 7:
            self.weights = numpy.array(
                6 * [1.1544011544012000e-02] +
                8 * [1.1943909085856000e-02] +
                24 * [1.1812303746904000e-02] +
                24 * [1.1110555710602999e-02] +
                24 * [1.1876501294537000e-02]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(3.7424303909034118e-01, 9.2733065715117247e-01),
                _llm(3.6960284645415015e-01, 8.5251831170126768e-01),
                _llm(6.9435400660266633e-01, 1.8906355288539489e-01),
                ])
            self.degree = 15
        elif index == 8:
            self.weights = numpy.array(
                6 * [3.8282704949370002e-03] +
                8 * [9.7937375124880002e-03] +
                24 * [9.6949963616630008e-03] +
                24 * [8.2117372831910004e-03] +
                24 * [9.9428148911780007e-03] +
                24 * [9.5954713360710004e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(4.7836902881215021e-01, 8.7815891060406615e-01),
                _llm(1.8511563534473630e-01, 9.6512403508659406e-01),
                _llm(6.9042104838229212e-01, 2.1595729184584847e-01),
                _llm(3.9568947305594182e-01, 8.2876998125259227e-01),
                ])
            self.degree = 17
        elif index == 9:
            self.weights = numpy.array(
                6 * [5.9963136886199996e-04] +
                12 * [7.3729997186210003e-03] +
                8 * [7.2105153601440004e-03] +
                24 * [7.1163554931180000e-03] +
                24 * [6.7538294863139997e-03] +
                24 * [7.5743941590539999e-03] +
                48 * [6.9910873533029997e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _llm(6.7644104001142624e-01, 2.9129888220952688e-01),
                _llm(4.1749612279654530e-01, 8.0708981835958249e-01),
                _llm(1.5746766720390826e-01, 9.7488864367717321e-01),
                _rsw(1.4035538117131835e-01, 4.4933283232695570e-01, 8.8227001126032267e-01),
                ])
            self.degree = 19
        elif index == 10:
            self.weights = numpy.array(
                6 * [5.5448429020370001e-03] +
                12 * [6.0713327706709997e-03] +
                8 * [6.3836747735150001e-03] +
                24 * [5.4771433851369998e-03] +
                24 * [5.1833875877479998e-03] +
                24 * [6.3179290098140002e-03] +
                24 * [6.2016700065889996e-03] +
                48 * [5.9683839876810002e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(2.6139313603359887e-01, 9.6523242197644832e-01),
                _llm(2.5512526211141340e-01, 9.3264259031269059e-01),
                _llm(6.7436014603627648e-01, 3.0079359513770149e-01),
                _llm(4.3189106967194102e-01, 7.9179555939349211e-01),
                _rsw(4.9904531617960368e-01, 1.4466307443251145e-01, 8.5441580468465883e-01),
                ])
            self.degree = 21
        elif index == 11:
            self.weights = numpy.array(
                6 * [1.7823404472450000e-03] +
                12 * [5.7169059499770003e-03] +
                8 * [5.5733831788490002e-03] +
                24 * [5.0518460646149996e-03] +
                24 * [5.6087040825879998e-03] +
                24 * [5.1582377118049999e-03] +
                24 * [5.5187714672740003e-03] +
                24 * [4.1067770281689999e-03] +
                48 * [5.5302489162329998e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(3.4577021976112848e-01, 9.3831921813759145e-01),
                _llm(6.7129734426952248e-01, 3.1419699418258634e-01),
                _llm(2.8924656275754396e-01, 9.1250909686747361e-01),
                _llm(4.4469331787174360e-01, 7.7749321931476723e-01),
                _llm(1.2993354476500654e-01, 9.8297230270725333e-01),
                _rsw(1.5904171053835303e-01, 8.3603601548245887e-01, 5.2511857244364202e-01),
                ])
            self.degree = 23
        elif index == 12:
            self.weights = numpy.array(
                6 * [-5.5226399197272999e-02] +
                8 * [4.4502746074450003e-03] +
                24 * [4.2310830953570001e-03] +
                24 * [5.1980698640639996e-03] +
                24 * [4.4968410679210001e-03] +
                24 * [5.0491534504790003e-03] +
                24 * [3.9764080180519999e-03] +
                24 * [4.4014006503809997e-03] +
                24 * [1.7245443505443998e-02] +
                48 * [4.6957209725689997e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(5.8238423097155845e-01, 8.1291365317336528e-01),
                _pq0(3.5458773905186880e-01, 9.3502274588059286e-01),
                _llm(4.4920446873976111e-01, 7.7228925314836394e-01),
                _llm(2.5204194902102001e-01, 9.3431777884581169e-01),
                _llm(6.9819066584472411e-01, 1.5830220546347812e-01),
                _llm(6.5874052434609598e-01, 3.6348568495672684e-01),
                _llm(4.0385440500976755e-02, 9.9836768396772746e-01),
                _rsw(2.2721818089981877e-01, 4.8646615358866480e-01, 8.4363652106889431e-01),
                ])
            self.degree = 25
        elif index == 13:
            self.weights = numpy.array(
                6 * [-1.3137691273270001e-03] +
                12 * [-2.5227287048589998e-03] +
                8 * [4.1868538817010003e-03] +
                24 * [4.2295827006469996e-03] +
                24 * [5.3151679778109997e-03] +
                24 * [4.0471423770859997e-03] +
                24 * [4.1124823944069999e-03] +
                24 * [3.5955848997590001e-03] +
                24 * [4.2561313514280002e-03] +
                48 * [4.0809142257810004e-03] +
                48 * [4.0714675938309996e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(8.5065080835203999e-01, 5.2573111211913359e-01),
                _llm(7.0393733915854739e-01, 9.4575076403712766e-02),
                _llm(1.0125262485724165e-01, 9.8969480746290539e-01),
                _llm(4.6474487264205383e-01, 7.5367393925081572e-01),
                _llm(3.2774206549716295e-01, 8.8609834499749895e-01),
                _llm(6.6203386636999739e-01, 3.5131512856463332e-01),
                _rsw(3.2334845426928976e-01, 1.1531120110097010e-01, 9.3922792974991576e-01),
                _rsw(2.3147901587126027e-01, 5.2449392409223650e-01, 8.1934338881912028e-01),
                ])
            self.degree = 27
        elif index == 14:
            self.weights = numpy.array(
                6 * [8.5459117251300001e-04] +
                8 * [3.5991192850260002e-03] +
                24 * [2.9823449631720001e-03] +
                24 * [3.6008209322159999e-03] +
                24 * [3.4497884243060001e-03] +
                24 * [3.6048226014199999e-03] +
                24 * [3.5767296617430001e-03] +
                24 * [2.3521014136889998e-03] +
                24 * [3.1089531224140002e-03] +
                24 * [3.6500458076770001e-03] +
                48 * [3.5715405542730001e-03] +
                48 * [3.3923122050060002e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(2.6441528870606645e-01, 9.6440891487920599e-01),
                _pq0(5.7189558918789596e-01, 8.2032641982775933e-01),
                _llm(3.5156403455701052e-01, 8.6764362454408328e-01),
                _llm(6.5663294102196113e-01, 3.7103417838482106e-01),
                _llm(4.7290541325810043e-01, 7.4345204298755574e-01),
                _llm(9.6183085226147977e-02, 9.9070562137940810e-01),
                _llm(2.2196452362941768e-01, 9.4945431722644313e-01),
                _llm(7.0117664160895443e-01, 1.2923867271051456e-01),
                _rsw(2.5100347517704641e-01, 8.0007274940739515e-01, 5.4486773725807747e-01),
                _rsw(1.2335485325833283e-01, 4.1277240831685313e-01, 9.0244252953300041e-01),
                ])
            self.degree = 29
        elif index == 15:
            self.weights = numpy.array(
                6 * [3.0067967494540001e-03] +
                8 * [3.0506277456509999e-03] +
                24 * [3.0079495552189999e-03] +
                24 * [2.8819646030549999e-03] +
                24 * [1.6211046002890001e-03] +
                24 * [3.0057014849020000e-03] +
                24 * [2.9909925296540000e-03] +
                24 * [2.9821706441080001e-03] +
                24 * [2.7215642373109999e-03] +
                24 * [3.0335137958109998e-03] +
                48 * [2.9583576265360002e-03] +
                48 * [3.0360200264069998e-03] +
                48 * [2.8321874039260002e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(1.9329450132303405e-01, 9.8114078284325723e-01),
                _pq0(3.8004949198993027e-01, 9.2496615269867899e-01),
                _llm(7.0689654639123156e-01, 2.4383301669355723e-02),
                _llm(4.7946826257120251e-01, 7.3499685058774555e-01),
                _llm(1.9275331548780175e-01, 9.6212905513601443e-01),
                _llm(6.9303579613271227e-01, 1.9850131122336515e-01),
                _llm(3.6083021155200906e-01, 8.6000181212754701e-01),
                _llm(6.4984861614961675e-01, 3.9419988860583893e-01),
                _rsw(2.8995588254995741e-01, 7.9345378565823144e-01, 5.3512304771827623e-01),
                _rsw(9.6841214551039542e-02, 8.2808015066868612e-01, 5.5218207434939925e-01),
                _rsw(1.8334346470416593e-01, 9.0746582653051255e-01, 3.7800918987448684e-01),
                ])
            self.degree = 31
        elif index == 16:
            self.weights = numpy.array(
                6 * [5.2658979682200003e-04] +
                12 * [2.5482199720029998e-03] +
                8 * [2.5123174189269999e-03] +
                24 * [2.4174423756390001e-03] +
                24 * [1.9109512821800000e-03] +
                24 * [2.5304038011860000e-03] +
                24 * [2.0142790209189999e-03] +
                24 * [2.5017251684030001e-03] +
                24 * [2.5132671745980001e-03] +
                24 * [2.3026947822269998e-03] +
                24 * [1.4624956215949999e-03] +
                24 * [2.4453734373129999e-03] +
                48 * [2.4169300443250001e-03] +
                48 * [2.5122368545629999e-03] +
                48 * [2.4966440545529998e-03] +
                48 * [2.2366077604380002e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(8.8181328777942880e-01, 4.7159869115131597e-01),
                _pq0(9.7764281111826490e-01, 2.1027252285730683e-01),
                _llm(6.9093463075091111e-01, 2.1264682470755175e-01),
                _llm(1.7748360546091577e-01, 9.6798715879147279e-01),
                _llm(4.9143426377847460e-01, 7.1901650104084347e-01),
                _llm(6.4566647074242556e-01, 4.0771266489776958e-01),
                _llm(2.8612890103076377e-01, 9.1447280112087248e-01),
                _llm(7.5680843671780448e-02, 9.9425591263127777e-01),
                _llm(3.9272597633680018e-01, 8.3158440041923232e-01),
                _rsw(2.0548236964030436e-01, 8.6894603228724110e-01, 4.5023303825826255e-01),
                _rsw(5.9051570489252714e-01, 7.9992785438572855e-01, 1.0680182607580473e-01),
                _rsw(5.5501523610768067e-01, 7.7174626269159008e-01, 3.1042840351665429e-01),
                _rsw(9.3718098585537213e-01, 3.3443631453434558e-01, 9.9217696364292271e-02),
                ])
            self.degree = 35
        elif index == 17:
            self.weights = numpy.array(
                6 * [3.0951212953099998e-04] +
                8 * [1.8523796985970000e-03] +
                24 * [1.8571611967740001e-03] +
                24 * [1.7051539963960000e-03] +
                24 * [1.3003216858860000e-03] +
                24 * [1.8717906392780000e-03] +
                24 * [1.8588125854380000e-03] +
                24 * [1.8520288282960001e-03] +
                24 * [1.8467159561510000e-03] +
                24 * [1.8184717781630001e-03] +
                24 * [1.7495646572809999e-03] +
                24 * [1.6172106472540000e-03] +
                24 * [1.3847372348520001e-03] +
                24 * [9.7643311650499995e-04] +
                48 * [1.8428664729050000e-03] +
                48 * [1.8026589343770000e-03] +
                48 * [1.8498305604440000e-03] +
                48 * [1.7139045071070001e-03] +
                48 * [1.5552136033970001e-03] +
                48 * [1.8022391280090001e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(6.1168434420098750e-01, 7.9110192962690196e-01),
                _pq0(3.9647553481998582e-01, 9.1804528771145388e-01),
                _pq0(1.7247820099077241e-01, 9.8501333502800192e-01),
                _llm(7.0409549382274683e-01, 9.2190407076898337e-02),
                _llm(6.8077440664552424e-01, 2.7035608835916486e-01),
                _llm(6.3725469392587519e-01, 4.3337386877715434e-01),
                _llm(5.0444197078003583e-01, 7.0076857537357284e-01),
                _llm(4.2157617840109668e-01, 8.0283687733527376e-01),
                _llm(3.3179207364721225e-01, 8.8307872793413256e-01),
                _llm(2.3847367014218870e-01, 9.4141415822040253e-01),
                _llm(1.4590364491577620e-01, 9.7848058376269387e-01),
                _llm(6.0950341155071593e-02, 9.9627812975401642e-01),
                _rsw(5.6102638086220602e-01, 3.5182809277335186e-01, 7.4931061190411596e-01),
                _rsw(4.7423928425519801e-01, 2.6347166559379498e-01, 8.4004748835905041e-01),
                _rsw(5.9841264978853792e-01, 1.8166408403602102e-01, 7.8032074247992034e-01),
                _rsw(3.7910354076955632e-01, 1.7207952256568781e-01, 9.0921347509237360e-01),
                _rsw(2.7786731905862455e-01, 8.2130215819325197e-02, 9.5710207431007255e-01),
                _rsw(5.0335642710751172e-01, 8.9992058420748922e-02, 8.5937985589072119e-01),
                ])
            self.degree = 41
        elif index == 18:
            self.weights = numpy.array(
                6 * [2.1929420881800001e-04] +
                12 * [1.4364336173189999e-03] +
                8 * [1.4219403443360000e-03] +
                24 * [9.2544014998699995e-04] +
                24 * [1.2502399950539999e-03] +
                24 * [1.3943658433290000e-03] +
                24 * [6.7981235110500002e-04] +
                24 * [9.9131842352900000e-04] +
                24 * [1.1802078332390001e-03] +
                24 * [1.2965996020809999e-03] +
                24 * [1.3658714274280000e-03] +
                24 * [1.4029886047750000e-03] +
                24 * [1.4186455635960000e-03] +
                24 * [1.4213767418520000e-03] +
                24 * [1.4239964754909999e-03] +
                24 * [1.4315540421789999e-03] +
                48 * [1.1270890946720000e-03] +
                48 * [1.3457537609110001e-03] +
                48 * [1.4249572833170000e-03] +
                48 * [1.2615233412380000e-03] +
                48 * [1.3925471060530000e-03] +
                48 * [1.4187616778780000e-03] +
                48 * [1.3383666844799999e-03] +
                48 * [1.3937008626760000e-03] +
                48 * [1.4159147574670000e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(1.4468656741953087e-01, 9.8947753749559852e-01),
                _pq0(3.3902634754112165e-01, 9.4077687879375871e-01),
                _pq0(5.3358046512635049e-01, 8.4574930519365332e-01),
                _llm(5.0872044105023655e-02, 9.9740867765282304e-01),
                _llm(1.2281987901788295e-01, 9.8479975357230121e-01),
                _llm(2.0268908144087852e-01, 9.5803667598339148e-01),
                _llm(2.8477451564642942e-01, 9.1531795048315479e-01),
                _llm(3.6567190789780252e-01, 8.5590192869788650e-01),
                _llm(4.4282648867134683e-01, 7.7962131952763514e-01),
                _llm(5.1406196272497351e-01, 6.8664444726415419e-01),
                _llm(6.3064012191668017e-01, 4.5231192031365847e-01),
                _llm(6.7168833320226118e-01, 3.1252130500165304e-01),
                _llm(6.9797926853368808e-01, 1.6015580349882880e-01),
                _rsw(6.9440243933494045e-02, 2.3551878942423241e-01, 9.6938586349843214e-01),
                _rsw(2.2690041095294594e-01, 4.1021824740457302e-01, 8.8331036052211276e-01),
                _rsw(8.0255746077753320e-02, 6.2143024174816053e-01, 7.7934810570266089e-01),
                _rsw(1.4679995278965718e-01, 3.2452843457173941e-01, 9.3441482705240220e-01),
                _rsw(1.5715077698247268e-01, 5.2244821896966287e-01, 8.3806413345831254e-01),
                _rsw(2.3657029931572460e-01, 6.0175466340895567e-01, 7.6284062460466995e-01),
                _rsw(7.7148158667657396e-02, 4.3465755161411646e-01, 8.9728533613283334e-01),
                _rsw(3.0629366662107294e-01, 4.9088265890376170e-01, 8.1560922320397533e-01),
                _rsw(3.8224773795247857e-01, 5.6487681490994990e-01, 7.3130079365976586e-01),
                ])
            self.degree = 47
        elif index == 19:
            self.weights = numpy.array(
                6 * [1.4382941905300000e-04] +
                8 * [1.1257722882870001e-03] +
                24 * [6.8233679271100000e-04] +
                24 * [9.4541581604500003e-04] +
                24 * [1.0744299753860001e-03] +
                24 * [1.1293000865690000e-03] +
                24 * [4.9480293419500003e-04] +
                24 * [7.3579901091299998e-04] +
                24 * [8.8891327713000001e-04] +
                24 * [9.8883478389199999e-04] +
                24 * [1.0532996817090000e-03] +
                24 * [1.0927788070149999e-03] +
                24 * [1.1143893940630000e-03] +
                24 * [1.1237247880519999e-03] +
                24 * [1.1252393252440000e-03] +
                24 * [1.1261532718160000e-03] +
                24 * [1.1302869311240000e-03] +
                24 * [1.1349865343640000e-03] +
                48 * [8.4368845008999998e-04] +
                48 * [1.0752557204489999e-03] +
                48 * [1.1085772368639999e-03] +
                48 * [9.5664753237800001e-04] +
                48 * [1.0806632507170000e-03] +
                48 * [1.1267971311959999e-03] +
                48 * [1.0225687153580000e-03] +
                48 * [1.1089602677130000e-03] +
                48 * [1.1227906534360000e-03] +
                48 * [1.0324018471170001e-03] +
                48 * [1.1072493822840001e-03] +
                48 * [1.1217800485200000e-03]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(1.2366867626579896e-01, 9.9232356543149025e-01),
                _pq0(2.9407771144683864e-01, 9.5578151249654852e-01),
                _pq0(4.6977538492076509e-01, 8.8278598070118164e-01),
                _pq0(6.3345632411395669e-01, 7.7377844725737477e-01),
                _llm(4.2929635453413405e-02, 9.9815534502384651e-01),
                _llm(1.0514268540864033e-01, 9.8888322435468556e-01),
                _llm(1.7500248676230876e-01, 9.6889022043470741e-01),
                _llm(2.4776533796502581e-01, 9.3660273040716313e-01),
                _llm(3.2065671239559562e-01, 8.9126794264760612e-01),
                _llm(3.9165207498499821e-01, 8.3259672370235194e-01),
                _llm(4.5908258741876240e-01, 7.6058290531525141e-01),
                _llm(5.2145638884158607e-01, 6.7540096910841430e-01),
                _llm(6.2531702446541992e-01, 4.6685890569574318e-01),
                _llm(6.6379267445231693e-01, 3.4461365423743789e-01),
                _llm(6.9104103984983001e-01, 2.1195415185018424e-01),
                _llm(7.0529070074577593e-01, 7.1624401449955438e-02),
                _rsw(5.9740486141813431e-02, 2.0291287527775245e-01, 9.7737272284530996e-01),
                _rsw(1.3757604084736344e-01, 4.6026219424840537e-01, 8.7705846186580272e-01),
                _rsw(3.3910165263362868e-01, 5.0306739996620375e-01, 7.9494229996420840e-01),
                _rsw(1.2716751914398197e-01, 2.8176064224421354e-01, 9.5102016937438993e-01),
                _rsw(2.6931207404135121e-01, 4.3315612917201568e-01, 8.6014346160176203e-01),
                _rsw(1.4197864526019177e-01, 6.2561673585808142e-01, 7.6710218622055837e-01),
                _rsw(6.7092846007382440e-02, 3.7983952168591584e-01, 9.2261611073080896e-01),
                _rsw(7.0577381832561778e-02, 5.5175054214235197e-01, 8.3101755241347430e-01),
                _rsw(2.7838884778821549e-01, 6.0296191561591872e-01, 7.4762061083408571e-01),
                _rsw(1.9795789389174070e-01, 3.5896063295890962e-01, 9.1211837840912147e-01),
                _rsw(2.0873070611032746e-01, 5.3486664381354776e-01, 8.1874853628102173e-01),
                _rsw(4.0551221378728358e-01, 5.6749975460743718e-01, 7.1659184546702381e-01),
                ])
            self.degree = 53
        elif index == 20:
            self.weights = numpy.array(
                6 * [1.1051892332700000e-04] +
                12 * [9.2052327380900002e-04] +
                8 * [9.1331597864399998e-04] +
                24 * [5.1769773129699997e-04] +
                24 * [7.3311436821000002e-04] +
                24 * [8.4632328363799998e-04] +
                24 * [9.0311226942499997e-04] +
                24 * [3.6904218980200000e-04] +
                24 * [5.6039909286800002e-04] +
                24 * [6.8652976292800003e-04] +
                24 * [7.7203385511499995e-04] +
                24 * [8.3015459588899998e-04] +
                24 * [8.6866925501799996e-04] +
                24 * [8.9270762858499999e-04] +
                24 * [9.0608202385700004e-04] +
                24 * [9.1197772549399996e-04] +
                24 * [9.1287201385999996e-04] +
                24 * [9.1307149356900002e-04] +
                24 * [9.1528737845499996e-04] +
                24 * [9.1874362743199999e-04] +
                48 * [6.4857784531600004e-04] +
                48 * [7.4350309109799998e-04] +
                48 * [7.9985278918400004e-04] +
                48 * [8.1017314974700003e-04] +
                48 * [8.4833895745900002e-04] +
                48 * [8.5562992573100000e-04] +
                48 * [8.8032086797400005e-04] +
                48 * [8.8110481824300001e-04] +
                48 * [8.8502823412699996e-04] +
                48 * [9.0213422990399999e-04] +
                48 * [9.0100916771099996e-04] +
                48 * [9.0226929384299998e-04] +
                48 * [9.1580161746900003e-04] +
                48 * [9.1315780031900004e-04] +
                48 * [9.1078135794799996e-04] +
                48 * [9.1057602589700000e-04]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(1.0723822154781655e-01, 9.9423335482132236e-01),
                _pq0(2.5820689594969687e-01, 9.6608964329611902e-01),
                _pq0(4.1727529553067177e-01, 9.0878013168191052e-01),
                _pq0(5.7003669117925027e-01, 8.2161923706143347e-01),
                _llm(3.7126364496570509e-02, 9.9862068179991925e-01),
                _llm(9.1400604122621881e-02, 9.9161073972201386e-01),
                _llm(1.5310778524699065e-01, 9.7627660639468505e-01),
                _llm(2.1809288916606115e-01, 9.5124706748057852e-01),
                _llm(2.8398745322001745e-01, 9.1580688620866835e-01),
                _llm(3.4911776009637630e-01, 8.6961691518195416e-01),
                _llm(4.1214314614443104e-01, 8.1257372229991565e-01),
                _llm(4.7189936271491273e-01, 7.4472946963210651e-01),
                _llm(5.2731454528423372e-01, 6.6624225373610424e-01),
                _llm(6.2094753324440188e-01, 4.7838093807695220e-01),
                _llm(6.5697227118572921e-01, 3.6983086645942587e-01),
                _llm(6.8417883090701448e-01, 2.5258395570071812e-01),
                _llm(7.0126043301236296e-01, 1.2832618665972301e-01),
                _rsw(9.8279860182639467e-01, 1.7717740226153267e-01, 5.2106394770112772e-02),
                _rsw(9.6242492303262284e-01, 2.4757164634262899e-01, 1.1156409571564836e-01),
                _rsw(9.4020079941288115e-01, 3.3546162890664888e-01, 5.9058888532354067e-02),
                _rsw(9.3208220401432007e-01, 3.1736152466119788e-01, 1.7465516775786305e-01),
                _rsw(9.0436741993932990e-01, 4.0902684270853557e-01, 1.2172350510959895e-01),
                _rsw(8.9124075600747465e-01, 3.8542911506692229e-01, 2.3902784793817222e-01),
                _rsw(8.6764356284627075e-01, 4.9322211848512842e-01, 6.2662506241542140e-02),
                _rsw(8.5819799860416190e-01, 4.7853206759224354e-01, 1.8575051945473370e-01),
                _rsw(8.3967536240498564e-01, 4.5074225931570627e-01, 3.0294669735289814e-01),
                _rsw(8.1652885640221873e-01, 5.6321230207621009e-01, 1.2677748006842779e-01),
                _rsw(8.0154693707835289e-01, 5.4343035696938990e-01, 2.4941121623622373e-01),
                _rsw(7.7735630690703517e-01, 5.1235184864198702e-01, 3.6498322605976535e-01),
                _rsw(7.6616212139003936e-01, 6.3942796347491015e-01, 6.4245492242207575e-02),
                _rsw(7.5535841435335105e-01, 6.2698055090243920e-01, 1.9060182227792333e-01),
                _rsw(7.3443057575595028e-01, 6.0311616930963097e-01, 3.1122759471496086e-01),
                _rsw(7.0438371840217662e-01, 5.6937024984684392e-01, 4.2386447815223405e-01),
                ])
            self.degree = 59
        elif index == 21:
            self.weights = numpy.array(
                6 * [7.7771607433000002e-05] +
                8 * [7.5576464130000000e-04] +
                24 * [4.0216804478700001e-04] +
                24 * [5.8048717939500002e-04] +
                24 * [6.7921519559499999e-04] +
                24 * [7.3367412112899997e-04] +
                24 * [7.5818663009899999e-04] +
                24 * [2.8416338060899997e-04] +
                24 * [4.3744191270500001e-04] +
                24 * [5.4171747408699995e-04] +
                24 * [6.1480008913600000e-04] +
                24 * [6.6643944857999998e-04] +
                24 * [7.0250393569199995e-04] +
                24 * [7.2685117892499996e-04] +
                24 * [7.4226375342100002e-04] +
                24 * [7.5095450358399999e-04] +
                24 * [7.5485350577200001e-04] +
                24 * [7.5540889697700000e-04] +
                24 * [7.5531471744400003e-04] +
                24 * [7.5647676532900000e-04] +
                24 * [7.5879918085200004e-04] +
                24 * [7.6082618320300001e-04] +
                48 * [7.5382578598000001e-04] +
                48 * [7.4835172470499997e-04] +
                48 * [7.3717636611100000e-04] +
                48 * [7.1834488957599999e-04] +
                48 * [6.8958155298200002e-04] +
                48 * [6.4801058017899997e-04] +
                48 * [5.8975588965899997e-04] +
                48 * [5.0957088492500002e-04] +
                48 * [7.5369064289099999e-04] +
                48 * [7.4725059655800002e-04] +
                48 * [7.3430171322800002e-04] +
                48 * [7.1308715821799995e-04] +
                48 * [6.8170220321099996e-04] +
                48 * [6.3809411456000002e-04] +
                48 * [7.5503813779200001e-04] +
                48 * [7.4786466401400003e-04] +
                48 * [7.3359187205999997e-04] +
                48 * [7.1101205276600003e-04] +
                48 * [7.5713639786899995e-04] +
                48 * [7.4899083290800000e-04]
                )
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(9.9555461940918566e-01, 9.4185985013862314e-02),
                _pq0(9.7341159017942092e-01, 2.2906303958598623e-01),
                _pq0(9.2756937323886257e-01, 3.7365098398005542e-01),
                _pq0(8.5680224227951030e-01, 5.1564514700014707e-01),
                _pq0(7.6234955537193716e-01, 6.4716547762083987e-01),
                _llm(3.2292906634138953e-02, 9.9895662386423845e-01),
                _llm(8.0367332714622527e-02, 9.9352009726259405e-01),
                _llm(1.3542899605316530e-01, 9.8148763316511711e-01),
                _llm(1.9389638611144258e-01, 9.6166958094027533e-01),
                _llm(2.5373437150112760e-01, 9.3340116640052229e-01),
                _llm(3.1352514347525706e-01, 8.9632804754600814e-01),
                _llm(3.7215583393753393e-01, 8.5029410825461882e-01),
                _llm(4.2868095751956953e-01, 7.9527685325313613e-01),
                _llm(4.8225101282829935e-01, 7.3134664916998071e-01),
                _llm(5.3206793335662639e-01, 6.5864059136012665e-01),
                _llm(6.1729981953942747e-01, 4.8773134571522136e-01),
                _llm(6.5106798491274798e-01, 3.9015504359588543e-01),
                _llm(6.7773152516873614e-01, 2.8523667293130089e-01),
                _llm(6.9631094106487401e-01, 1.7407511799995651e-01),
                _llm(7.0589350098317516e-01, 5.8555363028785078e-02),
                _rsw(5.7075229088922230e-01, 4.3870280398895012e-01, 6.9410494323044358e-01),
                _rsw(5.1964633884030842e-01, 3.8589084147626163e-01, 7.6227025456501063e-01),
                _rsw(4.6463375312153499e-01, 3.3019373723438539e-01, 8.2163712875659778e-01),
                _rsw(4.0639016975576908e-01, 2.7254235735637777e-01, 8.7210532240808258e-01),
                _rsw(3.4563294666430860e-01, 2.1395102374952493e-01, 9.1365355885952604e-01),
                _rsw(2.8313951210503313e-01, 1.5559223097866468e-01, 9.4637364415119130e-01),
                _rsw(2.1976820229253305e-01, 9.8928789796861025e-02, 9.7052307124067727e-01),
                _rsw(1.5646960986503558e-01, 4.5986429106755163e-02, 9.8661163054501488e-01),
                _rsw(6.0273566737212936e-01, 3.3766251401734260e-01, 7.2297561639723484e-01),
                _rsw(5.4960323202550976e-01, 2.8223013097279875e-01, 7.8630937964530889e-01),
                _rsw(4.9217077552345684e-01, 2.2486323425925384e-01, 8.4095448961231367e-01),
                _rsw(4.3094229985984817e-01, 1.6662247234564789e-01, 8.8686283375780750e-01),
                _rsw(3.6641081823136723e-01, 1.0869649018221700e-01, 9.2408234768611786e-01),
                _rsw(2.9901890577584372e-01, 5.2519897841200959e-02, 9.5280079466768230e-01),
                _rsw(6.2687240131449995e-01, 2.2975236575500224e-01, 7.4447622050685558e-01),
                _rsw(5.7073241448346068e-01, 1.7230806070938007e-01, 8.0285393644949621e-01),
                _rsw(5.0963609019603651e-01, 1.1402384653905129e-01, 8.5280104244198496e-01),
                _rsw(4.4387299383124562e-01, 5.6115220958825630e-02, 8.9433094955057280e-01),
                _rsw(6.4199784710823893e-01, 1.1641744231408761e-01, 7.5781643122423281e-01),
                _rsw(5.8172180618026115e-01, 5.7975895314452193e-02, 8.1131900987026206e-01),
                ])
            self.degree = 65
        elif index == 22:
            self.weights = numpy.concatenate([
                6.3090494373999997e-05 * numpy.ones(6),
                6.3982877055699999e-04 * numpy.ones(12),
                6.3571850735299996e-04 * numpy.ones(8),
                3.1869134499499998e-04 * numpy.ones(24),
                4.6780285585900001e-04 * numpy.ones(24),
                5.5388296975999996e-04 * numpy.ones(24),
                6.0444759071900001e-04 * numpy.ones(24),
                6.3135751035099999e-04 * numpy.ones(24),
                2.2212071621900001e-04 * numpy.ones(24),
                3.4757840222900002e-04 * numpy.ones(24),
                4.3507424435900002e-04 * numpy.ones(24),
                4.9785691365199995e-04 * numpy.ones(24),
                5.4350362220000005e-04 * numpy.ones(24),
                5.7659133882199997e-04 * numpy.ones(24),
                6.0012003592300003e-04 * numpy.ones(24),
                6.1621781727200003e-04 * numpy.ones(24),
                6.2652181524400002e-04 * numpy.ones(24),
                6.3239871609700000e-04 * numpy.ones(24),
                6.3507678515400002e-04 * numpy.ones(24),
                6.3543627753000003e-04 * numpy.ones(24),
                6.3523024627099999e-04 * numpy.ones(24),
                6.3581178814200003e-04 * numpy.ones(24),
                6.3731015903100005e-04 * numpy.ones(24),
                6.3904289613700004e-04 * numpy.ones(24),
                4.0786264318600000e-04 * numpy.ones(48),
                4.7599330578099999e-04 * numpy.ones(48),
                5.2681511864100005e-04 * numpy.ones(48),
                5.6430485605100001e-04 * numpy.ones(48),
                5.9145010766100000e-04 * numpy.ones(48),
                6.1045612578699995e-04 * numpy.ones(48),
                6.2302528607100004e-04 * numpy.ones(48),
                6.3056187617599999e-04 * numpy.ones(48),
                6.3430927676000001e-04 * numpy.ones(48),
                5.1762689457400003e-04 * numpy.ones(48),
                5.5648403133099996e-04 * numpy.ones(48),
                5.8564266710400001e-04 * numpy.ones(48),
                6.0663869257800000e-04 * numpy.ones(48),
                6.2088249622299998e-04 * numpy.ones(48),
                6.2963142978199999e-04 * numpy.ones(48),
                6.3404237567899998e-04 * numpy.ones(48),
                5.8296276771100004e-04 * numpy.ones(48),
                6.0486933760800000e-04 * numpy.ones(48),
                6.2023623177300002e-04 * numpy.ones(48),
                6.2990053283999996e-04 * numpy.ones(48),
                6.3477223906099997e-04 * numpy.ones(48),
                6.2037789812399995e-04 * numpy.ones(48),
                6.3084146712400000e-04 * numpy.ones(48),
                6.3627064669600001e-04 * numpy.ones(48),
                6.3754141703299997e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(8.3549511663546516e-02, 9.9650362723915009e-01),
                _pq0(2.0501430090994879e-01, 9.7875897769696341e-01),
                _pq0(3.3702082907066366e-01, 9.4149719105928431e-01),
                _pq0(4.6890514842339626e-01, 8.8324852775537233e-01),
                _pq0(5.9394004245573340e-01, 8.0450930757057226e-01),
                _llm(2.8609231261945482e-02, 9.9918117665076189e-01),
                _llm(7.1425567677115179e-02, 9.9488530824613330e-01),
                _llm(1.2091995409955608e-01, 9.8526987643037300e-01),
                _llm(1.7386731065943808e-01, 9.6929887886456823e-01),
                _llm(2.2846454384677328e-01, 9.4636562934722623e-01),
                _llm(2.8348076717015125e-01, 9.1612079404914992e-01),
                _llm(3.3796801454673392e-01, 8.7838217325186951e-01),
                _llm(3.9113554548195367e-01, 8.3308221090182277e-01),
                _llm(4.4228603530014027e-01, 7.8023466082773207e-01),
                _llm(4.9077815687260562e-01, 7.1991221789427640e-01),
                _llm(5.3600061532114684e-01, 6.5223207583704734e-01),
                _llm(6.1421059735966033e-01, 4.9547016477501288e-01),
                _llm(6.4593003879775024e-01, 4.0687684863785678e-01),
                _llm(6.7180561250892257e-01, 3.1201672712055528e-01),
                _llm(6.9108885331862524e-01, 2.1164213577994129e-01),
                _llm(7.0304674168232539e-01, 1.0700728021836595e-01),
                _rsw(1.3949833118322624e-01, 4.0975811620503434e-02, 9.8937414483052666e-01),
                _rsw(1.9679991804850133e-01, 8.8519873912933436e-02, 9.7643946262865755e-01),
                _rsw(2.5461837325489661e-01, 1.3976801829698185e-01, 9.5688786441378004e-01),
                _rsw(3.1212810747138753e-01, 1.9294525422265266e-01, 9.3024092223428112e-01),
                _rsw(3.6859810785024927e-01, 2.4678983370615618e-01, 8.9623111576674452e-01),
                _rsw(4.2337603215478553e-01, 3.0031041247854090e-01, 8.5473176585045596e-01),
                _rsw(4.7586712360592465e-01, 3.5266843281750332e-01, 8.0571425280000464e-01),
                _rsw(5.2551785797964623e-01, 4.0311348611457143e-01, 7.4921992649491220e-01),
                _rsw(5.7180256337345903e-01, 4.5094264483423491e-01, 6.8534119939592608e-01),
                _rsw(2.6869277727234125e-01, 4.7113225024232500e-02, 9.6207304061047882e-01),
                _rsw(3.3060068199048093e-01, 9.7844873039427002e-02, 9.3868502165919709e-01),
                _rsw(3.9049068505949841e-01, 1.5053958100252732e-01, 9.0821520546253087e-01),
                _rsw(4.4799579519043908e-01, 2.0397281562960506e-01, 8.7045669506059709e-01),
                _rsw(5.0270768489197815e-01, 2.5715299411211068e-01, 8.2532255583482295e-01),
                _rsw(5.5420873922602154e-01, 3.0921913758156688e-01, 7.7281058372593192e-01),
                _rsw(6.0208508873751865e-01, 3.5938075061302760e-01, 7.1297897725581727e-01),
                _rsw(4.0198514091795939e-01, 5.0633899343786593e-02, 9.1424512835366623e-01),
                _rsw(4.6356145674497978e-01, 1.0324222691606123e-01, 8.8002944178128761e-01),
                _rsw(5.2158609315915749e-01, 1.5663220940062539e-01, 8.3869797805959045e-01),
                _rsw(5.7582024990992720e-01, 2.0980828274910970e-01, 7.9019714266981411e-01),
                _rsw(6.2598936838767960e-01, 2.6188241145533908e-01, 7.3454401722152118e-01),
                _rsw(5.3137951248118898e-01, 5.2632450193386186e-02, 8.4549727314870426e-01),
                _rsw(5.8933179559319926e-01, 1.0610597309820056e-01, 8.0089297485730226e-01),
                _rsw(6.4262463212158016e-01, 1.5941715640342211e-01, 7.4941293852912005e-01),
                _rsw(6.5119043673761123e-01, 5.3547895365655913e-02, 7.5702287812416080e-01),
                ])
            self.degree = 71
        elif index == 23:
            self.weights = numpy.concatenate([
                4.6560318991999998e-05 * numpy.ones(6),
                5.4215491952999998e-04 * numpy.ones(8),
                2.5680024977299998e-04 * numpy.ones(24),
                3.8272117002899998e-04 * numpy.ones(24),
                4.5794915619200001e-04 * numpy.ones(24),
                5.0420039690799996e-04 * numpy.ones(24),
                5.3127088899800003e-04 * numpy.ones(24),
                5.4384017907500002e-04 * numpy.ones(24),
                1.7785221333499999e-04 * numpy.ones(24),
                2.8113254056799999e-04 * numpy.ones(24),
                3.5488963126299999e-04 * numpy.ones(24),
                4.0903108971700002e-04 * numpy.ones(24),
                4.4932861341700001e-04 * numpy.ones(24),
                4.7937284479600002e-04 * numpy.ones(24),
                5.0154153191599995e-04 * numpy.ones(24),
                5.1751273726800004e-04 * numpy.ones(24),
                5.2855222620799998e-04 * numpy.ones(24),
                5.3568327037100002e-04 * numpy.ones(24),
                5.3979147361800004e-04 * numpy.ones(24),
                5.4168994416000000e-04 * numpy.ones(24),
                5.4193084768899998e-04 * numpy.ones(24),
                5.4169369020300000e-04 * numpy.ones(24),
                5.4195443387000002e-04 * numpy.ones(24),
                5.4289836566299998e-04 * numpy.ones(24),
                5.4422865001000004e-04 * numpy.ones(24),
                5.4522503450600004e-04 * numpy.ones(24),
                3.3160418732000000e-04 * numpy.ones(48),
                3.8991135671499999e-04 * numpy.ones(48),
                4.3433433272000002e-04 * numpy.ones(48),
                4.6794152623200002e-04 * numpy.ones(48),
                4.9308479816299996e-04 * numpy.ones(48),
                5.1150318675400003e-04 * numpy.ones(48),
                5.2452171484600001e-04 * numpy.ones(48),
                5.3320414999000003e-04 * numpy.ones(48),
                5.3845831260199996e-04 * numpy.ones(48),
                5.4110672108000004e-04 * numpy.ones(48),
                4.2597973914700000e-04 * numpy.ones(48),
                4.6049313684600000e-04 * numpy.ones(48),
                4.8718148782600002e-04 * numpy.ones(48),
                5.0722429100699999e-04 * numpy.ones(48),
                5.2170698452400001e-04 * numpy.ones(48),
                5.3157859662800004e-04 * numpy.ones(48),
                5.3768337087600000e-04 * numpy.ones(48),
                5.4080320920700000e-04 * numpy.ones(48),
                4.8427449178999998e-04 * numpy.ones(48),
                5.0489260761899999e-04 * numpy.ones(48),
                5.2026079804800001e-04 * numpy.ones(48),
                5.3099323883300004e-04 * numpy.ones(48),
                5.3774197708999995e-04 * numpy.ones(48),
                5.4116963316800000e-04 * numpy.ones(48),
                5.1979962932800002e-04 * numpy.ones(48),
                5.3111208366200000e-04 * numpy.ones(48),
                5.3843093199600002e-04 * numpy.ones(48),
                5.4218595040499997e-04 * numpy.ones(48),
                5.3909483550499998e-04 * numpy.ones(48),
                5.4333127050299997e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(7.4790281683497675e-02, 9.9719928488026055e-01),
                _pq0(1.8489511539693673e-01, 9.8275825934069538e-01),
                _pq0(3.0595290665813057e-01, 9.5204664744299228e-01),
                _pq0(4.2855561010213628e-01, 9.0351540609443159e-01),
                _pq0(5.4687586534965260e-01, 8.3721370503478298e-01),
                _pq0(6.5658219783434391e-01, 7.5425447793634104e-01),
                _llm(2.5408353368143354e-02, 9.9935420705485556e-01),
                _llm(6.3993228005049127e-02, 9.9589644719668902e-01),
                _llm(1.0882694698041234e-01, 9.8808572058392030e-01),
                _llm(1.5706707988182866e-01, 9.7501787923852479e-01),
                _llm(2.0711639322825146e-01, 9.5614099342735048e-01),
                _llm(2.5789140444508452e-01, 9.3111978124550854e-01),
                _llm(3.0856875581696230e-01, 8.9976143830859068e-01),
                _llm(3.5847197062670239e-01, 8.6197197898192568e-01),
                _llm(4.0701355944287082e-01, 8.1772851537615421e-01),
                _llm(4.5366186262226399e-01, 7.6706051182693291e-01),
                _llm(4.9791956864635778e-01, 7.1003676406088312e-01),
                _llm(5.3930751111269981e-01, 6.4675715451384852e-01),
                _llm(6.1156176768439163e-01, 5.0198048628754932e-01),
                _llm(6.4143084351601587e-01, 4.2087165023634560e-01),
                _llm(6.6640994127216080e-01, 3.3435846085791010e-01),
                _llm(6.8591617712149144e-01, 2.4297735681762167e-01),
                _llm(6.9936255935038893e-01, 1.4759410949542379e-01),
                _llm(7.0623933877193790e-01, 4.9517600325051626e-02),
                _rsw(1.2539015723671160e-01, 3.6819172264396377e-02, 9.9142405509545584e-01),
                _rsw(1.7757215103839388e-01, 7.9824876072132941e-02, 9.8086498578329651e-01),
                _rsw(2.3056933582161115e-01, 1.2646409665923330e-01, 9.6480288848808138e-01),
                _rsw(2.8365028459920621e-01, 1.7515856834189567e-01, 9.4279477723585570e-01),
                _rsw(3.3617947462325892e-01, 2.2479959076326705e-01, 9.1457558727242294e-01),
                _rsw(3.8759791722648246e-01, 2.7452992574222468e-01, 8.8000066729160009e-01),
                _rsw(4.3740193169990726e-01, 3.2363734824411183e-01, 8.3901037953454971e-01),
                _rsw(4.8512758433400222e-01, 3.7149678594367413e-01, 7.9160682472536548e-01),
                _rsw(5.3033918038068673e-01, 4.1753536463217444e-01, 7.3783776877753993e-01),
                _rsw(5.7261973805962874e-01, 4.6120844063554606e-01, 6.7778566661670425e-01),
                _rsw(2.4315207325648619e-01, 4.2580401330439485e-02, 9.6905313512397817e-01),
                _rsw(3.0020968008958715e-01, 8.8694243067227160e-02, 9.4974074316480672e-01),
                _rsw(3.5585544574574318e-01, 1.3688117065106553e-01, 9.2446224739266247e-01),
                _rsw(4.0977825370488857e-01, 1.8607399850150316e-01, 8.9300517908477695e-01),
                _rsw(4.6163376660674582e-01, 2.3542350773958545e-01, 8.5526021626874349e-01),
                _rsw(5.1107070084178752e-01, 2.8420749213470115e-01, 8.1119223378653471e-01),
                _rsw(5.5774152861637960e-01, 3.3177844149841024e-01, 7.6082025013372923e-01),
                _rsw(6.0130604313669511e-01, 3.7752990020407012e-01, 7.0420324973632142e-01),
                _rsw(3.6615967672617800e-01, 4.5993678871646175e-02, 9.2941469358066031e-01),
                _rsw(4.2376331535065809e-01, 9.4048937736544422e-02, 9.0087704481446651e-01),
                _rsw(4.7863284546584522e-01, 1.4313771090919725e-01, 8.6626912386217680e-01),
                _rsw(5.3057020767897745e-01, 1.9241863888435701e-01, 8.2551215747157725e-01),
                _rsw(5.7934362242317883e-01, 2.4115909447751882e-01, 7.7859055883588280e-01),
                _rsw(6.2470690170947474e-01, 2.8868714915836047e-01, 7.2553498665975247e-01),
                _rsw(4.8743155525352044e-01, 4.8049787749532079e-02, 8.7183811389521115e-01),
                _rsw(5.4273373220590526e-01, 9.7168571993666422e-02, 8.3426516440671328e-01),
                _rsw(5.9434937472467009e-01, 1.4652058397950551e-01, 7.9074682372722715e-01),
                _rsw(6.4213140335649432e-01, 1.9535794498035772e-01, 7.4128438143297670e-01),
                _rsw(6.0206283747139788e-01, 4.9163750157381600e-02, 7.9693366437009783e-01),
                _rsw(6.5292225298568807e-01, 9.8616215401270177e-02, 7.5097761192729540e-01),
                ])
            self.degree = 77
        elif index == 24:
            self.weights = numpy.concatenate([
                3.9226162707000000e-05 * numpy.ones(6),
                4.7038317508499999e-04 * numpy.ones(12),
                4.6782028012799998e-04 * numpy.ones(8),
                2.0999422810700000e-04 * numpy.ones(24),
                3.1722691507099997e-04 * numpy.ones(24),
                3.8320513585500001e-04 * numpy.ones(24),
                4.2521938181499998e-04 * numpy.ones(24),
                4.5138079637499999e-04 * numpy.ones(24),
                4.6577974691100000e-04 * numpy.ones(24),
                1.4378322289800000e-04 * numpy.ones(24),
                2.3035724935800000e-04 * numpy.ones(24),
                2.9331107524499999e-04 * numpy.ones(24),
                3.4029059983600003e-04 * numpy.ones(24),
                3.7591384668699999e-04 * numpy.ones(24),
                4.0306384479000000e-04 * numpy.ones(24),
                4.2365914322399999e-04 * numpy.ones(24),
                4.3905226569500002e-04 * numpy.ones(24),
                4.5025234666300001e-04 * numpy.ones(24),
                4.5805777277800001e-04 * numpy.ones(24),
                4.6313916166199997e-04 * numpy.ones(24),
                4.6609289537000002e-04 * numpy.ones(24),
                4.6747518079400002e-04 * numpy.ones(24),
                4.6764149039300000e-04 * numpy.ones(24),
                4.6740864923500001e-04 * numpy.ones(24),
                4.6749285394800000e-04 * numpy.ones(24),
                4.6807489796900001e-04 * numpy.ones(24),
                4.6904498063900002e-04 * numpy.ones(24),
                4.6998770758599999e-04 * numpy.ones(24),
                2.7333628005199998e-04 * numpy.ones(48),
                3.2354853684600000e-04 * numpy.ones(48),
                3.6249087260100001e-04 * numpy.ones(48),
                3.9255400707100002e-04 * numpy.ones(48),
                4.1561297811200002e-04 * numpy.ones(48),
                4.3306449846200001e-04 * numpy.ones(48),
                4.4596777259200000e-04 * numpy.ones(48),
                4.5515930044600001e-04 * numpy.ones(48),
                4.6133414627500002e-04 * numpy.ones(48),
                4.6510196182699998e-04 * numpy.ones(48),
                4.6702495361000001e-04 * numpy.ones(48),
                3.5495555764399999e-04 * numpy.ones(48),
                3.8561082452500002e-04 * numpy.ones(48),
                4.0986228457600001e-04 * numpy.ones(48),
                4.2863286042700002e-04 * numpy.ones(48),
                4.4278021989899998e-04 * numpy.ones(48),
                4.5304735114900000e-04 * numpy.ones(48),
                4.6008054757000000e-04 * numpy.ones(48),
                4.6445990599599998e-04 * numpy.ones(48),
                4.6672744557100002e-04 * numpy.ones(48),
                4.0693605180200002e-04 * numpy.ones(48),
                4.2604428199200000e-04 * numpy.ones(48),
                4.4086785080300000e-04 * numpy.ones(48),
                4.5187481155500002e-04 * numpy.ones(48),
                4.5955648753800000e-04 * numpy.ones(48),
                4.6439887743199998e-04 * numpy.ones(48),
                4.6688274916500001e-04 * numpy.ones(48),
                4.4005418237400001e-04 * numpy.ones(48),
                4.5145128901899999e-04 * numpy.ones(48),
                4.5961986273500001e-04 * numpy.ones(48),
                4.6486590167999999e-04 * numpy.ones(48),
                4.6755020171599998e-04 * numpy.ones(48),
                4.5984944764599999e-04 * numpy.ones(48),
                4.6549169551499998e-04 * numpy.ones(48),
                4.6847097795099999e-04 * numpy.ones(48),
                4.6914455391099997e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(6.7440330883060826e-02, 9.9772330922474861e-01),
                _pq0(1.6786844853341654e-01, 9.8580940550746610e-01),
                _pq0(2.7935590495396162e-01, 9.6018762664770529e-01),
                _pq0(3.9352642180576397e-01, 9.1931330640905651e-01),
                _pq0(5.0526292682325580e-01, 8.6296545398874291e-01),
                _pq0(6.1079053154375318e-01, 7.9179222437234087e-01),
                _llm(2.2900246465305467e-02, 9.9947544113082465e-01),
                _llm(5.7790866522713057e-02, 9.9665461996275717e-01),
                _llm(9.8631035763759783e-02, 9.9022413501607598e-01),
                _llm(1.4281557929821839e-01, 9.7939135212611950e-01),
                _llm(1.8889781166014627e-01, 9.6365721784253544e-01),
                _llm(2.3590916829702094e-01, 9.4270553654193401e-01),
                _llm(2.8312288337061708e-01, 9.1634211178130187e-01),
                _llm(3.2994958579666922e-01, 8.8445833235105698e-01),
                _llm(3.7588408026607967e-01, 8.4700786088739854e-01),
                _llm(4.2047518310094811e-01, 8.0399082133594568e-01),
                _llm(4.6330685187510506e-01, 7.5544260007704012e-01),
                _llm(5.0398494745073152e-01, 7.0142593727789038e-01),
                _llm(5.4212657934407471e-01, 6.4202612403031167e-01),
                _llm(6.0926602305573097e-01, 5.0753307902018252e-01),
                _llm(6.3746542049848676e-01, 4.3275359656203288e-01),
                _llm(6.6151364726098927e-01, 3.5326956983998387e-01),
                _llm(6.8094872859581257e-01, 2.6947663729439048e-01),
                _llm(6.9529800216651982e-01, 1.8199279207290878e-01),
                _llm(7.0412454976953986e-01, 9.1745500291214574e-02),
                _rsw(1.1350810398435275e-01, 3.3319548846625982e-02, 9.9297820620320498e-01),
                _rsw(1.6128666260993751e-01, 7.2471674654365298e-02, 9.8424309438112900e-01),
                _rsw(2.1007865501682024e-01, 1.1515391108497439e-01, 9.7087925895456106e-01),
                _rsw(2.5922820094599425e-01, 1.5994910971436771e-01, 9.5247940772275197e-01),
                _rsw(3.0817405613202020e-01, 2.0586999560280264e-01, 9.2878754084970394e-01),
                _rsw(3.5642897815781655e-01, 2.5216249535029128e-01, 8.9965129881976447e-01),
                _rsw(4.0355872882407029e-01, 2.9820907857976742e-01, 8.6499231085744721e-01),
                _rsw(4.4916711963739042e-01, 3.4347620872357337e-01, 8.2478663463803203e-01),
                _rsw(4.9288547829174889e-01, 3.8748313572034376e-01, 7.7905116957838116e-01),
                _rsw(5.3436467919589881e-01, 4.2978148217469253e-01, 7.2783395579458132e-01),
                _rsw(5.7326832165309893e-01, 4.6994022609435387e-01, 6.7120757987929980e-01),
                _rsw(2.2141315832189867e-01, 3.8736020406439080e-02, 9.7441045460575482e-01),
                _rsw(2.7417965047500720e-01, 8.0894962569020151e-02, 9.5827006855915087e-01),
                _rsw(3.2597974391494855e-01, 1.2517321776208723e-01, 9.3705329203425225e-01),
                _rsw(3.7654411488268896e-01, 1.7062602864031837e-01, 9.1054999198157460e-01),
                _rsw(4.2557735745305580e-01, 2.1651151473004074e-01, 8.7863910498712572e-01),
                _rsw(4.7277951170584298e-01, 2.6220898122252584e-01, 8.4126451457162377e-01),
                _rsw(5.1785468958190117e-01, 3.0717214312961999e-01, 7.9841830825901594e-01),
                _rsw(5.6051411920974603e-01, 3.5089989988011383e-01, 7.5012877723138227e-01),
                _rsw(6.0047633193525107e-01, 3.9291608761669317e-01, 6.9645180944382445e-01),
                _rsw(3.3528426349469476e-01, 4.2025634572880000e-02, 9.4117921178252062e-01),
                _rsw(3.8919716298146706e-01, 8.6143097588708584e-02, 9.1711773239044925e-01),
                _rsw(4.4098755655422811e-01, 1.3145008793800003e-01, 8.8783492235067185e-01),
                _rsw(4.9048930585924838e-01, 1.7721896573838580e-01, 8.5323717629996032e-01),
                _rsw(5.3750561387695495e-01, 2.2282771100502949e-01, 8.1328686590834409e-01),
                _rsw(5.8182557086699704e-01, 2.6771799350143849e-01, 7.6798833392236943e-01),
                _rsw(6.2323348581449589e-01, 3.1136750355441667e-01, 7.1737737620432596e-01),
                _rsw(4.4894853544920571e-01, 4.4091623783681810e-02, 8.9246912620556884e-01),
                _rsw(5.0151368759331516e-01, 8.9390099177484791e-02, 8.6051928004292511e-01),
                _rsw(5.5113005505126222e-01, 1.3518060293833664e-01, 8.2339654299032650e-01),
                _rsw(5.9767204098579985e-01, 1.8083703550531960e-01, 7.8108008425097875e-01),
                _rsw(6.4099563789893554e-01, 2.2578521923016018e-01, 7.3358409672765212e-01),
                _rsw(5.5812223308275166e-01, 4.5321734216371340e-02, 8.2852007419636087e-01),
                _rsw(6.0747059841616946e-01, 9.1174880318403276e-02, 7.8909227170136809e-01),
                _rsw(6.5322725373790314e-01, 1.3692942131401573e-01, 7.4467743926659002e-01),
                _rsw(6.5947614945004873e-01, 4.5899014872756147e-02, 7.5032292297400027e-01),
                ])
            self.degree = 83
        elif index == 25:
            self.weights = numpy.concatenate([
                2.9986751498999999e-05 * numpy.ones(6),
                4.0778605295000000e-04 * numpy.ones(8),
                1.7389868117500000e-04 * numpy.ones(24),
                2.6596160452799998e-04 * numpy.ones(24),
                3.2405960081700002e-04 * numpy.ones(24),
                3.6211959644300002e-04 * numpy.ones(24),
                3.8688383307599998e-04 * numpy.ones(24),
                4.0189115326900001e-04 * numpy.ones(24),
                4.0899294329799999e-04 * numpy.ones(24),
                1.1853491925200000e-04 * numpy.ones(24),
                1.9134086434300000e-04 * numpy.ones(24),
                2.4528865772099998e-04 * numpy.ones(24),
                2.8624081832899999e-04 * numpy.ones(24),
                3.1780322582600001e-04 * numpy.ones(24),
                3.4229456676300002e-04 * numpy.ones(24),
                3.6127905202400001e-04 * numpy.ones(24),
                3.7586382298199997e-04 * numpy.ones(24),
                3.8687117988599998e-04 * numpy.ones(24),
                3.9494299331899998e-04 * numpy.ones(24),
                4.0060681075399998e-04 * numpy.ones(24),
                4.0431921496699997e-04 * numpy.ones(24),
                4.0649474958100000e-04 * numpy.ones(24),
                4.0752456198100002e-04 * numpy.ones(24),
                4.0764235408899998e-04 * numpy.ones(24),
                4.0742808622500000e-04 * numpy.ones(24),
                4.0741637560100003e-04 * numpy.ones(24),
                4.0776477950699999e-04 * numpy.ones(24),
                4.0845175527800002e-04 * numpy.ones(24),
                4.0924684592199997e-04 * numpy.ones(24),
                4.0978726872399997e-04 * numpy.ones(24),
                2.2799075277100001e-04 * numpy.ones(48),
                2.7152054905800000e-04 * numpy.ones(48),
                3.0579178966999999e-04 * numpy.ones(48),
                3.3269130524499999e-04 * numpy.ones(48),
                3.5373347118899999e-04 * numpy.ones(48),
                3.7005675007799999e-04 * numpy.ones(48),
                3.8252453725900001e-04 * numpy.ones(48),
                3.9181251715200000e-04 * numpy.ones(48),
                3.9847204199400000e-04 * numpy.ones(48),
                4.0297460033400002e-04 * numpy.ones(48),
                4.0574286321599999e-04 * numpy.ones(48),
                4.0717192741100000e-04 * numpy.ones(48),
                2.9902369506600002e-04 * numpy.ones(48),
                3.2629517342099999e-04 * numpy.ones(48),
                3.4826346082400002e-04 * numpy.ones(48),
                3.6565966817000002e-04 * numpy.ones(48),
                3.7917404677900002e-04 * numpy.ones(48),
                3.8940344501599999e-04 * numpy.ones(48),
                3.9686002455099998e-04 * numpy.ones(48),
                4.0199313514199998e-04 * numpy.ones(48),
                4.0521088012800003e-04 * numpy.ones(48),
                4.0689786139400000e-04 * numpy.ones(48),
                3.4542753513200000e-04 * numpy.ones(48),
                3.6299635370100000e-04 * numpy.ones(48),
                3.7701872338899997e-04 * numpy.ones(48),
                3.8786086136900002e-04 * numpy.ones(48),
                3.9590652702200000e-04 * numpy.ones(48),
                4.0152869754599997e-04 * numpy.ones(48),
                4.0508667856100001e-04 * numpy.ones(48),
                4.0693201850499998e-04 * numpy.ones(48),
                3.7601209640599999e-04 * numpy.ones(48),
                3.8709695644200000e-04 * numpy.ones(48),
                3.9552877905299998e-04 * numpy.ones(48),
                4.0153619112999998e-04 * numpy.ones(48),
                4.0538369867200002e-04 * numpy.ones(48),
                4.0735786732999999e-04 * numpy.ones(48),
                3.9546283792299999e-04 * numpy.ones(48),
                4.0176455088499998e-04 * numpy.ones(48),
                4.0590303486499999e-04 * numpy.ones(48),
                4.0805658094800000e-04 * numpy.ones(48),
                4.0630187536599998e-04 * numpy.ones(48),
                4.0871912928000000e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(6.1235549898947694e-02, 9.9812334279315074e-01),
                _pq0(1.5330703483123939e-01, 9.8817860383194556e-01),
                _pq0(2.5639026052442060e-01, 9.6657334657449545e-01),
                _pq0(3.6293469916633631e-01, 9.3181457605096574e-01),
                _pq0(4.6839499689875363e-01, 8.8351917176720995e-01),
                _pq0(5.6944792406579525e-01, 8.2202740938313990e-01),
                _pq0(6.6344654309939566e-01, 7.4822368610560686e-01),
                _llm(2.0655625388186778e-02, 9.9957325408378439e-01),
                _llm(5.2509181730223467e-02, 9.9723897420229457e-01),
                _llm(8.9934800820383770e-02, 9.9187875428541961e-01),
                _llm(1.3060239244360172e-01, 9.8279500923438501e-01),
                _llm(1.7320603885314198e-01, 9.6953562915944858e-01),
                _llm(2.1687270848202492e-01, 9.5180484167256751e-01),
                _llm(2.6095283091735866e-01, 9.2941230897402738e-01),
                _llm(3.0492529279389502e-01, 9.0224227989443873e-01),
                _llm(3.4834841380844045e-01, 8.7023374170063483e-01),
                _llm(3.9083215491064061e-01, 8.3336693801458794e-01),
                _llm(4.3202100718948133e-01, 7.9165377450876351e-01),
                _llm(4.7158247958900523e-01, 7.4513081394435088e-01),
                _llm(5.0919847940784535e-01, 6.9385431982331569e-01),
                _llm(5.4455801456508024e-01, 6.3789743497358231e-01),
                _llm(6.0725757968417682e-01, 5.1232456883525634e-01),
                _llm(6.3394845057558036e-01, 4.4296582715333938e-01),
                _llm(6.5707182574869583e-01, 3.6947697034395921e-01),
                _llm(6.7625573300907094e-01, 2.9215812010746539e-01),
                _llm(6.9111616969237888e-01, 2.1146366113228454e-01),
                _llm(7.0128419116599627e-01, 1.2806625801244187e-01),
                _llm(7.0645592724100192e-01, 4.2895754243422561e-02),
                _rsw(1.0339585735523012e-01, 3.0345440090635702e-02, 9.9417727340121920e-01),
                _rsw(1.4735214124143958e-01, 6.6188030442471402e-02, 9.8686700780688241e-01),
                _rsw(1.9245521587059655e-01, 1.0544311289877141e-01, 9.7562428210168028e-01),
                _rsw(2.3810943628903289e-01, 1.4682635512388584e-01, 9.6007599584155501e-01),
                _rsw(2.8381217079367582e-01, 1.8944861081878855e-01, 9.3997865697483396e-01),
                _rsw(3.2913231333734150e-01, 2.3263742387615793e-01, 9.1517853412843708e-01),
                _rsw(3.7368969787414602e-01, 2.7584858084857683e-01, 8.8558656840907179e-01),
                _rsw(4.1714060407600123e-01, 3.1861793319969217e-01, 8.5116175259151849e-01),
                _rsw(4.5916779852569145e-01, 3.6053297963037950e-01, 8.1189956484525250e-01),
                _rsw(4.9947338317184187e-01, 4.0121472535865094e-01, 7.6782360191534660e-01),
                _rsw(5.3777318304450950e-01, 4.4030500255706911e-01, 7.1897949089066215e-01),
                _rsw(5.7379178300013312e-01, 4.7745659042774841e-01, 6.6543083338438536e-01),
                _rsw(2.0273235862713887e-01, 3.5441225049761521e-02, 9.7859261714589352e-01),
                _rsw(2.5169423751872738e-01, 7.4183043886463226e-02, 9.6495952599039381e-01),
                _rsw(3.0002279952571809e-01, 1.1505027457271860e-01, 9.4696871864148335e-01),
                _rsw(3.4748066910463438e-01, 1.5719633712093639e-01, 9.2441684114600409e-01),
                _rsw(3.9381031803592081e-01, 1.9996318772470992e-01, 8.9717788479399074e-01),
                _rsw(4.3875195904557041e-01, 2.4280734578465352e-01, 8.6518281956282850e-01),
                _rsw(4.8205039600777871e-01, 2.8525751329061544e-01, 8.2840543625690799e-01),
                _rsw(5.2345737784751001e-01, 3.2688842086746384e-01, 7.8685216774168254e-01),
                _rsw(5.6273186472352810e-01, 3.6730333216759375e-01, 7.4055459663914247e-01),
                _rsw(5.9963906071569539e-01, 4.0612115518302883e-01, 6.8956406821759553e-01),
                _rsw(3.0847807537919453e-01, 3.8601255231000643e-02, 9.5044790499266141e-01),
                _rsw(3.5899882759202228e-01, 7.9289389871048610e-02, 9.2996399631460480e-01),
                _rsw(4.0786284158819730e-01, 1.2126146430300855e-01, 9.0495500425528963e-01),
                _rsw(4.5492872588897348e-01, 1.6387708273826926e-01, 8.7531945946278933e-01),
                _rsw(5.0002785129572791e-01, 2.0659657982601762e-01, 8.4100535142932908e-01),
                _rsw(5.4297850449281992e-01, 2.4894363788522333e-01, 8.0199838454650985e-01),
                _rsw(5.8359398504917115e-01, 2.9048113689468902e-01, 7.5831310797242368e-01),
                _rsw(6.2168703534448544e-01, 3.3079419576666086e-01, 7.0998621826882835e-01),
                _rsw(4.1511046627090914e-01, 4.0648291460525163e-02, 9.0886248530439884e-01),
                _rsw(4.6498042750092167e-01, 8.2584245472947626e-02, 8.8146074469639546e-01),
                _rsw(5.1246957570096630e-01, 1.2518419620272880e-01, 8.4953154797332175e-01),
                _rsw(5.5747111006062233e-01, 1.6791075059763316e-01, 8.1303870835373937e-01),
                _rsw(5.9985973332872278e-01, 2.1028050573587170e-01, 7.7197824401874160e-01),
                _rsw(6.3950071485165993e-01, 2.5184180877741075e-01, 7.2637079997473597e-01),
                _rsw(5.1884562247462518e-01, 4.1943216760774456e-02, 8.5383838436010673e-01),
                _rsw(5.6641907079427778e-01, 8.4576615519214693e-02, 8.1976596193539397e-01),
                _rsw(6.1104643532831526e-01, 1.2736529325193990e-01, 7.8128121438276399e-01),
                _rsw(6.5264303020515635e-01, 1.6981732390763551e-01, 7.3838956630323582e-01),
                _rsw(6.1675518803775464e-01, 4.2663988515488978e-02, 7.8599797844044361e-01),
                _rsw(6.6071954183553827e-01, 8.5519258142383162e-02, 7.4574536104719602e-01),
                ])
            self.degree = 89
        elif index == 26:
            self.weights = numpy.concatenate([
                2.5990959538000000e-05 * numpy.ones(6),
                3.6031340896900001e-04 * numpy.ones(12),
                3.5860679744099998e-04 * numpy.ones(8),
                1.4564470967399999e-04 * numpy.ones(24),
                2.2523701882799999e-04 * numpy.ones(24),
                2.7661354434699997e-04 * numpy.ones(24),
                3.1107294914999999e-04 * numpy.ones(24),
                3.3425067122999999e-04 * numpy.ones(24),
                3.4919818340299998e-04 * numpy.ones(24),
                3.5760036043500001e-04 * numpy.ones(24),
                9.8315284743999995e-05 * numpy.ones(24),
                1.6050231079500001e-04 * numpy.ones(24),
                2.0722001314600001e-04 * numpy.ones(24),
                2.4312976188099999e-04 * numpy.ones(24),
                2.7118190645000003e-04 * numpy.ones(24),
                2.9327620383199997e-04 * numpy.ones(24),
                3.1070325142000001e-04 * numpy.ones(24),
                3.2438080589199998e-04 * numpy.ones(24),
                3.3498990913700000e-04 * numpy.ones(24),
                3.4305806885100002e-04 * numpy.ones(24),
                3.4901241092899997e-04 * numpy.ones(24),
                3.5321489485599998e-04 * numpy.ones(24),
                3.5598626690599999e-04 * numpy.ones(24),
                3.5762243175500001e-04 * numpy.ones(24),
                3.5840505330900002e-04 * numpy.ones(24),
                3.5849035813699997e-04 * numpy.ones(24),
                3.5829918790400000e-04 * numpy.ones(24),
                3.5823711879600002e-04 * numpy.ones(24),
                3.5843536311199999e-04 * numpy.ones(24),
                3.5891201665200001e-04 * numpy.ones(24),
                3.5954457045299999e-04 * numpy.ones(24),
                3.6009435571099997e-04 * numpy.ones(24),
                1.9219213057900000e-04 * numpy.ones(48),
                2.3014582165000000e-04 * numpy.ones(48),
                2.6042485495200002e-04 * numpy.ones(48),
                2.8452754258699998e-04 * numpy.ones(48),
                3.0368708979699999e-04 * numpy.ones(48),
                3.1884148323000001e-04 * numpy.ones(48),
                3.3070464147200000e-04 * numpy.ones(48),
                3.3983309690300000e-04 * numpy.ones(48),
                3.4667578997099998e-04 * numpy.ones(48),
                3.5160959232300002e-04 * numpy.ones(48),
                3.5496451840500001e-04 * numpy.ones(48),
                3.5704159694399998e-04 * numpy.ones(48),
                3.5812517985000001e-04 * numpy.ones(48),
                2.5434913299100001e-04 * numpy.ones(48),
                2.7867110513299999e-04 * numpy.ones(48),
                2.9855523610800000e-04 * numpy.ones(48),
                3.1458679291499999e-04 * numpy.ones(48),
                3.2732906620700002e-04 * numpy.ones(48),
                3.3727055119399998e-04 * numpy.ones(48),
                3.4482744378499998e-04 * numpy.ones(48),
                3.5035927830499999e-04 * numpy.ones(48),
                3.5418547926600001e-04 * numpy.ones(48),
                3.5659955179100002e-04 * numpy.ones(48),
                3.5788020782999999e-04 * numpy.ones(48),
                2.9586445928599998e-04 * numpy.ones(48),
                3.1195481291200000e-04 * numpy.ones(48),
                3.2507452250100000e-04 * numpy.ones(48),
                3.3551534159399998e-04 * numpy.ones(48),
                3.4358475685499997e-04 * numpy.ones(48),
                3.4957868316200003e-04 * numpy.ones(48),
                3.5377678055299998e-04 * numpy.ones(48),
                3.5644598154200002e-04 * numpy.ones(48),
                3.5784640612300002e-04 * numpy.ones(48),
                3.2397487628399999e-04 * numpy.ones(48),
                3.3454917841699999e-04 * numpy.ones(48),
                3.4291261772999998e-04 * numpy.ones(48),
                3.4924203430999998e-04 * numpy.ones(48),
                3.5373990502400002e-04 * numpy.ones(48),
                3.5662091526600002e-04 * numpy.ones(48),
                3.5810843219200001e-04 * numpy.ones(48),
                3.4265221175900002e-04 * numpy.ones(48),
                3.4918487701200002e-04 * numpy.ones(48),
                3.5393182352299999e-04 * numpy.ones(48),
                3.5702314384599999e-04 * numpy.ones(48),
                3.5862073350499999e-04 * numpy.ones(48),
                3.5411962051599998e-04 * numpy.ones(48),
                3.5742969115700000e-04 * numpy.ones(48),
                3.5919932798200000e-04 * numpy.ones(48),
                3.5958550346600000e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(5.5911052220582243e-02, 9.9843575368652904e-01),
                _pq0(1.4073840785139161e-01, 9.9004681735534883e-01),
                _pq0(2.3640354389763085e-01, 9.7165496161582010e-01),
                _pq0(3.3606027378181719e-01, 9.4184048139039456e-01),
                _pq0(4.3562926300546645e-01, 9.0012618293954438e-01),
                _pq0(5.3215694152561743e-01, 8.4664572849929420e-01),
                _pq0(6.2329563050405545e-01, 7.8198628951827021e-01),
                _llm(1.8861085187235022e-02, 9.9964419616737621e-01),
                _llm(4.8002172446252683e-02, 9.9769313061726572e-01),
                _llm(8.2449220583972208e-02, 9.9317886206372263e-01),
                _llm(1.2004083624840230e-01, 9.8548485288489773e-01),
                _llm(1.5957735308099649e-01, 9.7420230792547702e-01),
                _llm(2.0026359734340657e-01, 9.5905629822141092e-01),
                _llm(2.4151275901399821e-01, 9.3986338074578313e-01),
                _llm(2.8285841584584753e-01, 9.1650544633971220e-01),
                _llm(3.2390910153381375e-01, 8.8891269981203158e-01),
                _llm(3.6432250979621944e-01, 8.5705205075979329e-01),
                _llm(4.0378970836918021e-01, 8.2091884058673226e-01),
                _llm(4.4202475151941262e-01, 7.8053074128339306e-01),
                _llm(4.7875725384649387e-01, 7.3592321867075772e-01),
                _llm(5.1372652512752337e-01, 6.8714635614314379e-01),
                _llm(5.4667640566546116e-01, 6.3426320638784062e-01),
                _llm(6.0548594208135365e-01, 5.1650125642026445e-01),
                _llm(6.3081067017645631e-01, 4.5183602864652178e-01),
                _llm(6.5303692301795835e-01, 3.8351734556663042e-01),
                _llm(6.7186095246111588e-01, 3.1177832047158882e-01),
                _llm(6.8696764998940152e-01, 2.3696180227217828e-01),
                _llm(6.9804670772407473e-01, 1.5956687523154703e-01),
                _llm(7.0482417212505244e-01, 8.0285570163445411e-02),
                _rsw(9.4698700868384583e-02, 2.7787483873094682e-02, 9.9511808937122770e-01),
                _rsw(1.3531703005681414e-01, 6.0765698786283642e-02, 9.8893722309741072e-01),
                _rsw(1.7716794817260753e-01, 9.7030727627110319e-02, 9.7938580551101606e-01),
                _rsw(2.1970666642317510e-01, 1.3541124585247619e-01, 9.6612254669161712e-01),
                _rsw(2.6247835573749262e-01, 1.7509964797440997e-01, 9.4891792376821504e-01),
                _rsw(3.0509695212144428e-01, 2.1548969074498028e-01, 9.2762063527545457e-01),
                _rsw(3.4722526371960200e-01, 2.5609546257401522e-01, 9.0213620384277560e-01),
                _rsw(3.8856102190263603e-01, 2.9650700506240968e-01, 8.7241270520717396e-01),
                _rsw(4.2882737760627648e-01, 3.3636414887344962e-01, 8.3843081978656042e-01),
                _rsw(4.6776624713029491e-01, 3.7534000298367892e-01, 8.0019661346812243e-01),
                _rsw(5.0513335895533595e-01, 4.1312975221442871e-01, 7.5773616615926720e-01),
                _rsw(5.4069421458104916e-01, 4.4944237760817957e-01, 7.1109163651985741e-01),
                _rsw(5.7422041225764575e-01, 4.8399389588415020e-01, 6.6031873129083796e-01),
                _rsw(1.8654070272251871e-01, 3.2591448510707971e-02, 9.8190649438312749e-01),
                _rsw(2.3211864536894303e-01, 6.8356795052973360e-02, 9.7028257896458836e-01),
                _rsw(2.7731591425238822e-01, 1.0622848644519879e-01, 9.5488815699532281e-01),
                _rsw(3.2192001922372543e-01, 1.4544044093230465e-01, 9.3552903715727231e-01),
                _rsw(3.6570325939440285e-01, 1.8540182825825097e-01, 9.1207855371497915e-01),
                _rsw(4.0843767783636226e-01, 2.2562974120147505e-01, 8.8446022138307345e-01),
                _rsw(4.4990049457514281e-01, 2.6571044250008963e-01, 8.5263562306969587e-01),
                _rsw(4.8987581413263348e-01, 3.0527554876315566e-01, 8.1659569314027347e-01),
                _rsw(5.2815474422663078e-01, 3.4398639206454223e-01, 7.7635425433581617e-01),
                _rsw(5.6453469898139919e-01, 3.8152294561219130e-01, 7.3194317786108087e-01),
                _rsw(5.9881812521598476e-01, 4.1757524209667346e-01, 6.8340893329010399e-01),
                _rsw(2.8504254244716021e-01, 3.5621495098625458e-02, 9.5785273298257989e-01),
                _rsw(3.3246194330278772e-01, 7.3303188868710803e-02, 9.4026363258237955e-01),
                _rsw(3.7858483330762821e-01, 1.1232262960084721e-01, 9.1872583008696684e-01),
                _rsw(4.2328910285621157e-01, 1.5210841933377073e-01, 8.9313457226274995e-01),
                _rsw(4.6642870508297218e-01, 1.9218444592236106e-01, 8.6342886320769752e-01),
                _rsw(5.0784584937357258e-01, 2.3213609896783033e-01, 8.2958147570328034e-01),
                _rsw(5.4737798162041784e-01, 2.7158864863605203e-01, 7.9159140417844387e-01),
                _rsw(5.8486171338113746e-01, 3.1019247075713552e-01, 7.4947808994423826e-01),
                _rsw(6.2013482815848864e-01, 3.4761210528909714e-01, 7.0327705718395450e-01),
                _rsw(3.8521911853878704e-01, 3.7632248800351008e-02, 9.2205750610383752e-01),
                _rsw(4.3250250610734225e-01, 7.6595819356371150e-02, 8.9837334258536106e-01),
                _rsw(4.7784862297344893e-01, 1.1633813060839000e-01, 8.7070438892251134e-01),
                _rsw(5.2116636930089999e-01, 1.5638905987529009e-01, 8.3900421778501266e-01),
                _rsw(5.6234695048537031e-01, 1.9633208101492003e-01, 8.0325563878764983e-01),
                _rsw(6.0127181886592462e-01, 2.3578474072587369e-01, 7.6346431211844812e-01),
                _rsw(6.3781792063901177e-01, 2.7438461212440601e-01, 7.1965365610206311e-01),
                _rsw(4.8369364602145354e-01, 3.8959026107390063e-02, 8.7436985943205781e-01),
                _rsw(5.2937925626837978e-01, 7.8712468193126370e-02, 8.4472596170810521e-01),
                _rsw(5.7262812531000329e-01, 1.1879638082029806e-01, 8.1116240667818795e-01),
                _rsw(6.1336587761690686e-01, 1.5879147080617859e-01, 7.7367148646854766e-01),
                _rsw(6.5150854918653067e-01, 1.9830585752276467e-01, 7.3226456776838722e-01),
                _rsw(5.7786927160649770e-01, 3.9772096897915492e-02, 8.1515966855660782e-01),
                _rsw(6.2079042880861923e-01, 7.9901575929811494e-02, 7.7989421184128760e-01),
                _rsw(6.6086881710468015e-01, 1.1996713087543126e-01, 7.4085106066481277e-01),
                _rsw(6.6562630894891295e-01, 4.0159559578060049e-02, 7.4520388257814552e-01),
                ])
            self.degree = 95
        elif index == 27:
            self.weights = numpy.concatenate([
                2.0403827308000001e-05 * numpy.ones(6),
                3.1781497038899998e-04 * numpy.ones(8),
                1.2317796117399999e-04 * numpy.ones(24),
                1.9246613738400001e-04 * numpy.ones(24),
                2.3808818674000000e-04 * numpy.ones(24),
                2.6931006630399998e-04 * numpy.ones(24),
                2.9086733828300002e-04 * numpy.ones(24),
                3.0539146193800002e-04 * numpy.ones(24),
                3.1439166841500002e-04 * numpy.ones(24),
                3.1870422440600002e-04 * numpy.ones(24),
                8.2881151280999997e-05 * numpy.ones(24),
                1.3608831925200000e-04 * numpy.ones(24),
                1.7668544545399999e-04 * numpy.ones(24),
                2.0831531612300000e-04 * numpy.ones(24),
                2.3332795446599999e-04 * numpy.ones(24),
                2.5328095399299998e-04 * numpy.ones(24),
                2.6924721842100000e-04 * numpy.ones(24),
                2.8199499468099998e-04 * numpy.ones(24),
                2.9209535939699998e-04 * numpy.ones(24),
                2.9998897829500002e-04 * numpy.ones(24),
                3.0602921204999999e-04 * numpy.ones(24),
                3.1051091675200000e-04 * numpy.ones(24),
                3.1369023875500002e-04 * numpy.ones(24),
                3.1579846524500002e-04 * numpy.ones(24),
                3.1705165184299998e-04 * numpy.ones(24),
                3.1765684256299998e-04 * numpy.ones(24),
                3.1771984112100000e-04 * numpy.ones(24),
                3.1755194923900002e-04 * numpy.ones(24),
                3.1746549526300000e-04 * numpy.ones(24),
                3.1756764154699998e-04 * numpy.ones(24),
                3.1789234178399999e-04 * numpy.ones(24),
                3.1837882875300000e-04 * numpy.ones(24),
                3.1887551519199999e-04 * numpy.ones(24),
                3.1919168893100000e-04 * numpy.ones(24),
                1.6352195358699999e-04 * numpy.ones(48),
                1.9681099177000001e-04 * numpy.ones(48),
                2.2367543422500001e-04 * numpy.ones(48),
                2.4531866870199998e-04 * numpy.ones(48),
                2.6275517915800000e-04 * numpy.ones(48),
                2.7676548601500000e-04 * numpy.ones(48),
                2.8794670277699998e-04 * numpy.ones(48),
                2.9676399189199998e-04 * numpy.ones(48),
                3.0359006846600003e-04 * numpy.ones(48),
                3.0873382373000001e-04 * numpy.ones(48),
                3.1246088388600001e-04 * numpy.ones(48),
                3.1500842942300001e-04 * numpy.ones(48),
                3.1659583985999999e-04 * numpy.ones(48),
                3.1743204409600000e-04 * numpy.ones(48),
                2.1821889098100000e-04 * numpy.ones(48),
                2.3997279339199999e-04 * numpy.ones(48),
                2.5797961335100001e-04 * numpy.ones(48),
                2.7271140526199998e-04 * numpy.ones(48),
                2.8463276562800001e-04 * numpy.ones(48),
                2.9414911020500002e-04 * numpy.ones(48),
                3.0160494921400001e-04 * numpy.ones(48),
                3.0729497261799998e-04 * numpy.ones(48),
                3.1147681428899998e-04 * numpy.ones(48),
                3.1438236736699998e-04 * numpy.ones(48),
                3.1622697646599998e-04 * numpy.ones(48),
                3.1721646637599999e-04 * numpy.ones(48),
                2.5545753989700000e-04 * numpy.ones(48),
                2.7017040691400002e-04 * numpy.ones(48),
                2.8236934134700002e-04 * numpy.ones(48),
                2.9228984632099998e-04 * numpy.ones(48),
                3.0018290621599999e-04 * numpy.ones(48),
                3.0628908645400000e-04 * numpy.ones(48),
                3.1083282792599998e-04 * numpy.ones(48),
                3.1402431462000002e-04 * numpy.ones(48),
                3.1606380309799999e-04 * numpy.ones(48),
                3.1714628822099999e-04 * numpy.ones(48),
                2.8123884160300002e-04 * numpy.ones(48),
                2.9121375002900003e-04 * numpy.ones(48),
                2.9932412565000002e-04 * numpy.ones(48),
                3.0571017389800000e-04 * numpy.ones(48),
                3.1053193262500003e-04 * numpy.ones(48),
                3.1395655144299999e-04 * numpy.ones(48),
                3.1615430068099998e-04 * numpy.ones(48),
                3.1729859606100002e-04 * numpy.ones(48),
                2.9894003368999999e-04 * numpy.ones(48),
                3.0545558839499998e-04 * numpy.ones(48),
                3.1047649608100002e-04 * numpy.ones(48),
                3.1410158259800001e-04 * numpy.ones(48),
                3.1645206211600000e-04 * numpy.ones(48),
                3.1766523059099998e-04 * numpy.ones(48),
                3.1050971610200001e-04 * numpy.ones(48),
                3.1430141178899999e-04 * numpy.ones(48),
                3.1681728662899999e-04 * numpy.ones(48),
                3.1814018655700000e-04 * numpy.ones(48),
                3.1706636591599999e-04 * numpy.ones(48),
                3.1854479446299998e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(5.1325376899460720e-02, 9.9868198426031918e-01),
                _pq0(1.2979946613312240e-01, 9.9154026574393661e-01),
                _pq0(2.1888520494013072e-01, 9.7575061724721290e-01),
                _pq0(3.1231748249034547e-01, 9.4997778401965416e-01),
                _pq0(4.0640376207381956e-01, 9.1369359315486409e-01),
                _pq0(4.9849583969447819e-01, 8.6689209121279742e-01),
                _pq0(5.8649750460213645e-01, 8.0995103376405841e-01),
                _pq0(6.6867116345801747e-01, 7.4355825270095766e-01),
                _llm(1.7214208329061814e-02, 9.9970362711315963e-01),
                _llm(4.4088753749817133e-02, 9.9805428889693970e-01),
                _llm(7.5946808138786973e-02, 9.9421535125296701e-01),
                _llm(1.1083353592048013e-01, 9.8763953678998051e-01),
                _llm(1.4765170543885686e-01, 9.7795600502374036e-01),
                _llm(1.8567318708606148e-01, 9.6490980676673033e-01),
                _llm(2.2436340994288209e-01, 9.4832595691439581e-01),
                _llm(2.6330068816627278e-01, 9.2808700843311798e-01),
                _llm(3.0213409049162837e-01, 9.0411834553093384e-01),
                _llm(3.4055940480300889e-01, 8.7637810538627714e-01),
                _llm(3.7830444340073743e-01, 8.4484998443896342e-01),
                _llm(4.1511947674079097e-01, 8.0953791761776284e-01),
                _llm(4.5077057664432585e-01, 7.7046205257850564e-01),
                _llm(4.8503460565731860e-01, 7.2765573084371338e-01),
                _llm(5.1769508177924695e-01, 6.8116341989507745e-01),
                _llm(5.4853842408209896e-01, 6.3103977260633504e-01),
                _llm(6.0391172389433068e-01, 5.2017425876907353e-01),
                _llm(6.2799566555731123e-01, 4.5961167095979960e-01),
                _llm(6.4936361695689504e-01, 3.9579513128046129e-01),
                _llm(6.6776441177045032e-01, 3.2890938075057818e-01),
                _llm(6.8293685721156228e-01, 2.5921901574535022e-01),
                _llm(6.9461958181841221e-01, 1.8710230652995335e-01),
                _llm(7.0257115420570249e-01, 1.1308203463032358e-01),
                _llm(7.0660047671401216e-01, 3.7835599837483046e-02),
                _rsw(8.7157387808359854e-02, 2.5571752333675844e-02, 9.9586629385325032e-01),
                _rsw(1.2483831231340088e-01, 5.6048233833766913e-02, 9.9059274743097991e-01),
                _rsw(1.6380626933833775e-01, 8.9685686019007546e-02, 9.8240723931003071e-01),
                _rsw(2.0355862033731778e-01, 1.2540866519762797e-01, 9.7099771100642507e-01),
                _rsw(2.4367989752937755e-01, 1.6247801501620140e-01, 9.5614904809682633e-01),
                _rsw(2.8382075077738067e-01, 2.0034223426832087e-01, 9.3771465307765201e-01),
                _rsw(3.2367875022176901e-01, 2.3856280262552620e-01, 9.1559808642128993e-01),
                _rsw(3.6298495548406906e-01, 2.7677311487835771e-01, 8.8974072907379020e-01),
                _rsw(4.0149480819920857e-01, 3.1465423082453087e-01, 8.6011315186625503e-01),
                _rsw(4.3898183792602252e-01, 3.5191964158950872e-01, 8.2670884344768136e-01),
                _rsw(4.7523311436743765e-01, 3.8830509840236532e-01, 7.8953950981781484e-01),
                _rsw(5.1004573183740176e-01, 4.2356134239086490e-01, 7.4863151193794786e-01),
                _rsw(5.4322383889548675e-01, 4.5744847171962200e-01, 7.0402326422997730e-01),
                _rsw(5.7457586850724429e-01, 4.8973116392555249e-01, 6.5576364523299835e-01),
                _rsw(1.7239814375928098e-01, 3.0106305978810945e-02, 9.8456715889199986e-01),
                _rsw(2.1495532578445978e-01, 6.3260315542046960e-02, 9.7457290152887865e-01),
                _rsw(2.5732560812474220e-01, 9.8485669802586329e-02, 9.6129293362989388e-01),
                _rsw(2.9931637512381054e-01, 1.3508359523842661e-01, 9.4454334462755241e-01),
                _rsw(3.4072380051480006e-01, 1.7251840554421807e-01, 9.2419948685942876e-01),
                _rsw(3.8134549784832622e-01, 2.1035592797307262e-01, 9.0018109002433488e-01),
                _rsw(4.2098481044233438e-01, 2.4822787745548602e-01, 8.7244180907999591e-01),
                _rsw(4.5945196999963006e-01, 2.8580995099828843e-01, 8.4096168710222163e-01),
                _rsw(4.9656401661859295e-01, 3.2280756599154276e-01, 8.0574155455594165e-01),
                _rsw(5.3214416555715616e-01, 3.5894599072041500e-01, 7.6679877595704182e-01),
                _rsw(5.6602084385821660e-01, 3.9396300888643093e-01, 7.2416403662925866e-01),
                _rsw(5.9802643159643631e-01, 4.2760299229490883e-01, 6.7787909548272174e-01),
                _rsw(2.6442158523507320e-01, 3.3009394290725504e-02, 9.6384210592312802e-01),
                _rsw(3.0901137434430631e-01, 6.8038876500785009e-02, 9.4862146392033209e-01),
                _rsw(3.5258710791978076e-01, 1.0443261362067081e-01, 9.2993341726229006e-01),
                _rsw(3.9504180053540278e-01, 1.4167515975176789e-01, 9.0767291737666056e-01),
                _rsw(4.3624756634301626e-01, 1.7934086105048200e-01, 8.8177373311834784e-01),
                _rsw(4.7606618121458544e-01, 2.1706307501757241e-01, 8.5219986656164015e-01),
                _rsw(5.1435510425121034e-01, 2.5451451578158080e-01, 8.1893906243822201e-01),
                _rsw(5.5097090269355975e-01, 2.9139401017066013e-01, 7.8199782302875065e-01),
                _rsw(5.8577110303294277e-01, 3.2741699109107053e-01, 7.4139755111305794e-01),
                _rsw(6.1861499174043921e-01, 3.6230813293172631e-01, 6.9717164945621735e-01),
                _rsw(3.5868945695570664e-01, 3.4973543864500155e-02, 9.3280154625642542e-01),
                _rsw(4.0352666100194412e-01, 7.1297367397570741e-02, 9.1218579207461781e-01),
                _rsw(4.4677753123325076e-01, 1.0847586201931653e-01, 8.8804438230546101e-01),
                _rsw(4.8836383466085442e-01, 1.4609156892417724e-01, 8.6032436818017544e-01),
                _rsw(5.2819083484345997e-01, 1.8377908323699801e-01, 8.2899921022395273e-01),
                _rsw(5.6615426871493102e-01, 2.2120753908740234e-01, 7.9406332786923051e-01),
                _rsw(6.0214501020314504e-01, 2.5806828411609861e-01, 7.5552772776439314e-01),
                _rsw(6.3605207836100486e-01, 2.9406563620941212e-01, 7.1341653696378504e-01),
                _rsw(4.5216110650871971e-01, 3.6310553658669455e-02, 8.9119687917643153e-01),
                _rsw(4.9593656515609619e-01, 7.3483184684843203e-02, 8.6524397999046854e-01),
                _rsw(5.3768158040382807e-01, 1.1110876438126471e-01, 8.3579504698945806e-01),
                _rsw(5.7733144802437664e-01, 1.4882260851454088e-01, 8.0283262908093378e-01),
                _rsw(6.1481132455750565e-01, 1.8628922741351497e-01, 7.6635720062221713e-01),
                _rsw(6.5004074628423802e-01, 2.2319097017144579e-01, 7.2638338293504467e-01),
                _rsw(5.4251514487072139e-01, 3.7182013061188894e-02, 8.3922274486015791e-01),
                _rsw(5.8418605569079285e-01, 7.4836163350673432e-02, 8.0816223680111721e-01),
                _rsw(6.2346321868514998e-01, 1.1259908342661221e-01, 7.7370217872139901e-01),
                _rsw(6.6029345518488436e-01, 1.5013038131576184e-01, 7.3584877634334023e-01),
                _rsw(6.2785739683751063e-01, 3.7675599302456363e-02, 7.7741600089889162e-01),
                _rsw(6.6656117112645774e-01, 7.5484433013601651e-02, 7.4161870628992377e-01),
                ])
            self.degree = 101
        elif index == 28:
            self.weights = numpy.concatenate([
                1.8073952522000000e-05 * numpy.ones(6),
                2.8480087822400002e-04 * numpy.ones(12),
                2.8360658375299998e-04 * numpy.ones(8),
                1.0511934069700000e-04 * numpy.ones(24),
                1.6578718387999999e-04 * numpy.ones(24),
                2.0646481137099999e-04 * numpy.ones(24),
                2.3479427458199999e-04 * numpy.ones(24),
                2.5477753266000001e-04 * numpy.ones(24),
                2.6868766848499998e-04 * numpy.ones(24),
                2.7786657555199998e-04 * numpy.ones(24),
                2.8309966167800002e-04 * numpy.ones(24),
                7.0131492667000007e-05 * numpy.ones(24),
                1.1627980219600001e-04 * numpy.ones(24),
                1.5187285839699999e-04 * numpy.ones(24),
                1.7987961082200000e-04 * numpy.ones(24),
                2.0225933859700001e-04 * numpy.ones(24),
                2.2030931055799999e-04 * numpy.ones(24),
                2.3492942343000001e-04 * numpy.ones(24),
                2.4676820587500001e-04 * numpy.ones(24),
                2.5630926835700001e-04 * numpy.ones(24),
                2.6392538967600000e-04 * numpy.ones(24),
                2.6991374792700000e-04 * numpy.ones(24),
                2.7451964201699998e-04 * numpy.ones(24),
                2.7795291974000002e-04 * numpy.ones(24),
                2.8039960866799998e-04 * numpy.ones(24),
                2.8203023567199999e-04 * numpy.ones(24),
                2.8300567474900002e-04 * numpy.ones(24),
                2.8348089507800002e-04 * numpy.ones(24),
                2.8352823390800002e-04 * numpy.ones(24),
                2.8338192670700001e-04 * numpy.ones(24),
                2.8328583369100001e-04 * numpy.ones(24),
                2.8332682354500002e-04 * numpy.ones(24),
                2.8354326770299999e-04 * numpy.ones(24),
                2.8390917227400002e-04 * numpy.ones(24),
                2.8433081788800001e-04 * numpy.ones(24),
                2.8467035505300002e-04 * numpy.ones(24),
                1.4030633401699999e-04 * numpy.ones(48),
                1.6965041259400001e-04 * numpy.ones(48),
                1.9357872427500000e-04 * numpy.ones(48),
                2.1306145105199999e-04 * numpy.ones(48),
                2.2893812659300001e-04 * numpy.ones(48),
                2.4186302928199999e-04 * numpy.ones(48),
                2.5234004956300001e-04 * numpy.ones(48),
                2.6076239734500000e-04 * numpy.ones(48),
                2.6744410326900000e-04 * numpy.ones(48),
                2.7264323603400000e-04 * numpy.ones(48),
                2.7657876859199997e-04 * numpy.ones(48),
                2.7944286906399999e-04 * numpy.ones(48),
                2.8140990020600001e-04 * numpy.ones(48),
                2.8264295315800001e-04 * numpy.ones(48),
                2.8329835425499998e-04 * numpy.ones(48),
                1.8866955652799999e-04 * numpy.ones(48),
                2.0818678827500000e-04 * numpy.ones(48),
                2.2451486806000001e-04 * numpy.ones(48),
                2.3803704915100000e-04 * numpy.ones(48),
                2.4913980418500001e-04 * numpy.ones(48),
                2.5816324058799999e-04 * numpy.ones(48),
                2.6539655062299999e-04 * numpy.ones(48),
                2.7108572167499999e-04 * numpy.ones(48),
                2.7544340938999998e-04 * numpy.ones(48),
                2.7865799325199999e-04 * numpy.ones(48),
                2.8090110806799998e-04 * numpy.ones(48),
                2.8233361845600002e-04 * numpy.ones(48),
                2.8311011758100000e-04 * numpy.ones(48),
                2.2216799703500000e-04 * numpy.ones(48),
                2.3561857342700000e-04 * numpy.ones(48),
                2.4692283448099998e-04 * numpy.ones(48),
                2.5627263486400001e-04 * numpy.ones(48),
                2.6387567267500002e-04 * numpy.ones(48),
                2.6993111573899998e-04 * numpy.ones(48),
                2.7462332683999999e-04 * numpy.ones(48),
                2.7812256744500001e-04 * numpy.ones(48),
                2.8058812540499999e-04 * numpy.ones(48),
                2.8217198769999997e-04 * numpy.ones(48),
                2.8302225023300003e-04 * numpy.ones(48),
                2.4579959567399998e-04 * numpy.ones(48),
                2.5514744074999998e-04 * numpy.ones(48),
                2.6290653352000002e-04 * numpy.ones(48),
                2.6919004499299998e-04 * numpy.ones(48),
                2.7412754857499997e-04 * numpy.ones(48),
                2.7785309701199999e-04 * numpy.ones(48),
                2.8050105676499998e-04 * numpy.ones(48),
                2.8220558340300001e-04 * numpy.ones(48),
                2.8310169012400001e-04 * numpy.ones(48),
                2.6244749011300001e-04 * numpy.ones(48),
                2.6880341630399997e-04 * numpy.ones(48),
                2.7389327512899997e-04 * numpy.ones(48),
                2.7779447912400001e-04 * numpy.ones(48),
                2.8060116616600002e-04 * numpy.ones(48),
                2.8241814566000000e-04 * numpy.ones(48),
                2.8335852165799999e-04 * numpy.ones(48),
                2.7381652369600000e-04 * numpy.ones(48),
                2.7783652081999998e-04 * numpy.ones(48),
                2.8078529404199998e-04 * numpy.ones(48),
                2.8272459496700002e-04 * numpy.ones(48),
                2.8373423448300002e-04 * numpy.ones(48),
                2.8092339076099998e-04 * numpy.ones(48),
                2.8299308097400001e-04 * numpy.ones(48),
                2.8410978741100001e-04 * numpy.ones(48),
                2.8434552060100000e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(4.7322243871801256e-02, 9.9887967505347597e-01),
                _pq0(1.2021005293268024e-01, 9.9274847931080823e-01),
                _pq0(2.0343048206648551e-01, 9.7908939273500317e-01),
                _pq0(2.9122856435730010e-01, 9.5665350221612944e-01),
                _pq0(3.8023617927267678e-01, 9.2488942472714908e-01),
                _pq0(4.6805985110561454e-01, 8.8369676687367704e-01),
                _pq0(5.5281510521555988e-01, 8.3330394181565559e-01),
                _pq0(6.3293863078030410e-01, 7.7420196955701026e-01),
                _llm(1.5878764198584534e-02, 9.9974783305344128e-01),
                _llm(4.0691935937511844e-02, 9.9834279318243935e-01),
                _llm(7.0258881152579944e-02, 9.9505144552348412e-01),
                _llm(1.0274954500287072e-01, 9.8938620467611438e-01),
                _llm(1.3714577308934250e-01, 9.8101074094397822e-01),
                _llm(1.7277585326719519e-01, 9.6968913011108115e-01),
                _llm(2.0914920389290370e-01, 9.5525557890123269e-01),
                _llm(2.4588132817519159e-01, 9.3759519245226908e-01),
                _llm(2.8265458594500664e-01, 9.1663120724122948e-01),
                _llm(3.1919572917996214e-01, 8.9231618440244853e-01),
                _llm(3.5526214692995789e-01, 8.6462570741184541e-01),
                _llm(3.9063295034062295e-01, 8.3355371525556810e-01),
                _llm(4.2510286140930303e-01, 7.9910895029604423e-01),
                _llm(4.5847775201118707e-01, 7.6131222361232120e-01),
                _llm(4.9057113587101925e-01, 7.2019436355780775e-01),
                _llm(5.2120116698473840e-01, 6.7579485575690323e-01),
                _llm(5.5018784887379957e-01, 6.2816133429497434e-01),
                _llm(6.0250378774793423e-01, 5.2342943316055923e-01),
                _llm(6.2545726895490150e-01, 4.6648302158058413e-01),
                _llm(6.4601071795282483e-01, 4.0662059045276039e-01),
                _llm(6.6395411381542502e-01, 3.4399108926707239e-01),
                _llm(6.7906885156674945e-01, 2.7880277915335056e-01),
                _llm(6.9113385803715133e-01, 2.1134800815093080e-01),
                _llm(6.9993859561264915e-01, 1.4202790128557530e-01),
                _llm(7.0530377486568951e-01, 7.1366450944524332e-02),
                _rsw(8.0565166513690709e-02, 2.3634546840031231e-02, 9.9646909743367740e-01),
                _rsw(1.1564760771393899e-01, 5.1912916325459406e-02, 9.9193280011734097e-01),
                _rsw(1.5204733827604247e-01, 8.3227157369945334e-02, 9.8486285705132015e-01),
                _rsw(1.8929866997459308e-01, 1.1658556679937120e-01, 9.7497375306206036e-01),
                _rsw(2.2701944467777907e-01, 1.5130771674095042e-01, 9.6206452309230961e-01),
                _rsw(2.6489081850932739e-01, 1.8688820258078601e-01, 9.4599453169961911e-01),
                _rsw(3.0263892595741354e-01, 2.2292776297762232e-01, 9.2666762811114545e-01),
                _rsw(3.4002202961513850e-01, 2.5909518407462356e-01, 9.0402140736032288e-01),
                _rsw(3.7682179533355092e-01, 2.9510472917508468e-01, 8.7801966571375800e-01),
                _rsw(4.1283729009218850e-01, 3.3070197141699298e-01, 8.4864691009291526e-01),
                _rsw(4.4788071318156303e-01, 3.6565441010876326e-01, 8.1590423404219858e-01),
                _rsw(4.8177420340892568e-01, 3.9974489519396944e-01, 7.7980615264055042e-01),
                _rsw(5.1434728146533426e-01, 4.3276671108120229e-01, 7.4037817892559432e-01),
                _rsw(5.4543462139056498e-01, 4.6451961235322931e-01, 6.9765507489570844e-01),
                _rsw(5.7487393131702524e-01, 4.9480635557033442e-01, 6.5167985512773974e-01),
                _rsw(1.5995987382863380e-01, 2.7923575900489703e-02, 9.8672848984594397e-01),
                _rsw(1.9980974125009515e-01, 5.8771410381390644e-02, 9.7807054378677216e-01),
                _rsw(2.3962289525662037e-01, 9.1645739146913860e-02, 9.6653087201860788e-01),
                _rsw(2.7922283410977466e-01, 1.2590496419626868e-01, 9.5193621052171429e-01),
                _rsw(3.1842511075467411e-01, 1.6105948234008624e-01, 9.3416555919666455e-01),
                _rsw(3.5704811644262446e-01, 1.9671516534608988e-01, 9.1313733154856536e-01),
                _rsw(3.9491647104921440e-01, 2.3254046061751674e-01, 8.8880026725346351e-01),
                _rsw(4.3186172939705036e-01, 2.6824611411514387e-01, 8.6112685995985172e-01),
                _rsw(4.6772210099316791e-01, 3.0357201160119734e-01, 8.3010846882497191e-01),
                _rsw(5.0234179392709555e-01, 3.3827818591974385e-01, 7.9575158875427621e-01),
                _rsw(5.3557018366361275e-01, 3.7213830656259417e-01, 7.5807496935280005e-01),
                _rsw(5.6726084513287711e-01, 4.0493463604660540e-01, 7.1710743554152134e-01),
                _rsw(5.9727042025401622e-01, 4.3645380986338006e-01, 6.7288640716344639e-01),
                _rsw(2.4616870223335963e-01, 3.0704231668333675e-02, 9.6874053295936824e-01),
                _rsw(2.8817745662868338e-01, 6.3380346692818820e-02, 9.5547720283851956e-01),
                _rsw(3.2939636041169773e-01, 9.7428624870679353e-02, 9.3915158563638457e-01),
                _rsw(3.6973038222413762e-01, 1.3237995322822896e-01, 9.1966025924994987e-01),
                _rsw(4.0906630231351265e-01, 1.6784970181293360e-01, 8.9693435540894473e-01),
                _rsw(4.4728193554117113e-01, 2.0350951053261138e-01, 8.7093211518540325e-01),
                _rsw(4.8425133772314360e-01, 2.3906925666720896e-01, 8.4163325292548541e-01),
                _rsw(5.1984776299629265e-01, 2.7426498180761477e-01, 8.0903462414276139e-01),
                _rsw(5.5394530118831464e-01, 3.0885038065800935e-01, 7.7314684611578810e-01),
                _rsw(5.8641967624012514e-01, 3.4259042459066125e-01, 7.3399166500527591e-01),
                _rsw(6.1714844666683899e-01, 3.7525622947894705e-01, 6.9159999784120285e-01),
                _rsw(3.3503378305657266e-01, 3.2615899346347420e-02, 9.4164142183775579e-01),
                _rsw(3.7757732247582843e-01, 6.6584389280815670e-02, 9.2358101142026794e-01),
                _rsw(4.1881552298489733e-01, 1.0145657971579546e-01, 9.0238579340504876e-01),
                _rsw(4.5868058920093446e-01, 1.3685733208438222e-01, 8.7799896796353283e-01),
                _rsw(4.9708957142242349e-01, 1.7246148519516083e-01, 8.5038755524016851e-01),
                _rsw(5.3395051339607469e-01, 2.0797793814164134e-01, 8.1953769070764515e-01),
                _rsw(5.6916657925314385e-01, 2.4313857883222867e-01, 7.8545084922273700e-01),
                _rsw(6.0263876826803764e-01, 2.7769018830498532e-01, 7.4814081181253855e-01),
                _rsw(6.3426761501633055e-01, 3.1138813563866308e-01, 7.0763127511789181e-01),
                _rsw(4.2379511195370678e-01, 3.3948778486643502e-02, 9.0512164018070518e-01),
                _rsw(4.6569186832349285e-01, 6.8802195562914448e-02, 8.8226829347035762e-01),
                _rsw(5.0588570691859802e-01, 1.0419468597216339e-01, 8.5628448482419239e-01),
                _rsw(5.4432046667139977e-01, 1.3980397387363919e-01, 8.2714574196556701e-01),
                _rsw(5.8092988137597412e-01, 1.7533733811961566e-01, 7.9484419277341656e-01),
                _rsw(6.1564160394471279e-01, 2.1052157935140109e-01, 7.5938533046127055e-01),
                _rsw(6.4838013510666026e-01, 2.4509533121570506e-01, 7.2078532103208992e-01),
                _rsw(5.1036165772516884e-01, 3.4855606438006559e-02, 8.5925320192819554e-01),
                _rsw(5.5067387925806810e-01, 7.0263086315120349e-02, 8.3175800411197998e-01),
                _rsw(5.8895730409952907e-01, 1.0590350612964014e-01, 8.0119519552807117e-01),
                _rsw(6.2516415895169308e-01, 1.4148239252360281e-01, 7.6756270556093298e-01),
                _rsw(6.5924149215701777e-01, 1.7672079082145309e-01, 7.3086963072084810e-01),
                _rsw(5.9303140175333824e-01, 3.5421893395615536e-02, 8.0439980482515061e-01),
                _rsw(6.3098122533901735e-01, 7.1095740403695873e-02, 7.7253355200024965e-01),
                _rsw(6.6662960113532310e-01, 1.0672597922827311e-01, 7.3770898072880087e-01),
                _rsw(6.7037152710499215e-01, 3.5694552688207613e-02, 7.4116659028541587e-01),
                ])
            self.degree = 107
        elif index == 29:
            self.weights = numpy.concatenate([
                1.4490630225000000e-05 * numpy.ones(6),
                2.5463773298299997e-04 * numpy.ones(8),
                9.0413396951000000e-05 * numpy.ones(24),
                1.4384263300800000e-04 * numpy.ones(24),
                1.8025230898200000e-04 * numpy.ones(24),
                2.0600522905700000e-04 * numpy.ones(24),
                2.2450022489700001e-04 * numpy.ones(24),
                2.3770598477299999e-04 * numpy.ones(24),
                2.4681189558800002e-04 * numpy.ones(24),
                2.5254108729699998e-04 * numpy.ones(24),
                2.5531014099300001e-04 * numpy.ones(24),
                6.0184329610999998e-05 * numpy.ones(24),
                1.0022865832600000e-04 * numpy.ones(24),
                1.3152229310300001e-04 * numpy.ones(24),
                1.5642137468800001e-04 * numpy.ones(24),
                1.7651188415099999e-04 * numpy.ones(24),
                1.9287370993100001e-04 * numpy.ones(24),
                2.0626585342600000e-04 * numpy.ones(24),
                2.1723954459499999e-04 * numpy.ones(24),
                2.2620761888800001e-04 * numpy.ones(24),
                2.3348856994599999e-04 * numpy.ones(24),
                2.3933552731800000e-04 * numpy.ones(24),
                2.4395592004700000e-04 * numpy.ones(24),
                2.4752518660599998e-04 * numpy.ones(24),
                2.5019655581600000e-04 * numpy.ones(24),
                2.5210814079300000e-04 * numpy.ones(24),
                2.5338810023900000e-04 * numpy.ones(24),
                2.5415829008500001e-04 * numpy.ones(24),
                2.5453657375300000e-04 * numpy.ones(24),
                2.5457269930700000e-04 * numpy.ones(24),
                2.5444561974699998e-04 * numpy.ones(24),
                2.5434815968800002e-04 * numpy.ones(24),
                2.5435064514299998e-04 * numpy.ones(24),
                2.5449056754900000e-04 * numpy.ones(24),
                2.5476114073400002e-04 * numpy.ones(24),
                2.5510603754499999e-04 * numpy.ones(24),
                2.5542919338200003e-04 * numpy.ones(24),
                2.5562557106900002e-04 * numpy.ones(24),
                1.2128797336700000e-04 * numpy.ones(48),
                1.4728728812700000e-04 * numpy.ones(48),
                1.6868466010100001e-04 * numpy.ones(48),
                1.8626984146599999e-04 * numpy.ones(48),
                2.0074309569899999e-04 * numpy.ones(48),
                2.1265681253900001e-04 * numpy.ones(48),
                2.2243946033699999e-04 * numpy.ones(48),
                2.3042645226699999e-04 * numpy.ones(48),
                2.3688542884199999e-04 * numpy.ones(48),
                2.4203520894600000e-04 * numpy.ones(48),
                2.4605971130799999e-04 * numpy.ones(48),
                2.4911819122599998e-04 * numpy.ones(48),
                2.5135281942100001e-04 * numpy.ones(48),
                2.5289430966900000e-04 * numpy.ones(48),
                2.5386603684899998e-04 * numpy.ones(48),
                2.5438686482999999e-04 * numpy.ones(48),
                1.6425955378299999e-04 * numpy.ones(48),
                1.8182466598500000e-04 * numpy.ones(48),
                1.9665656494900001e-04 * numpy.ones(48),
                2.0906779056599999e-04 * numpy.ones(48),
                2.1938204095100000e-04 * numpy.ones(48),
                2.2788708276599999e-04 * numpy.ones(48),
                2.3482831922799999e-04 * numpy.ones(48),
                2.4041397555800000e-04 * numpy.ones(48),
                2.4482274077600001e-04 * numpy.ones(48),
                2.4821104555900001e-04 * numpy.ones(48),
                2.5071923977700001e-04 * numpy.ones(48),
                2.5247659685299998e-04 * numpy.ones(48),
                2.5360523885400001e-04 * numpy.ones(48),
                2.5422305880299999e-04 * numpy.ones(48),
                1.9448170130500000e-04 * numpy.ones(48),
                2.0678623627500001e-04 * numpy.ones(48),
                2.1724407346499999e-04 * numpy.ones(48),
                2.2601259917200001e-04 * numpy.ones(48),
                2.3326550086900000e-04 * numpy.ones(48),
                2.3916996815300000e-04 * numpy.ones(48),
                2.4388015282700000e-04 * numpy.ones(48),
                2.4753705042600000e-04 * numpy.ones(48),
                2.5027072356399998e-04 * numpy.ones(48),
                2.5220317010500002e-04 * numpy.ones(48),
                2.5345112699799998e-04 * numpy.ones(48),
                2.5412849149599999e-04 * numpy.ones(48),
                2.1615092506899999e-04 * numpy.ones(48),
                2.2487785134400000e-04 * numpy.ones(48),
                2.3223888033999999e-04 * numpy.ones(48),
                2.3832654710000001e-04 * numpy.ones(48),
                2.4324766750200000e-04 * numpy.ones(48),
                2.4711222237499999e-04 * numpy.ones(48),
                2.5002917524900002e-04 * numpy.ones(48),
                2.5210559427600002e-04 * numpy.ones(48),
                2.5344727855799998e-04 * numpy.ones(48),
                2.5415997130800002e-04 * numpy.ones(48),
                2.3173809758599999e-04 * numpy.ones(48),
                2.3785507337200000e-04 * numpy.ones(48),
                2.4288844567400001e-04 * numpy.ones(48),
                2.4690026557599998e-04 * numpy.ones(48),
                2.4996575742700002e-04 * numpy.ones(48),
                2.5216761684900002e-04 * numpy.ones(48),
                2.5359356626499999e-04 * numpy.ones(48),
                2.5433567433599997e-04 * numpy.ones(48),
                2.4273532851999999e-04 * numpy.ones(48),
                2.4682580397400000e-04 * numpy.ones(48),
                2.5000609564399998e-04 * numpy.ones(48),
                2.5232383654199999e-04 * numpy.ones(48),
                2.5383992602500001e-04 * numpy.ones(48),
                2.5462559272699999e-04 * numpy.ones(48),
                2.5005833600500002e-04 * numpy.ones(48),
                2.5247776382599998e-04 * numpy.ones(48),
                2.5409511938600003e-04 * numpy.ones(48),
                2.5495240850300000e-04 * numpy.ones(48),
                2.5425695070099998e-04 * numpy.ones(48),
                2.5521141275800001e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(4.3822235011311141e-02, 9.9903934442974440e-01),
                _pq0(1.1174740774000069e-01, 9.9373664361509284e-01),
                _pq0(1.8971532529114390e-01, 9.8183913924312238e-01),
                _pq0(2.7240230099103302e-01, 9.6218344738141837e-01),
                _pq0(3.5671633087099031e-01, 9.3421274840902180e-01),
                _pq0(4.4047844830280863e-01, 8.9776318513333453e-01),
                _pq0(5.2198331541614118e-01, 8.5295569546558125e-01),
                _pq0(5.9981798689775534e-01, 8.0013647747988859e-01),
                _pq0(6.7278031545482231e-01, 7.3984231234534692e-01),
                _llm(1.4628961518310972e-02, 9.9978597058059959e-01),
                _llm(3.7698408124931658e-02, 9.9857781872505669e-01),
                _llm(6.5247019040968607e-02, 9.9573372595917176e-01),
                _llm(9.5605434161346592e-02, 9.9081744126637217e-01),
                _llm(1.2783358989291987e-01, 9.8352282464118623e-01),
                _llm(1.6130961044660314e-01, 9.7363156232485104e-01),
                _llm(1.9558062257453715e-01, 9.6098722163549755e-01),
                _llm(2.3029352184980276e-01, 9.4547860239564852e-01),
                _llm(2.6515843441130260e-01, 9.2702859142590299e-01),
                _llm(2.9992768251832075e-01, 9.0558642355016494e-01),
                _llm(3.3438286697187986e-01, 8.8112212351712760e-01),
                _llm(3.6832650137505191e-01, 8.5362238534941659e-01),
                _llm(4.0157632065181081e-01, 8.2308742997539941e-01),
                _llm(4.3396120263997712e-01, 7.8952856136211413e-01),
                _llm(4.6531806511145812e-01, 7.5296626522166155e-01),
                _llm(4.9548933310808019e-01, 7.1342879220859867e-01),
                _llm(5.2432070689249299e-01, 6.7095125951704782e-01),
                _llm(5.5165904790417064e-01, 6.2557540691025271e-01),
                _llm(6.0123719278041765e-01, 5.2633418664860232e-01),
                _llm(6.2315744664498174e-01, 4.7259876574300658e-01),
                _llm(6.4294165141812698e-01, 4.1623558923218357e-01),
                _llm(6.6041242729435945e-01, 3.5736655096741304e-01),
                _llm(6.7538514704082486e-01, 2.9615841421996880e-01),
                _llm(6.8767179706261627e-01, 2.3284114552489299e-01),
                _llm(6.9708950613192366e-01, 1.6772728126784311e-01),
                _llm(7.0347469125533091e-01, 1.0122607137705887e-01),
                _llm(7.0670172175422941e-01, 3.3843063384024721e-02),
                _rsw(7.4765639431660771e-02, 2.1931685094611856e-02, 9.9695992915928455e-01),
                _rsw(1.0753414820014159e-01, 4.8264192815338849e-02, 9.9302919124402067e-01),
                _rsw(1.4163448852032584e-01, 7.7511918835757379e-02, 9.8687971612551884e-01),
                _rsw(1.7663253153885849e-01, 1.0875581392476789e-01, 9.7825003027842294e-01),
                _rsw(2.1217441744815146e-01, 1.4136613742530962e-01, 9.6695275570732309e-01),
                _rsw(2.4796694434081465e-01, 1.7487682142588801e-01, 9.5285386699234076e-01),
                _rsw(2.8376004522941128e-01, 2.0892164066120730e-01, 9.3585895560967514e-01),
                _rsw(3.1933449331939828e-01, 2.4319876855459707e-01, 9.1590383793822516e-01),
                _rsw(3.5449354424387447e-01, 2.7744970543777686e-01, 8.9294792011735324e-01),
                _rsw(3.8905719322881527e-01, 3.1144603561569156e-01, 8.6696935776080675e-01),
                _rsw(4.2285812142590917e-01, 3.4498068519130126e-01, 8.3796141676636371e-01),
                _rsw(4.5573872113040520e-01, 3.7786186412482559e-01, 8.0592966796274146e-01),
                _rsw(4.8754879505416443e-01, 4.0990863916989778e-01, 7.7088979755547093e-01),
                _rsw(5.1814365299629972e-01, 4.4094749258539717e-01, 7.3286592473814827e-01),
                _rsw(5.4738240956006612e-01, 4.7080945177112921e-01, 6.9188941155880168e-01),
                _rsw(5.7512633989761741e-01, 4.9932751403546372e-01, 6.4799824604942724e-01),
                _rsw(1.4895157468400283e-01, 2.5993819932670203e-02, 9.8850278184963336e-01),
                _rsw(1.8636564443517664e-01, 5.4792865324621902e-02, 9.8095136907176961e-01),
                _rsw(2.2386028803563493e-01, 8.5567632514252603e-02, 9.7085774020013271e-01),
                _rsw(2.6127233757281593e-01, 1.1772578022670102e-01, 9.5805918725789119e-01),
                _rsw(2.9843329902061888e-01, 1.5081684561926995e-01, 9.4243930579805679e-01),
                _rsw(3.3517865846633332e-01, 1.8448018921777273e-01, 9.2391683970738614e-01),
                _rsw(3.7135055222091201e-01, 2.1841452360875982e-01, 9.0243773372016212e-01),
                _rsw(4.0679810989546628e-01, 2.5235906414862286e-01, 8.7796936195262953e-01),
                _rsw(4.4137699936875352e-01, 2.8608129769013729e-01, 8.5049622899820221e-01),
                _rsw(4.7494871825163942e-01, 3.1936867578089961e-01, 8.2001668517236859e-01),
                _rsw(5.0737981050754244e-01, 3.5202269495476024e-01, 7.8654036776640912e-01),
                _rsw(5.3854104488786536e-01, 3.8385443956678905e-01, 7.5008620317664321e-01),
                _rsw(5.6830653536705300e-01, 4.1468100376409628e-01, 7.1068090376504289e-01),
                _rsw(5.9655276206635088e-01, 4.4432240946811213e-01, 6.6835798679708991e-01),
                _rsw(2.2992277008561587e-01, 2.8657576640575885e-02, 9.7278685388796582e-01),
                _rsw(2.6957529985532686e-01, 5.9234216844860008e-02, 9.6115579656093564e-01),
                _rsw(3.0861787166113908e-01, 9.1178177760577198e-02, 9.4680597230457375e-01),
                _rsw(3.4696498716590762e-01, 1.2405938140826049e-01, 9.2963679335833160e-01),
                _rsw(3.8451535663196557e-01, 1.5752720582591748e-01, 9.0957853973083125e-01),
                _rsw(4.2116000334032155e-01, 1.9128451635254132e-01, 8.8658585900642173e-01),
                _rsw(4.5678678343298817e-01, 2.2507101778581717e-01, 8.6063283195204721e-01),
                _rsw(4.9128293192320610e-01, 2.5865213034409107e-01, 8.3170917769939379e-01),
                _rsw(5.2453647933038117e-01, 2.9181122428654077e-01, 7.9981728615483105e-01),
                _rsw(5.5643697889157562e-01, 3.2434392390678890e-01, 7.6496987362036895e-01),
                _rsw(5.8687576977752864e-01, 3.5605367878353511e-01, 7.2718815218122490e-01),
                _rsw(6.1574588535196184e-01, 3.8674808212425804e-01, 6.8650049209402386e-01),
                _rsw(3.1384611106721128e-01, 3.0513746375072946e-02, 9.4898341916502704e-01),
                _rsw(3.5424958720505700e-01, 6.2371112337307452e-02, 9.3306863322633116e-01),
                _rsw(3.9357515531201814e-01, 9.5162239524019004e-02, 9.1435373094327888e-01),
                _rsw(4.3176346681111466e-01, 1.2854673415085177e-01, 8.9277995377718788e-01),
                _rsw(4.6874138422508221e-01, 1.6223189316560330e-01, 8.6831004114582144e-01),
                _rsw(5.0442742370602833e-01, 1.9595811538364535e-01, 8.4092412929383165e-01),
                _rsw(5.3873540779257256e-01, 2.2948880811838371e-01, 8.1061646130520570e-01),
                _rsw(5.7157688983561050e-01, 2.6260311527139468e-01, 7.7739273398688835e-01),
                _rsw(6.0286272001361119e-01, 2.9509040752867127e-01, 7.4126796248209259e-01),
                _rsw(6.3250398126534646e-01, 3.2674584511132854e-01, 7.0226481215134939e-01),
                _rsw(3.9819867084234051e-01, 3.1832914587498762e-02, 9.1674668479807353e-01),
                _rsw(4.3827911821332988e-01, 6.4595481938809166e-02, 8.9651482879606570e-01),
                _rsw(4.7692330572181663e-01, 9.7957570370879557e-02, 8.7346921804171718e-01),
                _rsw(5.1408239111942367e-01, 1.3163072351266569e-01, 8.4758046683985588e-01),
                _rsw(5.4969778338629827e-01, 1.6535564863587043e-01, 8.1883444993747112e-01),
                _rsw(5.8370473065127260e-01, 1.9889317241265109e-01, 7.8722950489863930e-01),
                _rsw(6.1603495669268782e-01, 2.3201745814389499e-01, 7.5277409044751542e-01),
                _rsw(6.4661853532094404e-01, 2.6451065621686615e-01, 7.1548485834930142e-01),
                _rsw(4.8108351587954051e-01, 3.2759178077440158e-02, 8.7606249035138495e-01),
                _rsw(5.1999250413243425e-01, 6.6125461839671232e-02, 8.5160743241388548e-01),
                _rsw(5.5717176922074929e-01, 9.9814983314741562e-02, 8.2437648479884351e-01),
                _rsw(5.9257892508363774e-01, 1.3356870014103736e-01, 7.9436113946325071e-01),
                _rsw(6.2616585238596689e-01, 1.6714444028964626e-01, 7.6156356358876331e-01),
                _rsw(6.5788111266693305e-01, 2.0031063821560763e-01, 7.2599455219290321e-01),
                _rsw(5.6096246129981009e-01, 3.3375009402312646e-02, 8.2716819677732689e-01),
                _rsw(5.9799596599846727e-01, 6.7087503359017350e-02, 7.9868647887804756e-01),
                _rsw(6.3305237110540025e-01, 1.0087921264248501e-01, 7.6750770673294411e-01),
                _rsw(6.6609609981039741e-01, 1.3450503431717939e-01, 7.3363777271942010e-01),
                _rsw(6.3653843645858177e-01, 3.3727994607369992e-02, 7.7050713253715519e-01),
                _rsw(6.7109943028992747e-01, 6.7552493096780714e-02, 7.3828328935507148e-01),
                ])
            self.degree = 113
        elif index == 30:
            self.weights = numpy.concatenate([
                9.6875218794000004e-05 * numpy.ones(6),
                2.3078978953700000e-04 * numpy.ones(12),
                2.2973108525000001e-04 * numpy.ones(8),
                9.3122746966999993e-05 * numpy.ones(24),
                1.1999193858800000e-04 * numpy.ones(24),
                1.5980391388800000e-04 * numpy.ones(24),
                1.8222537635700001e-04 * numpy.ones(24),
                1.9885795936600001e-04 * numpy.ones(24),
                2.1126201025300001e-04 * numpy.ones(24),
                2.2015948877000000e-04 * numpy.ones(24),
                2.2616225908999999e-04 * numpy.ones(24),
                2.2964584534399999e-04 * numpy.ones(24),
                7.3862659439999994e-05 * numpy.ones(24),
                8.2579776985000004e-05 * numpy.ones(24),
                9.7060447620999995e-05 * numpy.ones(24),
                1.3023938471200001e-04 * numpy.ones(24),
                1.5419570045999999e-04 * numpy.ones(24),
                1.7044597700899999e-04 * numpy.ones(24),
                1.8273748909400001e-04 * numpy.ones(24),
                1.9263608174399999e-04 * numpy.ones(24),
                2.0080102394900000e-04 * numpy.ones(24),
                2.0756359832100000e-04 * numpy.ones(24),
                2.1313066386899999e-04 * numpy.ones(24),
                2.1765623299400001e-04 * numpy.ones(24),
                2.2126822629899999e-04 * numpy.ones(24),
                2.2407995156699999e-04 * numpy.ones(24),
                2.2619598161900000e-04 * numpy.ones(24),
                2.2771563688100000e-04 * numpy.ones(24),
                2.2873517721300001e-04 * numpy.ones(24),
                2.2934908140800000e-04 * numpy.ones(24),
                2.2965053123800001e-04 * numpy.ones(24),
                2.2967938323200000e-04 * numpy.ones(24),
                2.2957854438400001e-04 * numpy.ones(24),
                2.2950179315299999e-04 * numpy.ones(24),
                2.2950596381800001e-04 * numpy.ones(24),
                2.2962323432399999e-04 * numpy.ones(24),
                2.2985301787399999e-04 * numpy.ones(24),
                2.3015797902800001e-04 * numpy.ones(24),
                2.3046904049999999e-04 * numpy.ones(24),
                2.3070279959099999e-04 * numpy.ones(24),
                1.0060069902700000e-04 * numpy.ones(48),
                1.2276766896400000e-04 * numpy.ones(48),
                1.4678642802700001e-04 * numpy.ones(48),
                1.6441789121000001e-04 * numpy.ones(48),
                1.7776648907200001e-04 * numpy.ones(48),
                1.8848256645199999e-04 * numpy.ones(48),
                1.9732692464500000e-04 * numpy.ones(48),
                2.0467677758600000e-04 * numpy.ones(48),
                2.1076001259200000e-04 * numpy.ones(48),
                2.1574163622699999e-04 * numpy.ones(48),
                2.1975578169200000e-04 * numpy.ones(48),
                2.2291926118399999e-04 * numpy.ones(48),
                2.2533851102099999e-04 * numpy.ones(48),
                2.2711371075500000e-04 * numpy.ones(48),
                2.2834140929199999e-04 * numpy.ones(48),
                2.2911616731300001e-04 * numpy.ones(48),
                2.2953139085800001e-04 * numpy.ones(48),
                1.4382047213599999e-04 * numpy.ones(48),
                1.6077380254999999e-04 * numpy.ones(48),
                1.7414838535300000e-04 * numpy.ones(48),
                1.8519184675200000e-04 * numpy.ones(48),
                1.9446286380700001e-04 * numpy.ones(48),
                2.0224954462800000e-04 * numpy.ones(48),
                2.0874623824399999e-04 * numpy.ones(48),
                2.1410747548199999e-04 * numpy.ones(48),
                2.1846409137499999e-04 * numpy.ones(48),
                2.2193091652199999e-04 * numpy.ones(48),
                2.2461231183400001e-04 * numpy.ones(48),
                2.2660627669200001e-04 * numpy.ones(48),
                2.2800729522300000e-04 * numpy.ones(48),
                2.2890820252000001e-04 * numpy.ones(48),
                2.2940126951199999e-04 * numpy.ones(48),
                1.7224344887399999e-04 * numpy.ones(48),
                1.8302374214599999e-04 * numpy.ones(48),
                1.9238553500000000e-04 * numpy.ones(48),
                2.0040678619399999e-04 * numpy.ones(48),
                2.0718172973499999e-04 * numpy.ones(48),
                2.1282508341000000e-04 * numpy.ones(48),
                2.1745137194400000e-04 * numpy.ones(48),
                2.2116618391500000e-04 * numpy.ones(48),
                2.2406652578099999e-04 * numpy.ones(48),
                2.2624395166300000e-04 * numpy.ones(48),
                2.2778745572299999e-04 * numpy.ones(48),
                2.2878543144499999e-04 * numpy.ones(48),
                2.2932684996200000e-04 * numpy.ones(48),
                1.9126282015300000e-04 * numpy.ones(48),
                1.9924996722400001e-04 * numpy.ones(48),
                2.0612755334499999e-04 * numpy.ones(48),
                2.1193182159699999e-04 * numpy.ones(48),
                2.1674165818800001e-04 * numpy.ones(48),
                2.2064307305199999e-04 * numpy.ones(48),
                2.2371869387000001e-04 * numpy.ones(48),
                2.2604800750300000e-04 * numpy.ones(48),
                2.2770988845600000e-04 * numpy.ones(48),
                2.2878457151100001e-04 * numpy.ones(48),
                2.2935472682399999e-04 * numpy.ones(48),
                2.0560738398500000e-04 * numpy.ones(48),
                2.1142358658300000e-04 * numpy.ones(48),
                2.1631756297700000e-04 * numpy.ones(48),
                2.2033921581099999e-04 * numpy.ones(48),
                2.2354731768499999e-04 * numpy.ones(48),
                2.2600241414999999e-04 * numpy.ones(48),
                2.2776759293300001e-04 * numpy.ones(48),
                2.2891021122799999e-04 * numpy.ones(48),
                2.2950279546299999e-04 * numpy.ones(48),
                2.1612815898800001e-04 * numpy.ones(48),
                2.2019804774000000e-04 * numpy.ones(48),
                2.2349520665900001e-04 * numpy.ones(48),
                2.2605400985199999e-04 * numpy.ones(48),
                2.2791579818999999e-04 * numpy.ones(48),
                2.2912969185700000e-04 * numpy.ones(48),
                2.2975337525399999e-04 * numpy.ones(48),
                2.2349273564699999e-04 * numpy.ones(48),
                2.2612880129900000e-04 * numpy.ones(48),
                2.2808181609200001e-04 * numpy.ones(48),
                2.2937732951800000e-04 * numpy.ones(48),
                2.3005287673400000e-04 * numpy.ones(48),
                2.2818938550700000e-04 * numpy.ones(48),
                2.2957204448400000e-04 * numpy.ones(48),
                2.3032276490300000e-04 * numpy.ones(48),
                2.3048319132300000e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(4.5955576435859000e-02, 9.9894348438460112e-01),
                _pq0(1.0493167424350230e-01, 9.9447943354322588e-01),
                _pq0(1.7735488795492751e-01, 9.8414696245961919e-01),
                _pq0(2.5590714112361262e-01, 9.6670136811837570e-01),
                _pq0(3.3581568379859000e-01, 9.4192771830692268e-01),
                _pq0(4.1558357437638926e-01, 9.0955499707744125e-01),
                _pq0(4.9378942961674721e-01, 8.6958150808234613e-01),
                _pq0(5.6915696947933159e-01, 8.2222888789746573e-01),
                _pq0(6.4058408548942514e-01, 7.6788803182344034e-01),
                _llm(2.3357286088870957e-02, 9.9945428828592520e-01),
                _llm(4.3529878365506133e-02, 9.9810335105086612e-01),
                _llm(6.4392005210887837e-02, 9.9584503780951883e-01),
                _llm(9.0039436319932054e-02, 9.9185976822047672e-01),
                _llm(1.1967066155484724e-01, 9.8557489087641148e-01),
                _llm(1.5117154128381327e-01, 9.7687989548959020e-01),
                _llm(1.8359828285038024e-01, 9.6570354719695606e-01),
                _llm(2.1650812591554056e-01, 9.5197083086882495e-01),
                _llm(2.4962087204175631e-01, 9.3561682353526876e-01),
                _llm(2.8272006735679006e-01, 9.1659081766486417e-01),
                _llm(3.1561908239943459e-01, 8.9485707777872425e-01),
                _llm(3.4814767937491153e-01, 8.7039438571932870e-01),
                _llm(3.8014660869472261e-01, 8.4319458714806927e-01),
                _llm(4.1146521196340113e-01, 8.1326057244146965e-01),
                _llm(4.4195987865197506e-01, 7.8060420913793627e-01),
                _llm(4.7149259493295437e-01, 7.4524456780762793e-01),
                _llm(4.9992939728794678e-01, 7.0720661440248200e-01),
                _llm(5.2713872214312496e-01, 6.6652046872892579e-01),
                _llm(5.5298967808377608e-01, 6.2322133457191864e-01),
                _llm(6.0008560994817117e-01, 5.2895606761456349e-01),
                _llm(6.2105621927851762e-01, 4.7809867704371223e-01),
                _llm(6.4011658799342419e-01, 4.2485468992505437e-01),
                _llm(6.5711440292443324e-01, 3.6932549727649588e-01),
                _llm(6.7189108217188653e-01, 3.1164843557409849e-01),
                _llm(6.8428455910990094e-01, 2.5201048455875136e-01),
                _llm(6.9413534762698148e-01, 1.9066262966176370e-01),
                _llm(7.0129652422129929e-01, 1.2793111517629241e-01),
                _llm(7.0564714282426433e-01, 6.4220087573159779e-02),
                _rsw(7.3451338941432728e-02, 2.1778440814860446e-02, 9.9706098124607423e-01),
                _rsw(1.0098598340449304e-01, 4.5903621857751861e-02, 9.9382829938383621e-01),
                _rsw(1.3242896197487616e-01, 7.2550630956908960e-02, 9.8853375055078985e-01),
                _rsw(1.6542721096071281e-01, 1.0178254519606846e-01, 9.8095573364304778e-01),
                _rsw(1.9907671867764590e-01, 1.3256523209803631e-01, 9.7097627124422314e-01),
                _rsw(2.3301259455232792e-01, 1.6427653744967655e-01, 9.5849796558132427e-01),
                _rsw(2.6700806111082886e-01, 1.9653603743378886e-01, 9.4344066124566639e-01),
                _rsw(3.0087533762943164e-01, 2.2907267705422379e-01, 9.2574280436500656e-01),
                _rsw(3.3444755961678602e-01, 2.6166454953708235e-01, 9.0535986954466463e-01),
                _rsw(3.6757097240707859e-01, 2.9411507288431410e-01, 8.8226294501467606e-01),
                _rsw(4.0010008875878128e-01, 3.2624404009190666e-01, 8.5643723954515538e-01),
                _rsw(4.3189563504360279e-01, 3.5788353506119169e-01, 8.2788014577134772e-01),
                _rsw(4.6282390567955295e-01, 3.8887518540436772e-01, 7.9659909773250615e-01),
                _rsw(4.9275632297736360e-01, 4.1906780032228397e-01, 7.6260958877978346e-01),
                _rsw(5.2156871367079682e-01, 4.4831518368838497e-01, 7.2593358717877210e-01),
                _rsw(5.4914023469849049e-01, 4.7647406760878808e-01, 6.8659847475189084e-01),
                _rsw(5.7535201601260755e-01, 5.0340213109982768e-01, 6.4463660466527972e-01),
                _rsw(1.3883263564177542e-01, 2.4354365103727991e-02, 9.9001634541110095e-01),
                _rsw(1.7436869005372424e-01, 5.1188970573426501e-02, 9.8334899665407760e-01),
                _rsw(2.0997370379502681e-01, 8.0146950485396357e-02, 9.7441649721384016e-01),
                _rsw(2.4544925909085488e-01, 1.1051178741557000e-01, 9.6308971859010428e-01),
                _rsw(2.8072192578642768e-01, 1.4179505315709651e-01, 9.4925726928106036e-01),
                _rsw(3.1568422719758410e-01, 1.7366049457195970e-01, 9.3283733915602163e-01),
                _rsw(3.5020909451777510e-01, 2.0584663246939813e-01, 9.1377281313138692e-01),
                _rsw(3.8416848495196859e-01, 2.3812842611959187e-01, 8.9202546367439262e-01),
                _rsw(4.1743723679060174e-01, 2.7030312704225695e-01, 8.6757269024088390e-01),
                _rsw(4.4989264650118915e-01, 3.0218456830913071e-01, 8.4040531489274883e-01),
                _rsw(4.8141462298077009e-01, 3.3359933551657189e-01, 8.1052541238580833e-01),
                _rsw(5.1188636257347009e-01, 3.6438337355182315e-01, 7.7794415538025241e-01),
                _rsw(5.4119474551191438e-01, 3.9437895419581792e-01, 7.4267993639097807e-01),
                _rsw(5.6923015003572464e-01, 4.2343201444035428e-01, 7.0475695486975509e-01),
                _rsw(5.9588572041395771e-01, 4.5138979474192609e-01, 6.6420438225713363e-01),
                _rsw(2.1562702847857659e-01, 2.6812257554444861e-02, 9.7610762082586633e-01),
                _rsw(2.5323850549097093e-01, 5.5574957478056249e-02, 9.6580623493432249e-01),
                _rsw(2.9025646177715364e-01, 8.5693680629502444e-02, 9.5310428574043071e-01),
                _rsw(3.2669798231432562e-01, 1.1673674503241348e-01, 9.3789176385710193e-01),
                _rsw(3.6250396274936147e-01, 1.4838619940033043e-01, 9.2009369784741757e-01),
                _rsw(3.9758389375486985e-01, 1.8038215030114052e-01, 8.9966067340939893e-01),
                _rsw(4.3183960990097731e-01, 2.1249629656664240e-01, 8.7656139275354428e-01),
                _rsw(4.6517065557327431e-01, 2.4452218378059129e-01, 8.5077915044546137e-01),
                _rsw(4.9747526496209687e-01, 2.7627012243229865e-01, 8.2230966199001587e-01),
                _rsw(5.2865175796275166e-01, 3.0756277752113292e-01, 7.9115893263387882e-01),
                _rsw(5.5860011957318934e-01, 3.3823110898268771e-01, 7.5734115385944778e-01),
                _rsw(5.8722299020213187e-01, 3.6811088347413984e-01, 7.2087692239796053e-01),
                _rsw(6.1442586162351231e-01, 3.9703974468728370e-01, 6.8179197832393212e-01),
                _rsw(2.9516765080648605e-01, 2.8674995387504884e-02, 9.5501507975366906e-01),
                _rsw(3.3350854854727258e-01, 5.8678793419035140e-02, 9.4091915021896422e-01),
                _rsw(3.7095617606363834e-01, 8.9610992050222854e-02, 9.2431671279059857e-01),
                _rsw(4.0747228616674969e-01, 1.2116279276262988e-01, 9.0514413971256669e-01),
                _rsw(4.4299236488391153e-01, 1.5307489035548968e-01, 8.8336053941593640e-01),
                _rsw(4.7744280527217359e-01, 1.8511764367218764e-01, 8.5894110723325745e-01),
                _rsw(5.1074465395359037e-01, 2.1708291076581801e-01, 8.3187433444677639e-01),
                _rsw(5.4281513705429341e-01, 2.4877866890262701e-01, 8.0216014665636792e-01),
                _rsw(5.7356992925569639e-01, 2.8002399527950161e-01, 7.6980783207325254e-01),
                _rsw(6.0292537945628644e-01, 3.1064457028781184e-01, 7.3483395250777139e-01),
                _rsw(6.3079989870731445e-01, 3.4046895008411948e-01, 6.9726062689604029e-01),
                _rsw(3.7526522736927187e-01, 2.9971450981844765e-02, 9.2643279370579901e-01),
                _rsw(4.1353838793440278e-01, 6.0867258986779874e-02, 9.0844987670649813e-01),
                _rsw(4.5061138851539068e-01, 9.2388495484356320e-02, 8.8792665375151836e-01),
                _rsw(4.8644015546060720e-01, 1.2427866038518527e-01, 8.6482991953812749e-01),
                _rsw(5.2097080766117088e-01, 1.5630867314833877e-01, 8.3914064152767209e-01),
                _rsw(5.5414221358301230e-01, 1.8826965093885070e-01, 8.1085198751725496e-01),
                _rsw(5.8588809151138166e-01, 2.1996729791260586e-01, 7.7996764809457175e-01),
                _rsw(6.1613993906034448e-01, 2.5121654829248663e-01, 7.4649971289929085e-01),
                _rsw(6.4482964822550903e-01, 2.8183687018718889e-01, 7.1046653923492831e-01),
                _rsw(4.5447962749179494e-01, 3.0889704050602911e-02, 8.9022137380462030e-01),
                _rsw(4.9193890721466282e-01, 6.2409476776368697e-02, 8.6838998657109312e-01),
                _rsw(5.2793130269851829e-01, 9.4307061442803028e-02, 8.4403478470562388e-01),
                _rsw(5.6241699255711353e-01, 1.2635478187703719e-01, 8.1714233495751398e-01),
                _rsw(5.9534846270932873e-01, 1.5834307888225918e-01, 7.8771040193698449e-01),
                _rsw(6.2667307153391860e-01, 1.9007484625559873e-01, 7.5574626313012927e-01),
                _rsw(6.5633632042788703e-01, 2.2135995195925684e-01, 7.2126444952997482e-01),
                _rsw(5.3145747165856949e-01, 3.1525088115153456e-02, 8.4649815394814210e-01),
                _rsw(5.6746149322981854e-01, 6.3438652914656182e-02, 8.2095249010996885e-01),
                _rsw(6.0177060049702635e-01, 9.5515035042239516e-02, 7.9293695995225755e-01),
                _rsw(6.3434712702641793e-01, 1.2754400998011955e-01, 7.6245409563561317e-01),
                _rsw(6.6514945991278018e-01, 1.5932520376719581e-01, 7.2951468485718529e-01),
                _rsw(6.0501849860057044e-01, 3.1925383384960759e-02, 7.9557110697085709e-01),
                _rsw(6.3901635508804011e-01, 6.4028243539622470e-02, 7.6652363431222825e-01),
                _rsw(6.7111991070884480e-01, 9.6098050770029503e-02, 7.3509402805923607e-01),
                _rsw(6.7413544295722738e-01, 3.2118531962733059e-02, 7.3790907600694389e-01),
                ])
            self.degree = 119
        elif index == 31:
            self.weights = numpy.concatenate([
                9.0805107643000005e-05 * numpy.ones(6),
                2.0848243619900000e-04 * numpy.ones(8),
                7.5917081174000002e-05 * numpy.ones(24),
                1.0833839681700000e-04 * numpy.ones(24),
                1.4030193952899999e-04 * numpy.ones(24),
                1.6159701792900000e-04 * numpy.ones(24),
                1.7711441875000001e-04 * numpy.ones(24),
                1.8877600229900000e-04 * numpy.ones(24),
                1.9734746707699999e-04 * numpy.ones(24),
                2.0337876612299999e-04 * numpy.ones(24),
                2.0723436265200000e-04 * numpy.ones(24),
                2.0911778342300001e-04 * numpy.ones(24),
                5.0111056572000002e-05 * numpy.ones(24),
                5.9425204097000003e-05 * numpy.ones(24),
                9.5643948261000001e-05 * numpy.ones(24),
                1.1855306571300000e-04 * numpy.ones(24),
                1.3645101142300000e-04 * numpy.ones(24),
                1.5058288256099999e-04 * numpy.ones(24),
                1.6192987498700001e-04 * numpy.ones(24),
                1.7124505042700000e-04 * numpy.ones(24),
                1.7898910981599999e-04 * numpy.ones(24),
                1.8544749556299999e-04 * numpy.ones(24),
                1.9081486366700001e-04 * numpy.ones(24),
                1.9523774052799999e-04 * numpy.ones(24),
                1.9883492542799999e-04 * numpy.ones(24),
                2.0170798071600000e-04 * numpy.ones(24),
                2.0394730827100001e-04 * numpy.ones(24),
                2.0563602792900000e-04 * numpy.ones(24),
                2.0685258230700000e-04 * numpy.ones(24),
                2.0767248775299999e-04 * numpy.ones(24),
                2.0816942782399999e-04 * numpy.ones(24),
                2.0841576312199999e-04 * numpy.ones(24),
                2.0843815311300001e-04 * numpy.ones(24),
                2.0834762771299999e-04 * numpy.ones(24),
                2.0826861944600001e-04 * numpy.ones(24),
                2.0824756861100000e-04 * numpy.ones(24),
                2.0831398602900000e-04 * numpy.ones(24),
                2.0847455618299999e-04 * numpy.ones(24),
                2.0870913133800000e-04 * numpy.ones(24),
                2.0897184132999999e-04 * numpy.ones(24),
                2.0920033034800000e-04 * numpy.ones(24),
                2.0933361482600000e-04 * numpy.ones(24),
                9.3166844846999999e-05 * numpy.ones(48),
                1.1161936886800000e-04 * numpy.ones(48),
                1.2986235515600000e-04 * numpy.ones(48),
                1.4502368324599999e-04 * numpy.ones(48),
                1.5727199581499999e-04 * numpy.ones(48),
                1.6732347858700001e-04 * numpy.ones(48),
                1.7568601187299999e-04 * numpy.ones(48),
                1.8267762904400001e-04 * numpy.ones(48),
                1.8851163479900000e-04 * numpy.ones(48),
                1.9334578601700001e-04 * numpy.ones(48),
                1.9730606718999999e-04 * numpy.ones(48),
                2.0049870996199999e-04 * numpy.ones(48),
                2.0301709092799999e-04 * numpy.ones(48),
                2.0494614601200000e-04 * numpy.ones(48),
                2.0636535652000001e-04 * numpy.ones(48),
                2.0735079273800000e-04 * numpy.ones(48),
                2.0797645932599999e-04 * numpy.ones(48),
                2.0831505349700000e-04 * numpy.ones(48),
                1.2627151215900001e-04 * numpy.ones(48),
                1.4143861285500001e-04 * numpy.ones(48),
                1.5387404013100000e-04 * numpy.ones(48),
                1.6424349423299999e-04 * numpy.ones(48),
                1.7297906092400000e-04 * numpy.ones(48),
                1.8035051902599999e-04 * numpy.ones(48),
                1.8654753500799999e-04 * numpy.ones(48),
                1.9171826696799999e-04 * numpy.ones(48),
                1.9598517090300001e-04 * numpy.ones(48),
                1.9945295481200000e-04 * numpy.ones(48),
                2.0221389111500001e-04 * numpy.ones(48),
                2.0435180242100000e-04 * numpy.ones(48),
                2.0594503130199999e-04 * numpy.ones(48),
                2.0706857153200001e-04 * numpy.ones(48),
                2.0779553106900001e-04 * numpy.ones(48),
                2.0819803878200001e-04 * numpy.ones(48),
                1.5213186103800000e-04 * numpy.ones(48),
                1.6227727201899999e-04 * numpy.ones(48),
                1.7104981394200000e-04 * numpy.ones(48),
                1.7859111494500000e-04 * numpy.ones(48),
                1.8501253136900001e-04 * numpy.ones(48),
                1.9042297039299999e-04 * numpy.ones(48),
                1.9492599561200000e-04 * numpy.ones(48),
                1.9861615453600000e-04 * numpy.ones(48),
                2.0157905856399999e-04 * numpy.ones(48),
                2.0389341987100000e-04 * numpy.ones(48),
                2.0563340605400001e-04 * numpy.ones(48),
                2.0687059594599999e-04 * numpy.ones(48),
                2.0767539061100000e-04 * numpy.ones(48),
                2.0811793917299999e-04 * numpy.ones(48),
                1.7003452162300001e-04 * numpy.ones(48),
                1.7749067799900000e-04 * numpy.ones(48),
                1.8396593770000000e-04 * numpy.ones(48),
                1.8949874629800001e-04 * numpy.ones(48),
                1.9415488094500001e-04 * numpy.ones(48),
                1.9800784272499999e-04 * numpy.ones(48),
                2.0112962847400001e-04 * numpy.ones(48),
                2.0358884569699999e-04 * numpy.ones(48),
                2.0545163253500001e-04 * numpy.ones(48),
                2.0678310330899999e-04 * numpy.ones(48),
                2.0764853202800000e-04 * numpy.ones(48),
                2.0811414395300001e-04 * numpy.ones(48),
                1.8343830154700000e-04 * numpy.ones(48),
                1.8895405917800000e-04 * numpy.ones(48),
                1.9366770236000001e-04 * numpy.ones(48),
                1.9761764950700000e-04 * numpy.ones(48),
                2.0085360045599999e-04 * numpy.ones(48),
                2.0342803517100000e-04 * numpy.ones(48),
                2.0539444660299999e-04 * numpy.ones(48),
                2.0680776428800001e-04 * numpy.ones(48),
                2.0772509496600001e-04 * numpy.ones(48),
                2.0820624407099999e-04 * numpy.ones(48),
                1.9343744865500000e-04 * numpy.ones(48),
                1.9741070104800000e-04 * numpy.ones(48),
                2.0071292903900000e-04 * numpy.ones(48),
                2.0337369474699999e-04 * numpy.ones(48),
                2.0542871259000001e-04 * numpy.ones(48),
                2.0691849368200001e-04 * numpy.ones(48),
                2.0788836898100001e-04 * numpy.ones(48),
                2.0838863661200001e-04 * numpy.ones(48),
                2.0065932754700000e-04 * numpy.ones(48),
                2.0337284261400001e-04 * numpy.ones(48),
                2.0550087813800001e-04 * numpy.ones(48),
                2.0706517835200001e-04 * numpy.ones(48),
                2.0809533350900000e-04 * numpy.ones(48),
                2.0862849989900000e-04 * numpy.ones(48),
                2.0555493876400001e-04 * numpy.ones(48),
                2.0718718502699999e-04 * numpy.ones(48),
                2.0828566004299999e-04 * numpy.ones(48),
                2.0887058588200000e-04 * numpy.ones(48),
                2.0839958675400000e-04 * numpy.ones(48),
                2.0905097128900001e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a3(),
                _pq0(3.8403687078536211e-02, 9.9926230631339930e-01),
                _pq0(9.8354859541174067e-02, 9.9515140637223432e-01),
                _pq0(1.6657749476129985e-01, 9.8602836583895959e-01),
                _pq0(2.4057023353629103e-01, 9.7063173383951051e-01),
                _pq0(3.1652707701890448e-01, 9.4858347524815578e-01),
                _pq0(3.9273861456454423e-01, 9.1965014034138137e-01),
                _pq0(4.6788259183746556e-01, 8.8379063145942860e-01),
                _pq0(5.4080220242669352e-01, 8.4114979513190014e-01),
                _pq0(6.1049674457524383e-01, 7.9201876547404448e-01),
                _pq0(6.7609107026857374e-01, 7.3681806757373591e-01),
                _llm(2.3032616862615110e-02, 9.9946935776987178e-01),
                _llm(3.7572086201624104e-02, 9.9858734053507581e-01),
                _llm(5.8219120338218945e-02, 9.9660477023446359e-01),
                _llm(8.4031275291948343e-02, 9.9291363649897446e-01),
                _llm(1.1229277980605806e-01, 9.8730981115699257e-01),
                _llm(1.4201253191929872e-01, 9.7962486777119961e-01),
                _llm(1.7263964373419785e-01, 9.6973764845068200e-01),
                _llm(2.0381700581156947e-01, 9.5755796497340739e-01),
                _llm(2.3528498928765076e-01, 9.4301746942027542e-01),
                _llm(2.6683633543124596e-01, 9.2606519219072647e-01),
                _llm(2.9829412799004523e-01, 9.0666489201541112e-01),
                _llm(3.2950029220870763e-01, 8.8479326108913858e-01),
                _llm(3.6030949183635946e-01, 8.6043834188467505e-01),
                _llm(3.9058578951739203e-01, 8.3359791389743243e-01),
                _llm(4.2020057581608372e-01, 8.0427790729800808e-01),
                _llm(4.4903100615972269e-01, 7.7249097795014676e-01),
                _llm(4.7695861603114914e-01, 7.3825534687349326e-01),
                _llm(5.0386798870497507e-01, 7.0159397083840913e-01),
                _llm(5.2964542865199626e-01, 6.6253410464374329e-01),
                _llm(5.5417762071648491e-01, 6.2110734128170741e-01),
                _llm(5.9904673219212135e-01, 5.3130596204059455e-01),
                _llm(6.1914670962945884e-01, 4.8302660786961776e-01),
                _llm(6.3752512129018502e-01, 4.3257766868837566e-01),
                _llm(6.5405143811311683e-01, 3.8004398772290471e-01),
                _llm(6.6858990643915084e-01, 3.2553812989475567e-01),
                _llm(6.8100130096816469e-01, 2.6921080245661400e-01),
                _llm(6.9114695787303393e-01, 2.1126231383211960e-01),
                _llm(6.9889569151417352e-01, 1.5195270568782288e-01),
                _llm(7.0413357948687194e-01, 9.1606792750372312e-02),
                _llm(7.0677543980185664e-01, 3.0609726979898520e-02),
                _rsw(6.6556441202173980e-02, 1.9365088745884277e-02, 9.9759472406000904e-01),
                _rsw(9.4462461612701221e-02, 4.2524420021158423e-02, 9.9461978516814809e-01),
                _rsw(1.2426519254525055e-01, 6.8065293153543566e-02, 9.8991175252626129e-01),
                _rsw(1.5534380648467533e-01, 9.5609574912053824e-02, 9.8322281857775662e-01),
                _rsw(1.8711371105426686e-01, 1.2459316574528875e-01, 9.7440494774250164e-01),
                _rsw(2.1926126288362580e-01, 1.5453858287789787e-01, 9.6334953417789571e-01),
                _rsw(2.5156828072069537e-01, 1.8510042497233672e-01, 9.4997433271130738e-01),
                _rsw(2.8385358662872900e-01, 2.1601826082723835e-01, 9.3421798973643289e-01),
                _rsw(3.1595788175285211e-01, 2.4707990122771109e-01, 9.1603610156344561e-01),
                _rsw(3.4773708827913907e-01, 2.7810142089864015e-01, 8.9539852419428501e-01),
                _rsw(3.7905769608905399e-01, 3.0891725235157308e-01, 8.7228744931657298e-01),
                _rsw(4.0979383178101991e-01, 3.3937500554722438e-01, 8.4669570746759459e-01),
                _rsw(4.3982565728596368e-01, 3.6933224709877299e-01, 8.1862511716043806e-01),
                _rsw(4.6903841147184794e-01, 3.9865410056098777e-01, 7.8808494254735972e-01),
                _rsw(4.9732160483010518e-01, 4.2721124914085623e-01, 7.5509057071103591e-01),
                _rsw(5.2456815261324474e-01, 4.5487817353099358e-01, 7.1966249069201527e-01),
                _rsw(5.5067339118038894e-01, 4.8153153550232491e-01, 6.8182563501430737e-01),
                _rsw(5.7553398295224756e-01, 5.0704864458018539e-01, 6.4160915399994012e-01),
                _rsw(1.3054723860563652e-01, 2.2849703757223793e-02, 9.9117874751764634e-01),
                _rsw(1.6373279082164768e-01, 4.8122543382883791e-02, 9.8533029691982832e-01),
                _rsw(1.9727346341496380e-01, 7.5317344575119335e-02, 9.7745101065905993e-01),
                _rsw(2.3086946531101307e-01, 1.0390436398820177e-01, 9.6742088727255326e-01),
                _rsw(2.6438992183381599e-01, 1.3345265871176254e-01, 9.5513787335414069e-01),
                _rsw(2.9771715996221693e-01, 1.6364148689363814e-01, 9.4052429869263399e-01),
                _rsw(3.3072939030323106e-01, 1.9421954061665681e-01, 9.2352414177015851e-01),
                _rsw(3.6330691982190727e-01, 2.2497528799437527e-01, 9.0409855757067004e-01),
                _rsw(3.9533469559227269e-01, 2.5572188218200320e-01, 8.8222264618080326e-01),
                _rsw(4.2670183941849138e-01, 2.8628979252131942e-01, 8.5788326416533844e-01),
                _rsw(4.5730096225717048e-01, 3.1652245366365189e-01, 8.3107723241910991e-01),
                _rsw(4.8702795598561088e-01, 3.4627302216364958e-01, 8.0180968079097825e-01),
                _rsw(5.1578195814503225e-01, 3.7540168702828353e-01, 7.7009255614399674e-01),
                _rsw(5.4346516664653932e-01, 4.0377337849936124e-01, 7.3594338875830712e-01),
                _rsw(5.6998238877646279e-01, 4.3125577841391227e-01, 6.9938439364149951e-01),
                _rsw(5.9524033509477403e-01, 4.5771753671221083e-01, 6.6044197327423626e-01),
                _rsw(2.0251525992103694e-01, 2.5202536177195433e-02, 9.7895474955145501e-01),
                _rsw(2.3810666532744251e-01, 5.2232545061189968e-02, 9.6983347908961981e-01),
                _rsw(2.7328233836516141e-01, 8.0606696885886120e-02, 9.5855063713651367e-01),
                _rsw(3.0801376926111174e-01, 1.0993357540812557e-01, 9.4500906183144540e-01),
                _rsw(3.4224056145875997e-01, 1.3991209559598572e-01, 9.2913723614885468e-01),
                _rsw(3.7588087738904219e-01, 1.7029778016517055e-01, 9.1088541106116039e-01),
                _rsw(4.0884583834389326e-01, 2.0087992566016810e-01, 8.9022038616044086e-01),
                _rsw(4.4104505508411512e-01, 2.3147030521808354e-01, 8.6712211203964640e-01),
                _rsw(4.7238794205613122e-01, 2.6189721113758924e-01, 8.4158153674990177e-01),
                _rsw(5.0278435618743433e-01, 2.9200131956002706e-01, 8.1359886955943472e-01),
                _rsw(5.3214536744524576e-01, 3.2163225551905500e-01, 7.8318197126612255e-01),
                _rsw(5.6038391138340315e-01, 3.5064566159341992e-01, 7.5034491526787261e-01),
                _rsw(5.8741507068751464e-01, 3.7890071813062676e-01, 7.1510683154985843e-01),
                _rsw(6.1315593816600378e-01, 4.0625801705727821e-01, 6.7749112102554976e-01),
                _rsw(2.7784970163945061e-01, 2.6962712768761977e-02, 9.6024609107198011e-01),
                _rsw(3.1437335622619111e-01, 5.5234693169604360e-02, 9.4769115304804863e-01),
                _rsw(3.5014858102618274e-01, 8.4451932016264525e-02, 9.3287932895100156e-01),
                _rsw(3.8514303223036517e-01, 1.1432631193360840e-01, 9.1574796703244676e-01),
                _rsw(4.1930139794704147e-01, 1.4461778983444751e-01, 8.9625444631703577e-01),
                _rsw(4.5255859604585680e-01, 1.7511654384380909e-01, 8.7437115301066648e-01),
                _rsw(4.8484477796229469e-01, 2.0563383067456620e-01, 8.5008250715139233e-01),
                _rsw(5.1608712082768937e-01, 2.3599654872292247e-01, 8.2338308988383724e-01),
                _rsw(5.4621121856969257e-01, 2.6604302231391458e-01, 7.9427603198553687e-01),
                _rsw(5.7514250681017576e-01, 2.9561936644980313e-01, 7.6277145137971969e-01),
                _rsw(6.0280738728535965e-01, 3.2457639053127807e-01, 7.2888505303915085e-01),
                _rsw(6.2913382752784097e-01, 3.5276700262069721e-01, 6.9263703981391134e-01),
                _rsw(3.5417975284393893e-01, 2.8238534794354994e-02, 9.3475092288163819e-01),
                _rsw(3.9082349720746562e-01, 5.7412963747130938e-02, 9.1867336176918546e-01),
                _rsw(4.2644084501075913e-01, 8.7246466336501960e-02, 9.0029787282782325e-01),
                _rsw(4.6099496665532858e-01, 1.1750344229156161e-01, 8.7958887087552795e-01),
                _rsw(4.9443894965360058e-01, 1.4797556526284275e-01, 8.5652399683288871e-01),
                _rsw(5.2671948843460858e-01, 1.7847406594843526e-01, 8.3109180496923307e-01),
                _rsw(5.5777878102209899e-01, 2.0882457004312449e-01, 8.0329019064582485e-01),
                _rsw(5.8755637635366698e-01, 2.3886281365707610e-01, 7.7312486757183851e-01),
                _rsw(6.1599100163912690e-01, 2.6843089287691846e-01, 7.4060781905737771e-01),
                _rsw(6.4302196029562664e-01, 2.9737407619602518e-01, 7.0575591912794466e-01),
                _rsw(4.3006470362136456e-01, 2.9163999204940280e-02, 9.0232688746897161e-01),
                _rsw(4.6614863089355307e-01, 5.8988030247556562e-02, 8.8273771087655439e-01),
                _rsw(5.0096585552872619e-01, 8.9241626985254105e-02, 8.6085372950774641e-01),
                _rsw(5.3448242704477056e-01, 1.1971851996373199e-01, 8.3665764274165633e-01),
                _rsw(5.6665759974163710e-01, 1.5023007561613827e-01, 8.1014201781873774e-01),
                _rsw(5.9744574714047505e-01, 1.8060041919135653e-01, 7.8130792125234838e-01),
                _rsw(6.2679844441168853e-01, 2.1066217647862520e-01, 7.5016342051874008e-01),
                _rsw(6.5466647135754164e-01, 2.4025269326719154e-01, 7.1672202049199374e-01),
                _rsw(5.0427110044372525e-01, 2.9825292036076000e-02, 8.6303019020903149e-01),
                _rsw(5.3921274567743804e-01, 6.0087280623398950e-02, 8.4002329349029570e-01),
                _rsw(5.7268194376686177e-01, 9.0582276745714368e-02, 8.1475778144364586e-01),
                _rsw(6.0464692542072784e-01, 1.2112192358033980e-01, 7.8723031903468954e-01),
                _rsw(6.3507161574349524e-01, 1.5152864047915815e-01, 7.5744512275905795e-01),
                _rsw(6.6391776791854540e-01, 1.8163146812555495e-01, 7.2541243939472944e-01),
                _rsw(5.7572760409722523e-01, 3.0269917525756102e-02, 8.1708105961003197e-01),
                _rsw(6.0902658231397555e-01, 6.0784022978707185e-02, 7.9081725106719969e-01),
                _rsw(6.4067353443876607e-01, 9.1354599841766734e-02, 7.6235933742395390e-01),
                _rsw(6.7063979277937091e-01, 1.2180241559665891e-01, 7.3171472576111329e-01),
                _rsw(6.4350196744266641e-01, 3.0526083576607865e-02, 7.6483552226533735e-01),
                _rsw(6.7472186763756803e-01, 6.1121857739830476e-02, 7.3553689223457985e-01),
                ])
            self.degree = 125
        else:
            assert index == 32
            self.weights = numpy.concatenate([
                9.7353479460000007e-06 * numpy.ones(6),
                1.9075812417999999e-04 * numpy.ones(12),
                1.9010595467399999e-04 * numpy.ones(8),
                5.9929978441999997e-05 * numpy.ones(24),
                9.7490593825000000e-05 * numpy.ones(24),
                1.2416808046000001e-04 * numpy.ones(24),
                1.4376261543000001e-04 * numpy.ones(24),
                1.5842000547900000e-04 * numpy.ones(24),
                1.6944365509800001e-04 * numpy.ones(24),
                1.7766170140200000e-04 * numpy.ones(24),
                1.8361324344399999e-04 * numpy.ones(24),
                1.8764947270799999e-04 * numpy.ones(24),
                1.8999065353399999e-04 * numpy.ones(24),
                3.9264245389000002e-05 * numpy.ones(24),
                6.6679054673000004e-05 * numpy.ones(24),
                8.8688913149999993e-05 * numpy.ones(24),
                1.0663060009600001e-04 * numpy.ones(24),
                1.2145067433400000e-04 * numpy.ones(24),
                1.3380546816400000e-04 * numpy.ones(24),
                1.4416770236300001e-04 * numpy.ones(24),
                1.5288802008299999e-04 * numpy.ones(24),
                1.6023306237699999e-04 * numpy.ones(24),
                1.6641026534499999e-04 * numpy.ones(24),
                1.7158458540099999e-04 * numpy.ones(24),
                1.7589010001300000e-04 * numpy.ones(24),
                1.7943824852599999e-04 * numpy.ones(24),
                1.8232381067600001e-04 * numpy.ones(24),
                1.8462932529600000e-04 * numpy.ones(24),
                1.8642840793200001e-04 * numpy.ones(24),
                1.8778826946299999e-04 * numpy.ones(24),
                1.8877163218500000e-04 * numpy.ones(24),
                1.8943816381799999e-04 * numpy.ones(24),
                1.8984548995300000e-04 * numpy.ones(24),
                1.9004979295800001e-04 * numpy.ones(24),
                1.9006715019200001e-04 * numpy.ones(24),
                1.8998375555300001e-04 * numpy.ones(24),
                1.8990141131600000e-04 * numpy.ones(24),
                1.8985812577100000e-04 * numpy.ones(24),
                1.8988047561000000e-04 * numpy.ones(24),
                1.8997936104300000e-04 * numpy.ones(24),
                1.9014645548400000e-04 * numpy.ones(24),
                1.9035332462600000e-04 * numpy.ones(24),
                1.9055561584600000e-04 * numpy.ones(24),
                1.9070371556600000e-04 * numpy.ones(24),
                8.1432528208000006e-05 * numpy.ones(48),
                9.9988598908999995e-05 * numpy.ones(48),
                1.1561994030699999e-04 * numpy.ones(48),
                1.2876320926400001e-04 * numpy.ones(48),
                1.3983786433700000e-04 * numpy.ones(48),
                1.4918764684200000e-04 * numpy.ones(48),
                1.5708556791800001e-04 * numpy.ones(48),
                1.6374839481000001e-04 * numpy.ones(48),
                1.6935005666300000e-04 * numpy.ones(48),
                1.7403227693899999e-04 * numpy.ones(48),
                1.7791266372800001e-04 * numpy.ones(48),
                1.8109081088399999e-04 * numpy.ones(48),
                1.8365291326000001e-04 * numpy.ones(48),
                1.8567528417800001e-04 * numpy.ones(48),
                1.8722705666099999e-04 * numpy.ones(48),
                1.8837226455900000e-04 * numpy.ones(48),
                1.8917143245300000e-04 * numpy.ones(48),
                1.8968274804500001e-04 * numpy.ones(48),
                1.8996284170599999e-04 * numpy.ones(48),
                1.1233018290000000e-04 * numpy.ones(48),
                1.2536988267100000e-04 * numpy.ones(48),
                1.3662661176799999e-04 * numpy.ones(48),
                1.4627368561100001e-04 * numpy.ones(48),
                1.5450764666899999e-04 * numpy.ones(48),
                1.6150962808100000e-04 * numpy.ones(48),
                1.6743666397400000e-04 * numpy.ones(48),
                1.7242250024399999e-04 * numpy.ones(48),
                1.7658108229900001e-04 * numpy.ones(48),
                1.8001041260100000e-04 * numpy.ones(48),
                1.8279604373300001e-04 * numpy.ones(48),
                1.8501403007200001e-04 * numpy.ones(48),
                1.8673335073900001e-04 * numpy.ones(48),
                1.8801786886399999e-04 * numpy.ones(48),
                1.8892789256499999e-04 * numpy.ones(48),
                1.8952138325100000e-04 * numpy.ones(48),
                1.8985482774000001e-04 * numpy.ones(48),
                1.3491059359400001e-04 * numpy.ones(48),
                1.4440600683699999e-04 * numpy.ones(48),
                1.5267973909300001e-04 * numpy.ones(48),
                1.5982087714099999e-04 * numpy.ones(48),
                1.6593543686200000e-04 * numpy.ones(48),
                1.7112799109499999e-04 * numpy.ones(48),
                1.7549527256000000e-04 * numpy.ones(48),
                1.7912478507999999e-04 * numpy.ones(48),
                1.8209543008800000e-04 * numpy.ones(48),
                1.8447885245500000e-04 * numpy.ones(48),
                1.8634094817100000e-04 * numpy.ones(48),
                1.8774330087999999e-04 * numpy.ones(48),
                1.8874445437099999e-04 * numpy.ones(48),
                1.8940098293800001e-04 * numpy.ones(48),
                1.8976833450400001e-04 * numpy.ones(48),
                1.5173270374700001e-04 * numpy.ones(48),
                1.5877405574800000e-04 * numpy.ones(48),
                1.6490933822700001e-04 * numpy.ones(48),
                1.7019152161900001e-04 * numpy.ones(48),
                1.7468477531400000e-04 * numpy.ones(48),
                1.7845555120099999e-04 * numpy.ones(48),
                1.8156875621100000e-04 * numpy.ones(48),
                1.8408643706600001e-04 * numpy.ones(48),
                1.8606767853899999e-04 * numpy.ones(48),
                1.8756905837400000e-04 * numpy.ones(48),
                1.8864532363499999e-04 * numpy.ones(48),
                1.8935011233299999e-04 * numpy.ones(48),
                1.8973661845200001e-04 * numpy.ones(48),
                1.6439088151500001e-04 * numpy.ones(48),
                1.6963003509100000e-04 * numpy.ones(48),
                1.7415531038400000e-04 * numpy.ones(48),
                1.7800152823899999e-04 * numpy.ones(48),
                1.8121167870800001e-04 * numpy.ones(48),
                1.8383231580900000e-04 * numpy.ones(48),
                1.8591131198400000e-04 * numpy.ones(48),
                1.8749692202200000e-04 * numpy.ones(48),
                1.8863756126800000e-04 * numpy.ones(48),
                1.8938195758100000e-04 * numpy.ones(48),
                1.8977947482600000e-04 * numpy.ones(48),
                1.7389639265799999e-04 * numpy.ones(48),
                1.7774423598700001e-04 * numpy.ones(48),
                1.8100108150700001e-04 * numpy.ones(48),
                1.8369203182499999e-04 * numpy.ones(48),
                1.8584894732100001e-04 * numpy.ones(48),
                1.8750793425000001e-04 * numpy.ones(48),
                1.8870802391000001e-04 * numpy.ones(48),
                1.8949057521800000e-04 * numpy.ones(48),
                1.8989910611999999e-04 * numpy.ones(48),
                1.8090650164599999e-04 * numpy.ones(48),
                1.8362971216000001e-04 * numpy.ones(48),
                1.8584269162399999e-04 * numpy.ones(48),
                1.8756541011300001e-04 * numpy.ones(48),
                1.8882407518300000e-04 * numpy.ones(48),
                1.8964973838699999e-04 * numpy.ones(48),
                1.9007755302200000e-04 * numpy.ones(48),
                1.8585250414800001e-04 * numpy.ones(48),
                1.8762486900800000e-04 * numpy.ones(48),
                1.8894044390600000e-04 * numpy.ones(48),
                1.8981685392700001e-04 * numpy.ones(48),
                1.9027799406599999e-04 * numpy.ones(48),
                1.8901256417300000e-04 * numpy.ones(48),
                1.8994346378000001e-04 * numpy.ones(48),
                1.9045208568300000e-04 * numpy.ones(48),
                1.9055344987300001e-04 * numpy.ones(48),
                ])
            self.points = numpy.concatenate([
                _a1(),
                _a2(),
                _a3(),
                _pq0(3.5524703124725880e-02, 9.9936879852629990e-01),
                _pq0(9.1511766208412962e-02, 9.9580399509412332e-01),
                _pq0(1.5661979300689804e-01, 9.8765896970486544e-01),
                _pq0(2.2654675992719081e-01, 9.7400029033183133e-01),
                _pq0(2.9882423185813611e-01, 9.5430816744613212e-01),
                _pq0(3.7174824197038864e-01, 9.2833358475923156e-01),
                _pq0(4.4400944917588919e-01, 8.9602210298771279e-01),
                _pq0(5.1453370967566425e-01, 8.5747015202128130e-01),
                _pq0(5.8240536728602299e-01, 8.1289851036672023e-01),
                _pq0(6.4682839610433684e-01, 7.6263557876163313e-01),
                _llm(1.1823616624004491e-02, 9.9986019231683443e-01),
                _llm(3.0621450091390490e-02, 9.9906188676608065e-01),
                _llm(5.3297940368342289e-02, 9.9715528334607206e-01),
                _llm(7.8481655328622141e-02, 9.9382154311212167e-01),
                _llm(1.0540381576362022e-01, 9.8882762463684115e-01),
                _llm(1.3355777977662128e-01, 9.8200032531678882e-01),
                _llm(1.6257699555022517e-01, 9.7320986484710692e-01),
                _llm(1.9217871934127925e-01, 9.6235891416076758e-01),
                _llm(2.2213405346905488e-01, 9.4937501788219292e-01),
                _llm(2.5225049127911325e-01, 9.3420521262669698e-01),
                _llm(2.8236108606796967e-01, 9.1681210405896874e-01),
                _llm(3.1231739662675612e-01, 8.9717093551260962e-01),
                _llm(3.4198470369537887e-01, 8.7526734480201407e-01),
                _llm(3.7123864569997589e-01, 8.5109560912843141e-01),
                _llm(3.9996276498768274e-01, 8.2465724591906375e-01),
                _llm(4.2804664586480939e-01, 7.9595988462219180e-01),
                _llm(4.5538443601857120e-01, 7.6501635986696126e-01),
                _llm(4.8187360944378343e-01, 7.3184400594883636e-01),
                _llm(5.0741387092606283e-01, 6.9646416073166140e-01),
                _llm(5.3190613045707080e-01, 6.5890191740832826e-01),
                _llm(5.5525149786772854e-01, 6.1918619835336075e-01),
                _llm(5.9810090252461834e-01, 5.3343286437795889e-01),
                _llm(6.1739901922281160e-01, 4.8748015562217561e-01),
                _llm(6.3513652394111297e-01, 4.3954885042734421e-01),
                _llm(6.5120102282272008e-01, 3.8971073342838103e-01),
                _llm(6.6547583639481200e-01, 3.3805890366806024e-01),
                _llm(6.7784104148533686e-01, 2.8471572656976174e-01),
                _llm(6.8817608874841107e-01, 2.2984199300797589e-01),
                _llm(6.9636452670945970e-01, 1.7364588069234491e-01),
                _llm(7.0230106171535778e-01, 1.1638916370075923e-01),
                _llm(7.0590046366287540e-01, 5.8387248617102705e-02),
                _rsw(6.0959642591043459e-02, 1.7878282753429252e-02, 9.9798010450156804e-01),
                _rsw(8.8119622709593781e-02, 3.9538887407920921e-02, 9.9532487584509943e-01),
                _rsw(1.1659367224288320e-01, 6.3781217977229923e-02, 9.9112959386059107e-01),
                _rsw(1.4602328570317852e-01, 8.9858908137450400e-02, 9.8519164463610487e-01),
                _rsw(1.7611971101817553e-01, 1.1726065105761618e-01, 9.7735959968909003e-01),
                _rsw(2.0664711904637187e-01, 1.4561028769709949e-01, 9.6751982527832603e-01),
                _rsw(2.3740760263281518e-01, 1.7461538230117743e-01, 9.5558730552260529e-01),
                _rsw(2.6823054743370517e-01, 2.0403830702955847e-01, 9.4149919951528716e-01),
                _rsw(2.9896533121423691e-01, 2.3367886340036984e-01, 9.2521020289006373e-01),
                _rsw(3.2947627527722095e-01, 2.6336327526542191e-01, 9.0668912493253084e-01),
                _rsw(3.5963908872760847e-01, 2.9293690980516002e-01, 8.8591630120061504e-01),
                _rsw(3.8933830463988123e-01, 3.2225927852755121e-01, 8.6288159207567128e-01),
                _rsw(4.1846537893583458e-01, 3.5120047911957430e-01, 8.3758280193558765e-01),
                _rsw(4.4691723190761667e-01, 3.7963856776845373e-01, 8.1002441054992336e-01),
                _rsw(4.7459508132769757e-01, 4.0745753782638794e-01, 7.8021654920157502e-01),
                _rsw(5.0140346014102621e-01, 4.3454569060278286e-01, 7.4817418622748322e-01),
                _rsw(5.2724934045512395e-01, 4.6079425152051351e-01, 7.1391651525601130e-01),
                _rsw(5.5204130518463657e-01, 4.8609612841817207e-01, 6.7746656840533981e-01),
                _rsw(5.7568872375030777e-01, 5.1034473953427895e-01, 6.3885110955247681e-01),
                _rsw(1.2250394305883552e-01, 2.1364559226558007e-02, 9.9223804580558816e-01),
                _rsw(1.5391132173213751e-01, 4.5209261661371999e-02, 9.8704986079868329e-01),
                _rsw(1.8562130986377104e-01, 7.0864681778648075e-02, 9.8006271544267454e-01),
                _rsw(2.1749987280351291e-01, 9.7852394887729094e-02, 9.7114299366529522e-01),
                _rsw(2.4941283369383305e-01, 1.2581063962672101e-01, 9.6019004438992583e-01),
                _rsw(2.8123215621434788e-01, 1.5445291250470003e-01, 9.4712869882072737e-01),
                _rsw(3.1283722764561112e-01, 1.8354335122027537e-01, 9.3190380792324201e-01),
                _rsw(3.4411451601779719e-01, 2.1288132586195840e-01, 9.1447621126254119e-01),
                _rsw(3.7495677148535100e-01, 2.4229137348808286e-01, 8.9481970801415667e-01),
                _rsw(4.0526217320156099e-01, 2.7161637483914530e-01, 8.7291873384135188e-01),
                _rsw(4.3493354535223849e-01, 3.0071276712402806e-01, 8.4876654199841217e-01),
                _rsw(4.6387766415249654e-01, 3.2944706772164806e-01, 8.2236375301324627e-01),
                _rsw(4.9200464104626868e-01, 3.5769325436991556e-01, 7.9371718449784823e-01),
                _rsw(5.1922735548617038e-01, 3.8533070597577634e-01, 7.6283890851676395e-01),
                _rsw(5.4546090811365233e-01, 4.1224250444526939e-01, 7.2974551403110510e-01),
                _rsw(5.7062206614241406e-01, 4.3831395877810275e-01, 6.9445758054155493e-01),
                _rsw(5.9462867551815157e-01, 4.6343125363005533e-01, 6.5699939985543676e-01),
                _rsw(1.9053707909242948e-01, 2.3713115377819675e-02, 9.8139355492585312e-01),
                _rsw(2.2425187177480099e-01, 4.9178780592548058e-02, 9.7328954866726491e-01),
                _rsw(2.5771908080259370e-01, 7.5954989604951464e-02, 9.6322983495341230e-01),
                _rsw(2.9087245349271867e-01, 1.0369910831910997e-01, 9.5112549683674641e-01),
                _rsw(3.2363540200562180e-01, 1.3213485844502335e-01, 9.3691008413421040e-01),
                _rsw(3.5592673593045443e-01, 1.6103165713147899e-01, 9.2053515090483229e-01),
                _rsw(3.8766371236769553e-01, 1.9019120803957074e-01, 9.0196682339083034e-01),
                _rsw(4.1876367052188418e-01, 2.1943849501379500e-01, 8.8118314507094353e-01),
                _rsw(4.4914490198831059e-01, 2.4861553347638571e-01, 8.5817199530872768e-01),
                _rsw(4.7872709324254453e-01, 2.7757689318123346e-01, 8.3292943192529711e-01),
                _rsw(5.0743151530555763e-01, 3.0618637865911197e-01, 8.0545835323641946e-01),
                _rsw(5.3518105077383349e-01, 3.3431447181525564e-01, 7.7576741155291007e-01),
                _rsw(5.6190010259753809e-01, 3.6183627290284265e-01, 7.4387014075889335e-01),
                _rsw(5.8751440352680462e-01, 3.8862975836204083e-01, 7.0978428875539701e-01),
                _rsw(6.1195073087344953e-01, 4.1457422777920316e-01, 6.7353137465505519e-01),
                _rsw(2.6197338701194600e-01, 2.5400471863893818e-02, 9.6474077374524847e-01),
                _rsw(2.9681497432379478e-01, 5.2081070185439907e-02, 9.5351372991976591e-01),
                _rsw(3.3104515048604888e-01, 7.9718284708856016e-02, 9.4024151334789885e-01),
                _rsw(3.6462155673766766e-01, 1.0804659991779283e-01, 9.2486596467185678e-01),
                _rsw(3.9749167852793604e-01, 1.3684138493666295e-01, 9.0734491835776543e-01),
                _rsw(4.2959674037720297e-01, 1.6590731847635598e-01, 8.8764936902656955e-01),
                _rsw(4.6087428544734471e-01, 1.9507037304546138e-01, 8.6576119257755135e-01),
                _rsw(4.9125988589499042e-01, 2.2417211443767227e-01, 8.4167130616350716e-01),
                _rsw(5.2068827589455580e-01, 2.5306552554064898e-01, 8.1537816939674690e-01),
                _rsw(5.4909409140198195e-01, 2.8161184097310654e-01, 7.8688655460057877e-01),
                _rsw(5.7641233020255433e-01, 3.0967805045932373e-01, 7.5620653967958651e-01),
                _rsw(6.0257860042135047e-01, 3.3713483663949878e-01, 7.2335270251676320e-01),
                _rsw(6.2752919647949557e-01, 3.6385478276943944e-01, 6.8834352224859541e-01),
                _rsw(3.3481894798617690e-01, 2.6648419355374525e-02, 9.4190558646569766e-01),
                _rsw(3.6995155458552942e-01, 5.4240000668435061e-02, 9.2746637113549202e-01),
                _rsw(4.0420030714746680e-01, 8.2519927154308656e-02, 9.1094048835494246e-01),
                _rsw(4.3753201001826236e-01, 1.1126951824837100e-01, 8.9229189983892299e-01),
                _rsw(4.6990544903359471e-01, 1.4029641164678153e-01, 8.7149629135617812e-01),
                _rsw(5.0127398794319522e-01, 1.6942751175842882e-01, 8.4853916071733115e-01),
                _rsw(5.3158748837549663e-01, 1.9850382353126886e-01, 8.2341421790378266e-01),
                _rsw(5.6079371096221176e-01, 2.2737656600208930e-01, 7.9612204527844155e-01),
                _rsw(5.8883932234955216e-01, 2.5590414928497635e-01, 7.6666897604745476e-01),
                _rsw(6.1567059791601619e-01, 2.8394972519768985e-01, 7.3506616601629238e-01),
                _rsw(6.4123388090781230e-01, 3.1137910605006891e-01, 7.0132885459773109e-01),
                _rsw(4.0760512592571652e-01, 2.7577922908584410e-02, 9.1274175947369085e-01),
                _rsw(4.4237881257915207e-01, 5.5841368349842782e-02, 8.9508811173083769e-01),
                _rsw(4.7604809173282581e-01, 8.4577720877271279e-02, 8.7534268917306979e-01),
                _rsw(5.0858387259462956e-01, 1.1359758463592511e-01, 8.5348581318117611e-01),
                _rsw(5.3995136373912178e-01, 1.4272869047650524e-01, 8.2950650733500852e-01),
                _rsw(5.7011184336363796e-01, 1.7181127400576329e-01, 8.0340112781911832e-01),
                _rsw(5.9902405306060225e-01, 2.0069448559853506e-01, 7.7517217913518299e-01),
                _rsw(6.2664526851396951e-01, 2.2923350905989082e-01, 7.4482729929369806e-01),
                _rsw(6.5293209714159417e-01, 2.5728715123537144e-01, 7.1237840950682030e-01),
                _rsw(4.7915838346101264e-01, 2.8260941977359226e-02, 8.7727336829381841e-01),
                _rsw(5.1303739527969405e-01, 5.6998713596836330e-02, 8.5647170279754881e-01),
                _rsw(5.4562524296284765e-01, 8.6027125285544070e-02, 8.3360208010587333e-01),
                _rsw(5.7689563296823865e-01, 1.1517481372212791e-01, 8.0865702924431959e-01),
                _rsw(6.0681869446990444e-01, 1.4428116541363642e-01, 7.8163547600446315e-01),
                _rsw(6.3536222480249072e-01, 1.7319303216576803e-01, 7.5254170442790513e-01),
                _rsw(6.6249270357317969e-01, 2.0176199587560606e-01, 7.2138444309022287e-01),
                _rsw(5.4849335080284889e-01, 2.8742197559073444e-02, 8.3566077459968058e-01),
                _rsw(5.8102076821421056e-01, 5.7783121237136748e-02, 8.1183494492653052e-01),
                _rsw(6.1209551971813536e-01, 8.6952623714395050e-02, 7.8598875053665285e-01),
                _rsw(6.4169442842943203e-01, 1.1608937670571640e-01, 7.5812368195348101e-01),
                _rsw(6.6979263917312604e-01, 1.4503788267432510e-01, 7.2824572302132151e-01),
                _rsw(6.1475943905854891e-01, 2.9049576223413566e-02, 7.8817951902447858e-01),
                _rsw(6.4553900263567821e-01, 5.8238091526172896e-02, 7.6150359209364404e-01),
                _rsw(6.7472585883654757e-01, 8.7403848998846748e-02, 7.3287487513044824e-01),
                _rsw(6.7721357503953472e-01, 2.9199461358081776e-02, 7.3520688601139361e-01),
                ])
            self.degree = 131

        return

def _a1():
    return numpy.array([
        [+1.0, 0.0, 0.0],
        [0.0, +1.0, 0.0],
        [0.0, 0.0, +1.0],
        [-1.0, 0.0, 0.0],
        [0.0, -1.0, 0.0],
        [0.0, 0.0, -1.0],
        ])

def _a2():
    return numpy.array([
        [+1.0, +1.0, 0.0],
        [+1.0, 0.0, +1.0],
        [0.0, +1.0, +1.0],
        [-1.0, +1.0, 0.0],
        [-1.0, 0.0, +1.0],
        [0.0, -1.0, +1.0],
        [+1.0, -1.0, 0.0],
        [+1.0, 0.0, -1.0],
        [0.0, +1.0, -1.0],
        [-1.0, -1.0, 0.0],
        [-1.0, 0.0, -1.0],
        [0.0, -1.0, -1.0],
        ]) / numpy.sqrt(2.0)

def _a3():
    return numpy.array([
        [+1.0, +1.0, +1.0],
        [+1.0, +1.0, -1.0],
        [+1.0, -1.0, +1.0],
        [+1.0, -1.0, -1.0],
        [-1.0, +1.0, +1.0],
        [-1.0, +1.0, -1.0],
        [-1.0, -1.0, +1.0],
        [-1.0, -1.0, -1.0],
        ]) / numpy.sqrt(3.0)

def _pq0(p, q):
    assert abs(p**2 + q**2 - 1.0) < 1.0e-12
    return numpy.array([
        [+p, +q, 0.0],
        [+p, 0.0, +q],
        [0.0, +p, +q],
        [+q, +p, 0.0],
        [+q, 0.0, +p],
        [0.0, +q, +p],
        [-p, +q, 0.0],
        [-p, 0.0, +q],
        [0.0, -p, +q],
        [+q, -p, 0.0],
        [+q, 0.0, -p],
        [0.0, +q, -p],
        [+p, -q, 0.0],
        [+p, 0.0, -q],
        [0.0, +p, -q],
        [-q, +p, 0.0],
        [-q, 0.0, +p],
        [0.0, -q, +p],
        [-p, -q, 0.0],
        [-p, 0.0, -q],
        [0.0, -p, -q],
        [-q, -p, 0.0],
        [-q, 0.0, -p],
        [0.0, -q, -p],
        ])

def _llm(l, m):
    assert abs(2*l**2 + m**2 - 1.0) < 1.0e-12
    return numpy.array([
        [+l, +l, +m],
        [+l, +m, +l],
        [+m, +l, +l],
        #
        [-l, +l, +m],
        [-l, +m, +l],
        [+m, -l, +l],
        #
        [+l, -l, +m],
        [+l, +m, -l],
        [+m, +l, -l],
        #
        [+l, +l, -m],
        [+l, -m, +l],
        [-m, +l, +l],
        #
        [-l, -l, +m],
        [-l, +m, -l],
        [+m, -l, -l],
        #
        [-l, +l, -m],
        [-l, -m, +l],
        [-m, -l, +l],
        #
        [+l, -l, -m],
        [+l, -m, -l],
        [-m, +l, -l],
        #
        [-l, -l, -m],
        [-l, -m, -l],
        [-m, -l, -l],
        ])

def _rsw(r, s, w):
    assert abs(r**2 + s**2 + w**2 - 1.0) < 1.0e-12
    return numpy.array([
        [+r, +s, +w],
        [+w, +r, +s],
        [+s, +w, +r],
        [+s, +r, +w],
        [+w, +s, +r],
        [+r, +w, +s],
        #
        [-r, +s, +w],
        [+w, -r, +s],
        [+s, +w, -r],
        [+s, -r, +w],
        [+w, +s, -r],
        [-r, +w, +s],
        #
        [+r, -s, +w],
        [+w, +r, -s],
        [-s, +w, +r],
        [-s, +r, +w],
        [+w, -s, +r],
        [+r, +w, -s],
        #
        [+r, +s, -w],
        [-w, +r, +s],
        [+s, -w, +r],
        [+s, +r, -w],
        [-w, +s, +r],
        [+r, -w, +s],
        #
        [-r, -s, +w],
        [+w, -r, -s],
        [-s, +w, -r],
        [-s, -r, +w],
        [+w, -s, -r],
        [-r, +w, -s],
        #
        [-r, +s, -w],
        [-w, -r, +s],
        [+s, -w, -r],
        [+s, -r, -w],
        [-w, +s, -r],
        [-r, -w, +s],
        #
        [+r, -s, -w],
        [-w, +r, -s],
        [-s, -w, +r],
        [-s, +r, -w],
        [-w, -s, +r],
        [+r, -w, -s],
        #
        [-r, -s, -w],
        [-w, -r, -s],
        [-s, -w, -r],
        [-s, -r, -w],
        [-w, -s, -r],
        [-r, -w, -s],
        ])
